@@ -1,6 +1,4 @@
 /*
- * $Id: afp_asp.c,v 1.29 2010-03-09 06:55:12 franklahm Exp $
- *
  * Copyright (c) 1997 Adrian Sun (asun@zoology.washington.edu)
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * All Rights Reserved.  See COPYRIGHT.
@@ -37,11 +35,7 @@
 #include "switch.h"
 #include "auth.h"
 #include "fork.h"
-
-#ifdef FORCE_UIDGID
-#warning UIDGID
-#include "uid.h"
-#endif /* FORCE_UIDGID */
+#include "dircache.h"
 
 static AFPObj *child;
 
@@ -51,8 +45,12 @@ static void afp_asp_close(AFPObj *obj)
 {
     ASP asp = obj->handle;
 
-    if (seteuid( obj->uid ) < 0) {
-        LOG(log_error, logtype_afpd, "can't seteuid back %s", strerror(errno));
+    if ( obj->uid != geteuid() ) {
+        if (seteuid(obj->uid) < 0) {
+            LOG(log_error, logtype_afpd,
+			"afp_asp_close: can't seteuid back to %i from %i (%s)",
+			obj->uid, geteuid(), strerror(errno));
+        }
         exit(EXITERR_SYS);
     }
     close_all_vol();
@@ -81,7 +79,7 @@ static void afp_authprint_remove(AFPObj *obj)
 
     memset( addr_filename_buff, 0, 256 );
 
-    if(lstat(addr_filename, &cap_st) == 0) {
+    if (stat(addr_filename, &cap_st) == 0) {
 	if( S_ISREG(cap_st.st_mode) ) {
 	    int len;
 	    int capfd = open( addr_filename, O_RDONLY );
@@ -212,9 +210,7 @@ void afp_over_asp(AFPObj *obj)
     ASP asp;
     struct sigaction  action;
     int		func,  reply = 0;
-#ifdef DEBUG1
     int ccnt = 0;
-#endif    
 
     AFPobj = obj;
     obj->exit = afp_asp_die;
@@ -306,31 +302,13 @@ void afp_over_asp(AFPObj *obj)
             afp_asp_close(obj);
             LOG(log_info, logtype_afpd, "done" );
 
-#ifdef DEBUG1
-            if ( obj->options.flags & OPTION_DEBUG ) {
-                printf( "done\n" );
-            }
-#endif
             return;
             break;
 
         case ASPFUNC_CMD :
-#ifdef AFS
-            if ( writtenfork ) {
-                if ( flushfork( writtenfork ) < 0 ) {
-                    LOG(log_error, logtype_afpd, "main flushfork: %s",
-						strerror(errno));
-                }
-                writtenfork = NULL;
-            }
-#endif /* AFS */
             func = (u_char) asp->commands[0];
-#ifdef DEBUG1
-            if ( obj->options.flags & OPTION_DEBUG ) {
-                printf("command: %d (%s)\n", func, AfpNum2name(func));
-                bprint( asp->commands, asp->cmdlen );
-            }
-#endif            
+            LOG(log_debug9, logtype_afpd, "command: %d (%s)\n", func, AfpNum2name(func));
+
             if ( afp_switch[ func ] != NULL ) {
                 /*
                  * The function called from afp_switch is expected to
@@ -342,22 +320,14 @@ void afp_over_asp(AFPObj *obj)
                 reply = (*afp_switch[ func ])(obj,
                                               asp->commands, asp->cmdlen,
                                               asp->data, &asp->datalen);
-#ifdef FORCE_UIDGID
-            	/* bring everything back to old euid, egid */
-		if (obj->force_uid)
-            	    restore_uidgid ( &obj->uidgid );
-#endif /* FORCE_UIDGID */
             } else {
                 LOG(log_error, logtype_afpd, "bad function %X", func );
                 asp->datalen = 0;
                 reply = AFPERR_NOOP;
             }
-#ifdef DEBUG1
-            if ( obj->options.flags & OPTION_DEBUG ) {
-                printf( "reply: %d, %d\n", reply, ccnt++ );
-                bprint( asp->data, asp->datalen );
-            }
-#endif
+
+            LOG(log_debug9, logtype_afpd, "reply: %d, %d\n", reply, ccnt++ );
+
             if ( asp_cmdreply( asp, reply ) < 0 ) {
                 LOG(log_error, logtype_afpd, "asp_cmdreply: %s", strerror(errno) );
                 afp_asp_die(EXITERR_CLNT);
@@ -366,33 +336,22 @@ void afp_over_asp(AFPObj *obj)
 
         case ASPFUNC_WRITE :
             func = (u_char) asp->commands[0];
-#ifdef DEBUG1
-            if ( obj->options.flags & OPTION_DEBUG ) {
-                printf( "(write) command: %d\n", func );
-                bprint( asp->commands, asp->cmdlen );
-            }
-#endif
+
+            LOG(log_debug9, logtype_afpd, "(write) command: %d\n", func );
+
             if ( afp_switch[ func ] != NULL ) {
                 asp->datalen = ASP_DATASIZ;
                 reply = (*afp_switch[ func ])(obj,
                                               asp->commands, asp->cmdlen,
                                               asp->data, &asp->datalen);
-#ifdef FORCE_UIDGID
-            	/* bring everything back to old euid, egid */
-		if (obj->force_uid)
-            	    restore_uidgid ( &obj->uidgid );
-#endif /* FORCE_UIDGID */
             } else {
                 LOG(log_error, logtype_afpd, "(write) bad function %X", func );
                 asp->datalen = 0;
                 reply = AFPERR_NOOP;
             }
-#ifdef DEBUG1
-            if ( obj->options.flags & OPTION_DEBUG ) {
-                printf( "(write) reply code: %d, %d\n", reply, ccnt++ );
-                bprint( asp->data, asp->datalen );
-            }
-#endif
+
+            LOG(log_debug9, logtype_afpd, "(write) reply code: %d, %d\n", reply, ccnt++ );
+
             if ( asp_wrtreply( asp, reply ) < 0 ) {
                 LOG(log_error, logtype_afpd, "asp_wrtreply: %s", strerror(errno) );
                 afp_asp_die(EXITERR_CLNT);
@@ -406,15 +365,12 @@ void afp_over_asp(AFPObj *obj)
             LOG(log_info, logtype_afpd, "main: asp_getrequest: %d", reply );
             break;
         }
-#ifdef DEBUG1
+
         if ( obj->options.flags & OPTION_DEBUG ) {
-#ifdef notdef
-            pdesc( stdout );
-#endif /* notdef */
             of_pforkdesc( stdout );
             fflush( stdout );
         }
-#endif
+
     }
 }
 
