@@ -108,7 +108,7 @@ static char * srvloc_encode(const struct afp_options *options, const char *name)
 		return (char*)name;
 
 	/* Escape characters */
-	p = conv_name;
+	p = (unsigned char *)conv_name;
 	while (*p && i<(sizeof(buf)-4)) {
 	    if (*p == '@')
 		break;
@@ -142,11 +142,9 @@ static char * srvloc_encode(const struct afp_options *options, const char *name)
 
 	return buf;
 }
-#endif /* USE_SRVLOC */
 
 static void dsi_cleanup(const AFPConfig *config)
 {
-#ifdef USE_SRVLOC
     SLPError err;
     SLPError callbackerr;
     SLPHandle hslp;
@@ -179,8 +177,8 @@ static void dsi_cleanup(const AFPConfig *config)
 srvloc_dereg_err:
     dsi->srvloc_url[0] = '\0';
     SLPClose(hslp);
-#endif /* USE_SRVLOC */
 }
+#endif /* USE_SRVLOC */
 
 #ifndef NO_DDP
 static void asp_cleanup(const AFPConfig *config)
@@ -315,8 +313,8 @@ static AFPConfig *ASPConfigInit(const struct afp_options *options,
     config->optcount = refcount;
     (*refcount)++;
 
-    config->server_start = asp_start;
-    config->server_cleanup = asp_cleanup;
+    config->server_start = (void *) asp_start;
+    config->server_cleanup = (void *) asp_cleanup;
 
     return config;
 
