@@ -158,10 +158,10 @@ void afp_options_init(struct afp_options *options)
     options->sigconffile = _PATH_AFPDSIGCONF;
     options->uuidconf = _PATH_AFPDUUIDCONF;
     options->uampath = _PATH_AFPDUAMPATH;
-    options->uamlist = "uams_dhx.so,uams_dhx2.so";
+    options->uamlist = "uams_dhx2.so";
     options->guest = "nobody";
     options->loginmesg = "";
-    options->transports = AFPTRANS_TCP; /*  TCP only */
+    options->transports = AFPTRANS_ALL; /*  TCP and DDP */
     options->passwdfile = _PATH_AFPDPWFILE;
     options->tickleval = 30;
     options->timeout = 4;       /* 4 tickles = 2 minutes */
@@ -216,8 +216,6 @@ int afp_options_parseline(char *buf, struct afp_options *options)
         options->server = opt;
 
     /* parse toggles */
-    if (strstr(buf, " -nodebug"))
-        options->flags &= ~OPTION_DEBUG;
 #ifdef USE_SRVLOC
     if (strstr(buf, " -slp"))
         options->flags &= ~OPTION_NOSLP;
@@ -318,7 +316,7 @@ int afp_options_parseline(char *buf, struct afp_options *options)
         options->loginmaxfail = atoi(c);
     if ((c = getoption(buf, "-tickleval"))) {
         options->tickleval = atoi(c);
-        if (options->tickleval <= 0) {
+        if (options->tickleval < 0) {
             options->tickleval = 30;
         }
     }
@@ -404,17 +402,6 @@ int afp_options_parseline(char *buf, struct afp_options *options)
         options->uamlist = opt;
 
     if ((c = getoption(buf, "-ipaddr"))) {
-#if 0
-        struct in_addr inaddr;
-        if (inet_aton(c, &inaddr) && (opt = strdup(c))) {
-            if (!gethostbyaddr((const char *) &inaddr, sizeof(inaddr), AF_INET))
-                LOG(log_info, logtype_afpd, "WARNING: can't find %s", opt);
-            options->ipaddr = opt;
-        }
-        else {
-            LOG(log_error, logtype_afpd, "Error parsing -ipaddr, is %s in numbers-and-dots notation?", c);
-        }
-#endif
         options->ipaddr = strdup(c);
     }
 
@@ -552,30 +539,11 @@ static void show_version( void )
 #endif
 
 	printf( "         CNID backends:\t" );
-#ifdef CNID_BACKEND_CDB
-	printf( "cdb ");
-#endif
-#ifdef CNID_BACKEND_DB3
-	printf( "db3 " );
-#endif
 #ifdef CNID_BACKEND_DBD
-#ifdef CNID_BACKEND_DBD_TXN
-	printf( "dbd-txn " );
-#else
 	printf( "dbd " );
 #endif
-#endif
-#ifdef CNID_BACKEND_HASH
-	printf( "hash " );
-#endif
 #ifdef CNID_BACKEND_LAST
 	printf( "last " );
-#endif
-#ifdef CNID_BACKEND_MTAB
-	printf( "mtab " );
-#endif
-#ifdef CNID_BACKEND_TDB
-	printf( "tdb " );
 #endif
 	puts( "" );
 }
@@ -639,19 +607,6 @@ static void show_version_extended(void )
 	puts( "No" );
 #endif
 
-	printf( "        Dropbox kludge:\t" );
-#ifdef DROPKLUDGE
-	puts( "Yes" );
-#else
-	puts( "No" );
-#endif
-
-	printf( "  Force volume uid/gid:\t" );
-#ifdef FORCE_UIDGID
-	puts( "Yes" );
-#else
-	puts( "No" );
-#endif
 
 	printf( "            EA support:\t" );
 	puts( EA_MODULES );
@@ -719,15 +674,6 @@ int afp_options_parse(int ac, char **av, struct afp_options *options)
         *p = '\0';
     }
 
-#ifdef ultrix
-    if (NULL == ( p = strrchr( av[ 0 ], '/' )) ) {
-        p = av[ 0 ];
-    } else {
-        p++;
-    }
-    openlog( p, LOG_PID ); /* ultrix only */
-#endif /* ultrix */
-
     while (EOF != ( c = getopt( ac, av, OPTIONS )) ) {
         switch ( c ) {
         case 'd' :
