@@ -21,9 +21,6 @@
 #endif
 
 
-/*
- * Functions (actually they are just facades)
- */
 void zeroconf_register(const AFPConfig *configs)
 {
 #if defined (HAVE_MDNS)
@@ -36,14 +33,3 @@ void zeroconf_register(const AFPConfig *configs)
 	av_zeroconf_register(configs);
 #endif
 }
-
-void zeroconf_deregister(void)
-{
-#if defined (HAVE_MDNS)
-  LOG(log_debug, logtype_afpd, "Attempting to de-register mDNS using mDNSResponder");
-	md_zeroconf_unregister();
-#elif defined (HAVE_AVAHI)
-  LOG(log_debug, logtype_afpd, "Attempting to de-register mDNS using Avahi");
-	av_zeroconf_unregister();
-#endif
-}
