@@ -1,6 +1,4 @@
 /*
- * $Id: appl.c,v 1.18.4.1 2010-02-01 10:56:08 franklahm Exp $
- *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * All Rights Reserved.  See COPYRIGHT.
  */
@@ -95,6 +93,11 @@ static int copyapplfile(int sfd, int dfd, char *mpath, u_short mplen)
         p = buf + sizeof(appltag);
         memcpy( &len, p, sizeof(len));
         len = ntohs( len );
+        if (len > MAXPATHLEN - (sizeof(appltag) + sizeof(len))) {
+            errno = EINVAL;
+            cc = -1;
+            break;
+        }
         p += sizeof( len );
         if (( cc = read( sa.sdt_fd, p, len )) < len ) {
             break;
@@ -130,7 +133,7 @@ makemacpath(const struct vol *vol, char *mpath, int mpathlen, struct dir *dir, c
 
     p = mpath + mpathlen;
     p -= strlen( path );
-    memcpy( p, path, strlen( path )); 
+    memcpy( p, path, strlen( path ));
 
     while ( dir->d_did != DIRDID_ROOT ) {
         p -= blength(dir->d_m_name) + 1;
@@ -144,53 +147,6 @@ makemacpath(const struct vol *vol, char *mpath, int mpathlen, struct dir *dir, c
     }
     return( p );
 
-
-#if 0
-    char buffer[12 + MAXPATHLEN + 1];
-    int buflen = 12 + MAXPATHLEN + 1;
-    char *ret = mpath;
-    char *path = name;
-    char *uname = NULL;
-    struct bstrList *pathlist = NULL;
-    cnid_t cnid = dir->d_pdid;
-
-    /* Create list for path elements, request 16 list elements for now*/
-    if ((pathlist = bstListCreateMin(16)) == NULL) {
-        LOG(log_error, logtype_afpd, "makemacpath: OOM: %s", strerror(errno));
-        return NULL;
-    }
-
-    while ( cnid != DIRDID_ROOT ) {
-
-        /* construct path, copy already found uname to path element list*/
-        if ((bstrListPush(pathlist, bfromcstr(path))) != BSTR_OK) {
-            afp_errno = AFPERR_MISC;
-            ret = NULL;
-            goto exit;
-        }
-
-        /* next part */
-        if ((uname = cnid_resolve(vol->v_cdb, &cnid, buffer, buflen)) == NULL ) {
-            afp_errno = AFPERR_NOOBJ;
-            ret = NULL;
-            goto exit;
-        }
-
-        if ((path = utompath(vol, uname, cnid, utf8_encoding())) == NULL) {
-            afp_errno = AFPERR_MISC;
-            ret = NULL;
-            goto exit;
-        }
-    }
-
-
-
-exit:
-    if (pathlist)
-        bstrListDestroy(pathlist);
-
-    return(ret);
-#endif
 }
 
 
@@ -359,14 +315,20 @@ int afp_getappl(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf, size_t
 {
     struct vol		*vol;
     char		*p, *q;
-    int			cc; 
+    int			cc;
     size_t		buflen;
     u_int16_t		vid, aindex, bitmap, len;
     u_char		creator[ 4 ];
     u_char		appltag[ 4 ];
     char                *buf, *cbuf;
     struct path         *path;
-    
+#if defined(APPLCNAME)
+    char		utomname[ MAXPATHLEN + 1];
+    char		*u, *m;
+    int			i, h;
+#endif
+
+    memset(appltag, 0, sizeof(u_char) * 4);
     ibuf += 2;
 
     memcpy( &vid, ibuf, sizeof( vid ));
@@ -410,6 +372,10 @@ int afp_getappl(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf, size_t
         memcpy( &len, p, sizeof( len ));
         len = ntohs( len );
         p += sizeof( u_short );
+        if ( len > sizeof(obj->oldtmp) - (p - buf) ) {
+            *rbuflen = 0;
+            return( AFPERR_NOITEM );
+        }
         if (( cc = read( sa.sdt_fd, p, len )) < len ) {
             break;
         }
@@ -430,37 +396,31 @@ int afp_getappl(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf, size_t
      * there are any ':'s in the name, it is a upath and must be converted
      * to an mpath.  Hopefully, this code will go away.
      */
-    {
+
 #define hextoint( c )	( isdigit( c ) ? c - '0' : c + 10 - 'a' )
 #define islxdigit(x)	(!isupper(x)&&isxdigit(x))
-
-        char	utomname[ MAXPATHLEN + 1];
-        char		*u, *m;
-        int		i, h;
-
-        u = p;
-        m = utomname;
-        i = len;
-        while ( i ) {
-            if ( *u == ':' && *(u+1) != '\0' && islxdigit( *(u+1)) &&
-                    *(u+2) != '\0' && islxdigit( *(u+2))) {
-                ++u, --i;
-                h = hextoint( *u ) << 4;
-                ++u, --i;
-                h |= hextoint( *u );
-                *m++ = h;
-            } else {
-                *m++ = *u;
-            }
+    u = p;
+    m = utomname;
+    i = len;
+    while ( i ) {
+        if ( *u == ':' && *(u+1) != '\0' && islxdigit( *(u+1)) &&
+                *(u+2) != '\0' && islxdigit( *(u+2))) {
             ++u, --i;
+            h = hextoint( *u ) << 4;
+            ++u, --i;
+            h |= hextoint( *u );
+            *m++ = h;
+        } else {
+            *m++ = *u;
         }
+        ++u, --i;
+    }
 
-        len = m - utomname;
-        p = utomname;
+    len = m - utomname;
+    p = utomname;
 
-        if ( p[ len - 1 ] == '\0' ) {
-            len--;
-        }
+    if ( p[ len - 1 ] == '\0' ) {
+        len--;
     }
 #endif /* APPLCNAME */
 
@@ -496,4 +456,3 @@ int afp_getappl(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf, size_t
     rbuf += sizeof( appltag );
     return( AFP_OK );
 }
-
