@@ -29,6 +29,9 @@
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -93,7 +96,7 @@ struct scrit {
 	time_t mdate;               /* Last modification date */
 	time_t bdate;               /* Last backup date */
 	u_int32_t pdid;             /* Parent DID */
-    u_int16_t offcnt;           /* Offspring count */
+	u_int16_t offcnt;           /* Offspring count */
 	struct finderinfo finfo;    /* Finder info */
 	char lname[64];             /* Long name */ 
 	char utf8name[514];         /* UTF8 or UCS2 name */ /* for convert_charset dest_len parameter +2 */
@@ -108,12 +111,8 @@ struct scrit {
  *
  */
 struct dsitem {
-//	struct dir *dir; /* Structure describing this directory */
-//  cnid_t did;      /* CNID of this directory           */
-	int pidx;        /* Parent's dsitem structure index. */
-	int checked;     /* Have we checked this directory ? */
-	int path_len;
-	char *path;      /* absolute UNIX path to this directory */
+    cnid_t ds_did;               /* CNID of this directory           */
+    u_int32_t ds_checked;    /* Have we checked this directory ? */
 };
  
 
@@ -130,7 +129,6 @@ static void clearstack(void)
 	save_cidx = -1;
 	while (dsidx > 0) {
 		dsidx--;
-		free(dstack[dsidx].path);
 	}
 }
 
@@ -155,23 +153,8 @@ static int addstack(char *uname, struct dir *dir, int pidx)
 
 	/* Put new element. Allocate and copy lname and path. */
 	ds = dstack + dsidx++;
-//	ds->did = dir->d_did;
-	ds->pidx = pidx;
-	ds->checked = 0;
-	if (pidx >= 0) {
-	    l = dstack[pidx].path_len;
-	    u = strlen(uname) +1;
-	    if (!(ds->path = malloc(l + u + 1) ))
-			return -1;
-		memcpy(ds->path, dstack[pidx].path, l);
-		ds->path[l] = '/';
-		memcpy(&ds->path[l+1], uname, u);
-		ds->path_len = l +u;
-	}
-	else {
-	    ds->path = strdup(uname);
-		ds->path_len = strlen(uname);
-	}
+	ds->ds_did = dir->d_did;
+	ds->ds_checked = 0;
 	return 0;
 }
 
@@ -186,9 +169,9 @@ static int reducestack(void)
 	}
 
 	while (dsidx > 0) {
-		if (dstack[dsidx-1].checked) {
+		if (dstack[dsidx-1].ds_checked) {
 			dsidx--;
-			free(dstack[dsidx].path);
+//			free(dstack[dsidx].path);
 		} else
 			return dsidx - 1;
 	} 
@@ -210,7 +193,7 @@ static struct adouble *adl_lkup(struct vol *vol, struct path *path, struct adoub
 	    
 	isdir  = S_ISDIR(path->st.st_mode);
 
-	if (!isdir && (of = of_findname(path))) {
+	if (!isdir && (of = of_findname(vol, path))) {
 		adp = of->of_ad;
 	} else {
 		ad_init(&ad, vol->v_adouble, vol->v_ad_options);
@@ -309,11 +292,11 @@ static int crit_check(struct vol *vol, struct path *path) {
 	 */
 
 	/* Check for filename */
-	if ((c1.rbitmap & (1<<DIRPBIT_LNAME))) { 
+	if ((c1.rbitmap & (1U<<DIRPBIT_LNAME))) { 
 		if ( (size_t)(-1) == (len = convert_string(vol->v_maccharset, CH_UCS2, path->m_name, -1, convbuf, 512)) )
 			goto crit_check_ret;
 
-		if ((c1.rbitmap & (1<<CATPBIT_PARTIAL))) {
+		if ((c1.rbitmap & (1U<<CATPBIT_PARTIAL))) {
 			if (strcasestr_w( (ucs2_t*) convbuf, (ucs2_t*) c1.lname) == NULL)
 				goto crit_check_ret;
 		} else
@@ -321,12 +304,12 @@ static int crit_check(struct vol *vol, struct path *path) {
 				goto crit_check_ret;
 	} 
 	
-	if ((c1.rbitmap & (1<<FILPBIT_PDINFO))) { 
+	if ((c1.rbitmap & (1U<<FILPBIT_PDINFO))) { 
 		if ( (size_t)(-1) == (len = convert_charset( CH_UTF8_MAC, CH_UCS2, CH_UTF8, path->m_name, strlen(path->m_name), convbuf, 512, &flags))) {
 			goto crit_check_ret;
 		}
 
-		if (c1.rbitmap & (1<<CATPBIT_PARTIAL)) {
+		if (c1.rbitmap & (1U<<CATPBIT_PARTIAL)) {
 			if (strcasestr_w((ucs2_t *) convbuf, (ucs2_t*)c1.utf8name) == NULL)
 				goto crit_check_ret;
 		} else
@@ -344,13 +327,13 @@ static int crit_check(struct vol *vol, struct path *path) {
 		c2.bdate = 0x7fffffff;
 
 	/* Check for modification date */
-	if ((c1.rbitmap & (1<<DIRPBIT_MDATE))) {
+	if ((c1.rbitmap & (1U<<DIRPBIT_MDATE))) {
 		if (path->st.st_mtime < c1.mdate || path->st.st_mtime > c2.mdate)
 			goto crit_check_ret;
 	}
 	
 	/* Check for creation date... */
-	if ((c1.rbitmap & (1<<DIRPBIT_CDATE))) {
+	if ((c1.rbitmap & (1U<<DIRPBIT_CDATE))) {
 		c_date = path->st.st_mtime;
 		adp = adl_lkup(vol, path, adp);
 		if (adp && ad_getdate(adp, AD_DATE_CREATE, &ac_date) >= 0)
@@ -361,7 +344,7 @@ static int crit_check(struct vol *vol, struct path *path) {
 	}
 
 	/* Check for backup date... */
-	if ((c1.rbitmap & (1<<DIRPBIT_BDATE))) {
+	if ((c1.rbitmap & (1U<<DIRPBIT_BDATE))) {
 		b_date = path->st.st_mtime;
 		adp = adl_lkup(vol, path, adp);
 		if (adp && ad_getdate(adp, AD_DATE_BACKUP, &ab_date) >= 0)
@@ -372,7 +355,7 @@ static int crit_check(struct vol *vol, struct path *path) {
 	}
 				
 	/* Check attributes */
-	if ((c1.rbitmap & (1<<DIRPBIT_ATTR)) && c2.attr != 0) {
+	if ((c1.rbitmap & (1U<<DIRPBIT_ATTR)) && c2.attr != 0) {
 		if ((adp = adl_lkup(vol, path, adp))) {
 			ad_getattr(adp, &attr);
 			if ((attr & c2.attr) != c1.attr)
@@ -382,14 +365,14 @@ static int crit_check(struct vol *vol, struct path *path) {
 	}		
 
         /* Check file type ID */
-	if ((c1.rbitmap & (1<<DIRPBIT_FINFO)) && c2.finfo.f_type != 0) {
+	if ((c1.rbitmap & (1U<<DIRPBIT_FINFO)) && c2.finfo.f_type != 0) {
 	    finfo = unpack_finderinfo(vol, path, &adp, &finderinfo,islnk);
 		if (finfo->f_type != c1.finfo.f_type)
 			goto crit_check_ret;
 	}
 	
 	/* Check creator ID */
-	if ((c1.rbitmap & (1<<DIRPBIT_FINFO)) && c2.finfo.creator != 0) {
+	if ((c1.rbitmap & (1U<<DIRPBIT_FINFO)) && c2.finfo.creator != 0) {
 		if (!finfo) {
 			finfo = unpack_finderinfo(vol, path, &adp, &finderinfo,islnk);
 		}
@@ -398,7 +381,7 @@ static int crit_check(struct vol *vol, struct path *path) {
 	}
 		
 	/* Check finder info attributes */
-	if ((c1.rbitmap & (1<<DIRPBIT_FINFO)) && c2.finfo.attrs != 0) {
+	if ((c1.rbitmap & (1U<<DIRPBIT_FINFO)) && c2.finfo.attrs != 0) {
 		if (!finfo) {
 			finfo = unpack_finderinfo(vol, path, &adp, &finderinfo,islnk);
 		}
@@ -408,7 +391,7 @@ static int crit_check(struct vol *vol, struct path *path) {
 	}
 	
 	/* Check label */
-	if ((c1.rbitmap & (1<<DIRPBIT_FINFO)) && c2.finfo.label != 0) {
+	if ((c1.rbitmap & (1U<<DIRPBIT_FINFO)) && c2.finfo.label != 0) {
 		if (!finfo) {
 			finfo = unpack_finderinfo(vol, path, &adp, &finderinfo,islnk);
 		}
@@ -519,7 +502,8 @@ static int catsearch(struct vol *vol,
     int num_rounds = NUM_ROUNDS;
     int cwd = -1;
     int error;
-        
+    int unlen;
+
 	if (*pos != 0 && *pos != cur_pos) {
 		result = AFPERR_CATCHNG;
 		goto catsearch_end;
@@ -553,20 +537,26 @@ static int catsearch(struct vol *vol,
     start_time = time(NULL);
 
 	while ((cidx = reducestack()) != -1) {
-        LOG(log_debug, logtype_afpd, "catsearch: dir: \"%s\"", dstack[cidx].path);
+        if ((currentdir = dirlookup(vol, dstack[cidx].ds_did)) == NULL) {
+            result = AFPERR_MISC;
+            goto catsearch_end;
+        }
+        LOG(log_debug, logtype_afpd, "catsearch: current struct dir: \"%s\"", cfrombstr(currentdir->d_fullpath));
 
-		error = lchdir(dstack[cidx].path);
+		error = movecwd(vol, currentdir);
 
 		if (!error && dirpos == NULL)
 			dirpos = opendir(".");
 
-		if (dirpos == NULL)
-			dirpos = opendir(dstack[cidx].path);
+		if (dirpos == NULL) {
+			const char *temp = bdata(currentdir->d_fullpath);
+			dirpos = opendir(temp);
+		}
 
 		if (error || dirpos == NULL) {
 			switch (errno) {
 			case EACCES:
-				dstack[cidx].checked = 1;
+				dstack[cidx].ds_checked = 1;
 				continue;
 			case EMFILE:
 			case ENFILE:
@@ -581,11 +571,6 @@ static int catsearch(struct vol *vol,
 			goto catsearch_end;
 		}
 
-        if ((currentdir = dirlookup_bypath(vol, dstack[cidx].path)) == NULL) {
-            result = AFPERR_MISC;
-            goto catsearch_end;
-        }
-        LOG(log_debug, logtype_afpd, "catsearch: current struct dir: \"%s\"", cfrombstr(currentdir->d_fullpath));
 		
 		while ((entry = readdir(dirpos)) != NULL) {
 			(*pos)++;
@@ -598,7 +583,7 @@ static int catsearch(struct vol *vol,
 
 			memset(&path, 0, sizeof(path));
 			path.u_name = entry->d_name;
-			if (of_stat(&path) != 0) {
+			if (of_stat(vol, &path) != 0) {
 				switch (errno) {
 				case EACCES:
 				case ELOOP:
@@ -613,12 +598,13 @@ static int catsearch(struct vol *vol,
 					goto catsearch_end;
 				} 
 			}
-			if (S_ISDIR(path.st.st_mode)) {
+            switch (S_IFMT & path.st.st_mode) {
+            case S_IFDIR:
 				/* here we can short cut 
 				   ie if in the same loop the parent dir wasn't in the cache
 				   ALL dirsearch_byname will fail.
 				*/
-                int unlen = strlen(path.u_name);
+                unlen = strlen(path.u_name);
                 path.d_dir = dircache_search_by_name(vol,
                                                      currentdir,
                                                      path.u_name,
@@ -639,8 +625,12 @@ static int catsearch(struct vol *vol,
 					result = AFPERR_MISC;
 					goto catsearch_end;
 				} 
-            } else {
+                break;
+            case S_IFREG:
             	path.d_dir = currentdir;
+                break;
+            default:
+                continue;
             }
 
 			ccr = crit_check(vol, &path);
@@ -672,7 +662,7 @@ static int catsearch(struct vol *vol,
 		} /* while ((entry=readdir(dirpos)) != NULL) */
 		closedir(dirpos);
 		dirpos = NULL;
-		dstack[cidx].checked = 1;
+		dstack[cidx].ds_checked = 1;
 	} /* while (current_idx = reducestack()) != -1) */
 
 	/* We have finished traversing our tree. Return EOF here. */
@@ -786,7 +776,7 @@ static int catsearch_db(struct vol *vol,
         path.u_name = name;
         path.m_name = utompath(vol, name, cnid, utf8_encoding());
 
-        if (of_stat(&path) != 0) {
+        if (of_stat(vol, &path) != 0) {
             switch (errno) {
             case EACCES:
             case ELOOP:
@@ -861,7 +851,7 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
     u_int16_t	namelen;
     u_int16_t	flags;
     char  	    tmppath[256];
-    char        *uname;
+    char        *uname = NULL;
 
     *rbuflen = 0;
 
@@ -913,8 +903,8 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
     }
     else {
         /* with catsearch only name and parent id are allowed */
-    	c1.fbitmap &= (1<<FILPBIT_LNAME) | (1<<FILPBIT_PDID);
-    	c1.dbitmap &= (1<<DIRPBIT_LNAME) | (1<<DIRPBIT_PDID);
+    	c1.fbitmap &= (1U<<FILPBIT_LNAME) | (1U<<FILPBIT_PDID);
+    	c1.dbitmap &= (1U<<DIRPBIT_LNAME) | (1U<<DIRPBIT_PDID);
         spec_len = *(unsigned char*)ibuf;
     }
 
@@ -928,7 +918,7 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
     bspec1 = spec1;
     bspec2 = spec2;
     /* File attribute bits... */
-    if (c1.rbitmap & (1 << FILPBIT_ATTR)) {
+    if (c1.rbitmap & (1U << FILPBIT_ATTR)) {
 	    memcpy(&c1.attr, spec1, sizeof(c1.attr));
 	    spec1 += sizeof(c1.attr);
 	    memcpy(&c2.attr, spec2, sizeof(c2.attr));
@@ -936,7 +926,7 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
     }
 
     /* Parent DID */
-    if (c1.rbitmap & (1 << FILPBIT_PDID)) {
+    if (c1.rbitmap & (1U << FILPBIT_PDID)) {
             memcpy(&c1.pdid, spec1, sizeof(pdid));
 	    spec1 += sizeof(c1.pdid);
 	    memcpy(&c2.pdid, spec2, sizeof(pdid));
@@ -944,7 +934,7 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
     } /* FIXME: PDID - do we demarshall this argument ? */
 
     /* Creation date */
-    if (c1.rbitmap & (1 << FILPBIT_CDATE)) {
+    if (c1.rbitmap & (1U << FILPBIT_CDATE)) {
 	    memcpy(&c1.cdate, spec1, sizeof(c1.cdate));
 	    spec1 += sizeof(c1.cdate);
 	    c1.cdate = AD_DATE_TO_UNIX(c1.cdate);
@@ -955,7 +945,7 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
     }
 
     /* Modification date */
-    if (c1.rbitmap & (1 << FILPBIT_MDATE)) {
+    if (c1.rbitmap & (1U << FILPBIT_MDATE)) {
 	    memcpy(&c1.mdate, spec1, sizeof(c1.mdate));
 	    c1.mdate = AD_DATE_TO_UNIX(c1.mdate);
 	    spec1 += sizeof(c1.mdate);
@@ -965,7 +955,7 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
     }
     
     /* Backup date */
-    if (c1.rbitmap & (1 << FILPBIT_BDATE)) {
+    if (c1.rbitmap & (1U << FILPBIT_BDATE)) {
 	    memcpy(&c1.bdate, spec1, sizeof(c1.bdate));
 	    spec1 += sizeof(c1.bdate);
 	    c1.bdate = AD_DATE_TO_UNIX(c1.bdate);
@@ -975,7 +965,7 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
     }
 
     /* Finder info */
-    if (c1.rbitmap & (1 << FILPBIT_FINFO)) {
+    if (c1.rbitmap & (1U << FILPBIT_FINFO)) {
     	packed_finder buf;
     	
 	    memcpy(buf, spec1, sizeof(buf));
@@ -987,7 +977,7 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
 	    spec2 += sizeof(buf);
     } /* Finder info */
 
-    if ((c1.rbitmap & (1 << DIRPBIT_OFFCNT)) != 0) {
+    if ((c1.rbitmap & (1U << DIRPBIT_OFFCNT)) != 0) {
         /* Offspring count - only directories */
 		if (c1.fbitmap == 0) {
 	    	memcpy(&c1.offcnt, spec1, sizeof(c1.offcnt));
@@ -1006,7 +996,7 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
     } /* Offspring count/ressource fork length */
 
     /* Long name */
-    if (c1.rbitmap & (1 << FILPBIT_LNAME)) {
+    if (c1.rbitmap & (1U << FILPBIT_LNAME)) {
         /* Get the long filename */	
 		memcpy(tmppath, bspec1 + spec1[1] + 1, (bspec1 + spec1[1])[0]);
 		tmppath[(bspec1 + spec1[1])[0]]= 0;
@@ -1021,7 +1011,7 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
 #endif
     }
         /* UTF8 Name */
-    if (c1.rbitmap & (1 << FILPBIT_PDINFO)) {
+    if (c1.rbitmap & (1U << FILPBIT_PDINFO)) {
 
 		/* offset */
 		memcpy(&namelen, spec1, sizeof(namelen));
@@ -1049,8 +1039,8 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
     
     /* Call search */
     *rbuflen = 24;
-    if ((c1.rbitmap & (1 << FILPBIT_PDINFO))
-        && !(c1.rbitmap & (1<<CATPBIT_PARTIAL))
+    if ((c1.rbitmap & (1U << FILPBIT_PDINFO))
+        && !(c1.rbitmap & (1U<<CATPBIT_PARTIAL))
         && (strcmp(vol->v_cnidscheme, "dbd") == 0)
         && (vol->v_flags & AFPVOL_SEARCHDB))
         /* we've got a name and it's a dbd volume, so search CNID database */
