@@ -132,7 +132,7 @@ static int netatalk_mkdir(const struct vol *vol, const char *name)
 }
 
 /* ------------------- */
-static int deletedir(int dirfd, char *dir)
+static int deletedir(const struct vol *vol, int dirfd, char *dir)
 {
     char path[MAXPATHLEN + 1];
     DIR *dp;
@@ -149,10 +149,9 @@ static int deletedir(int dirfd, char *dir)
     if ((dp = opendirat(dirfd, dir)) == NULL)
         return AFP_OK;
 
-    strcpy(path, dir);
-    strcat(path, "/");
+    snprintf(path, MAXPATHLEN, "%s/", dir);
     len++;
-    remain = sizeof(path) -len -1;
+    remain = strlen(path) -len -1;
     while ((de = readdir(dp)) && err == AFP_OK) {
         /* skip this and previous directory */
         if (!strcmp(de->d_name, ".") || !strcmp(de->d_name, ".."))
@@ -163,11 +162,11 @@ static int deletedir(int dirfd, char *dir)
             break;
         }
         strcpy(path + len, de->d_name);
-        if (lstatat(dirfd, path, &st)) {
+        if (ostatat(dirfd, path, &st, vol_syml_opt(vol))) {
             continue;
         }
         if (S_ISDIR(st.st_mode)) {
-            err = deletedir(dirfd, path);
+            err = deletedir(vol, dirfd, path);
         } else {
             err = netatalk_unlinkat(dirfd, path);
         }
@@ -207,15 +206,13 @@ static int copydir(const struct vol *vol, int dirfd, char *src, char *dst)
     }
 
     /* set things up to copy */
-    strcpy(spath, src);
-    strcat(spath, "/");
+    snprintf(spath, MAXPATHLEN, "%s/", src);
     slen++;
-    srem = sizeof(spath) - slen -1;
+    srem = strlen(spath) - slen -1;
 
-    strcpy(dpath, dst);
-    strcat(dpath, "/");
+    snprintf(dpath, MAXPATHLEN, "%s/", dst);
     dlen++;
-    drem = sizeof(dpath) - dlen -1;
+    drem = strlen(dpath) - dlen -1;
 
     err = AFP_OK;
     while ((de = readdir(dp))) {
@@ -229,7 +226,7 @@ static int copydir(const struct vol *vol, int dirfd, char *src, char *dst)
         }
         strcpy(spath + slen, de->d_name);
 
-        if (lstatat(dirfd, spath, &st) == 0) {
+        if (ostatat(dirfd, spath, &st, vol_syml_opt(vol)) == 0) {
             if (strlen(de->d_name) > drem) {
                 err = AFPERR_PARAM;
                 break;
@@ -251,7 +248,7 @@ static int copydir(const struct vol *vol, int dirfd, char *src, char *dst)
     }
 
     /* keep the same time stamp. */
-    if (lstatat(dirfd, src, &st) == 0) {
+    if (ostatat(dirfd, src, &st, vol_syml_opt(vol)) == 0) {
         ut.actime = ut.modtime = st.st_mtime;
         utime(dst, &ut);
     }
@@ -299,7 +296,7 @@ static int set_dir_errors(struct path *path, const char *where, int err)
  *
  * @note If the passed ret->m_name is mangled, we'll demangle it
  */
-static int cname_mtouname(const struct vol *vol, const struct dir *dir, struct path *ret, int toUTF8)
+static int cname_mtouname(const struct vol *vol, struct dir *dir, struct path *ret, int toUTF8)
 {
     static char temp[ MAXPATHLEN + 1];
     char *t;
@@ -329,6 +326,12 @@ static int cname_mtouname(const struct vol *vol, const struct dir *dir, struct p
 
         /* check for OS X mangled filename :( */
         t = demangle_osx(vol, ret->m_name, dir->d_did, &fileid);
+
+        if (curdir == NULL) {
+            /* demangle_osx() calls dirlookup() which might have clobbered curdir */
+            movecwd(vol, dir);
+        }
+
         LOG(log_maxdebug, logtype_afpd, "cname_mtouname('%s',did:%u) {demangled:'%s', fileid:%u}",
             ret->m_name, ntohl(dir->d_did), t, ntohl(fileid));
 
@@ -431,96 +434,6 @@ int get_afp_errno(const int param)
     return param;
 }
 
-/*!
- * Resolve struct dir for an absolute path
- *
- * Given a path like "/Volumes/volume/dir/subdir" in a volume "/Volumes/volume" return
- * a pointer to struct dir of "subdir".
- * 1. Remove volue path from absolute path
- * 2. start path
- * 3. loop through all elements of the remaining path from 1.
- * 4. we only allow dirs
- * 5. search dircache
- * 6. if not found in the dircache query the CNID database for the DID
- * 7. and use dirlookup to resolve the DID to a it's struct dir *
- *
- * @param vol   (r) volume the path is in, must be known
- * @param path  (r) absoule path
- *
- * @returns pointer to struct dir or NULL on error
- */
-struct dir *dirlookup_bypath(const struct vol *vol, const char *path)
-{
-    EC_INIT;
-
-    struct dir *dir = NULL;
-    cnid_t cnid, did;
-    bstring rpath = NULL;
-    bstring statpath = NULL;
-    struct bstrList *l = NULL;
-    struct stat st;
-
-    cnid = htonl(2);
-    dir = vol->v_root;
-
-    LOG(log_debug, logtype_afpd, "dirlookup_bypath(\"%s\")", path);
-
-    if (strcmp(vol->v_path, path) == 0)
-        return dir;
-
-    EC_NULL(rpath = rel_path_in_vol(path, vol->v_path)); /* 1. */
-
-    LOG(log_debug, logtype_afpd, "dirlookup_bypath: rpath: \"%s\"", cfrombstr(rpath));
-
-    EC_NULL(statpath = bfromcstr(vol->v_path));          /* 2. */
-
-    l = bsplit(rpath, '/');
-    for (int i = 0; i < l->qty ; i++) {                  /* 3. */
-        did = cnid;
-        EC_ZERO(bcatcstr(statpath, "/"));
-        EC_ZERO(bconcat(statpath, l->entry[i]));
-
-        LOG(log_debug, logtype_afpd, "dirlookup_bypath: statpath: \"%s\"", cfrombstr(statpath));
-
-        EC_ZERO_LOGSTR(lstat(cfrombstr(statpath), &st),
-                       "lstat(rpath: %s, elem: %s): %s: %s",
-                       cfrombstr(rpath), cfrombstr(l->entry[i]),
-                       cfrombstr(statpath), strerror(errno));
-
-        if (!(S_ISDIR(st.st_mode)))                      /* 4. */
-            EC_FAIL;
-
-        if ((dir = dircache_search_by_name(vol,          /* 5. */
-                                           dir,
-                                           cfrombstr(l->entry[i]),
-                                           blength(l->entry[i]))) == NULL) {
-
-            if ((cnid = cnid_add(vol->v_cdb,             /* 6. */
-                                 &st,
-                                 did,
-                                 cfrombstr(l->entry[i]),
-                                 blength(l->entry[i]),
-                                 0)) == CNID_INVALID)
-                EC_FAIL;
-
-            if ((dir = dirlookup(vol, cnid)) == NULL) /* 7. */
-                EC_FAIL;
-        }
-    }
-
-EC_CLEANUP:
-    bdestroy(rpath);
-    bstrListDestroy(l);
-    bdestroy(statpath);
-    if (ret != 0)
-        return NULL;
-
-    LOG(log_debug, logtype_afpd, "dirlookup_bypath: result: \"%s\"",
-        cfrombstr(dir->d_fullpath));
-
-    return dir;
-}
-
 /*!
  * @brief Resolve a DID
  *
@@ -641,7 +554,7 @@ struct dir *dirlookup(const struct vol *vol, cnid_t did)
     LOG(log_debug, logtype_afpd, "dirlookup(did: %u): stating \"%s\"",
         ntohl(did), cfrombstr(fullpath));
 
-    if (lstat(cfrombstr(fullpath), &st) != 0) { /* 5a */
+    if (ostat(cfrombstr(fullpath), &st, vol_syml_opt(vol)) != 0) { /* 5a */
         switch (errno) {
         case ENOENT:
             afp_errno = AFPERR_NOOBJ;
@@ -703,8 +616,6 @@ exit:
     return ret;
 }
 
-#define ENUMVETO "./../Network Trash Folder/TheVolumeSettingsFolder/TheFindByContentFolder/:2eDS_Store/Contents/Desktop Folder/Trash/Benutzer/"
-
 int caseenumerate(const struct vol *vol, struct path *path, struct dir *dir)
 {
     DIR               *dp;
@@ -720,16 +631,13 @@ int caseenumerate(const struct vol *vol, struct path *path, struct dir *dir)
     if (!(vol->v_flags & AFPVOL_CASEINSEN))
         return -1;
 
-    if (veto_file(ENUMVETO, path->u_name))
-        return -1;
-
     savepath = path->u_name;
 
     /* very simple cache */
     if ( dir->d_did == did && strcmp(lname, path->u_name) == 0) {
         path->u_name = cname;
         path->d_dir = NULL;
-        if (of_stat( path ) == 0 ) {
+        if (of_stat(vol, path ) == 0 ) {
             return 0;
         }
         /* something changed, we cannot stat ... */
@@ -760,7 +668,7 @@ int caseenumerate(const struct vol *vol, struct path *path, struct dir *dir)
             strlcpy(cname, de->d_name, sizeof(cname));
             path->u_name = cname;
             path->d_dir = NULL;
-            if (of_stat( path ) == 0 ) {
+            if (of_stat(vol, path ) == 0 ) {
                 LOG(log_debug, logtype_afpd, "caseenumerate: using dir: %s, path: %s", de->d_name, path->u_name);
                 strlcpy(lname, tmp, sizeof(lname));
                 did = dir->d_did;
@@ -893,7 +801,7 @@ struct dir *dir_add(struct vol *vol, const struct dir *dir, struct path *path, i
     cnid_t      id;
     struct adouble  ad;
     struct adouble *adp = NULL;
-    bstring fullpath;
+    bstring fullpath = NULL;
 
     AFP_ASSERT(vol);
     AFP_ASSERT(dir);
@@ -917,7 +825,7 @@ struct dir *dir_add(struct vol *vol, const struct dir *dir, struct path *path, i
         adp = &ad;
 
     /* Get CNID */
-    if ((id = get_id(vol, adp, &path->st, dir->d_did, path->u_name, len)) == 0) { /* 2 */
+    if ((id = get_id(vol, adp, &path->st, dir->d_did, path->u_name, len)) == CNID_INVALID) { /* 2 */
         err = 1;
         goto exit;
     }
@@ -1028,87 +936,6 @@ int dir_remove(const struct vol *vol, struct dir *dir)
     return 0;
 }
 
-#if 0 /* unused */
-/*!
- * @brief Modify a struct dir, adjust cache
- *
- * Any value that is 0 or NULL is not changed. If new_uname is NULL it is set to new_mname.
- * If given new_uname == new_mname, new_uname will point to new_mname.
- *
- * @param vol       (r) pointer to struct vol
- * @param dir       (rw) pointer to struct dir
- * @param pdid      (r) new parent DID
- * @param did       (r) new DID
- * @param new_mname (r) new mac-name
- * @param new_uname (r) new unix-name
- * @param pdir_fullpath (r) new fullpath of parent dir
- */
-int dir_modify(const struct vol *vol,
-               struct dir *dir,
-               cnid_t pdid,
-               cnid_t did,
-               const char *new_mname,
-               const char *new_uname,
-               bstring pdir_fullpath)
-{
-    int ret = 0;
-
-    /* Remove it from the cache */
-    dircache_remove(vol, dir, DIRCACHE | DIDNAME_INDEX | QUEUE_INDEX);
-
-    if (pdid)
-        dir->d_pdid = pdid;
-    if (did)
-        dir->d_did = did;
-
-    if (new_mname) {
-        /* free uname if it's not the same as mname */
-        if (dir->d_m_name != dir->d_u_name)
-            bdestroy(dir->d_u_name);
-
-        if (new_uname == NULL)
-            new_uname = new_mname;
-
-        /* assign new name */
-        if ((bassigncstr(dir->d_m_name, new_mname)) != BSTR_OK) {
-            LOG(log_error, logtype_afpd, "dir_modify: bassigncstr: %s", strerror(errno) );
-            return -1;
-        }
-
-        if (new_mname == new_uname || (strcmp(new_mname, new_uname) == 0)) {
-            dir->d_u_name = dir->d_m_name;
-        } else {
-            if ((dir->d_u_name = bfromcstr(new_uname)) == NULL) {
-                LOG(log_error, logtype_afpd, "dir_modify: bassigncstr: %s", strerror(errno) );
-                return -1;
-            }
-        }
-    }
-
-    if (pdir_fullpath) {
-        if (bassign(dir->d_fullpath, pdir_fullpath) != BSTR_OK)
-            return -1;
-        if (bcatcstr(dir->d_fullpath, "/") != BSTR_OK)
-            return -1;
-        if (bcatcstr(dir->d_fullpath, new_uname) != BSTR_OK)
-            return -1;
-    }
-
-    if (dir->d_m_name_ucs2)
-        free(dir->d_m_name_ucs2);
-    if ((size_t)-1 == convert_string_allocate((utf8_encoding())?CH_UTF8_MAC:vol->v_maccharset, CH_UCS2, dir->d_m_name, -1, (char**)&dir->d_m_name_ucs2))
-        dir->d_m_name_ucs2 = NULL;
-
-    /* Re-add it to the cache */
-    if ((dircache_add(vol, dir)) != 0) {
-        dircache_dump();
-        AFP_PANIC("dir_modify");
-    }
-
-    return ret;
-}
-#endif
-
 /*!
  * @brief Resolve a catalog node name path
  *
@@ -1127,7 +954,7 @@ int dir_modify(const struct vol *vol,
  *     return with curdir = dir parent, struct path = dirname
  * 11. If it's there and it's a file, it must should be the last element of the requested
  *     path. Return with curdir = cnode name parent dir, struct path = filename
- * 12. Treat symlinks like files, dont follow them
+ * 12. Treat symlinks like files, don't follow them
  * 13. If it's a dir:
  * 14. Search the dircache for it
  * 15. If it's not in the cache, create a struct dir for it and add it to the cache
@@ -1230,7 +1057,7 @@ struct path *cname(struct vol *vol, struct dir *dir, char **cpath)
             return NULL;
         }
 
-        LOG(log_maxdebug, logtype_afpd, "came('%s'): {node: '%s}", cfrombstr(dir->d_fullpath), ret.u_name);
+        LOG(log_maxdebug, logtype_afpd, "cname('%s'): {node: '%s}", cfrombstr(dir->d_fullpath), ret.u_name);
 
         /* Prevent access to our special folders like .AppleDouble */
         if (check_name(vol, ret.u_name)) {
@@ -1259,7 +1086,7 @@ struct path *cname(struct vol *vol, struct dir *dir, char **cpath)
              *   and thus call continue which should terminate the while loop because
              *   len = 0. Ok?
              */
-            if (of_stat(&ret) != 0) { /* 9 */
+            if (of_stat(vol, &ret) != 0) { /* 9 */
                 /*
                  * ret.u_name doesn't exist, might be afp_createfile|dir
                  * that means it should have been the last part
@@ -1357,7 +1184,7 @@ struct path *cname(struct vol *vol, struct dir *dir, char **cpath)
  * @param vol   (r) pointer to struct vol
  * @param dir   (r) pointer to struct dir
  *
- * @returns 0 on success, -1 on error with afp_errno set appropiately
+ * @returns 0 on success, -1 on error with afp_errno set appropriately
  */
 int movecwd(const struct vol *vol, struct dir *dir)
 {
@@ -1377,7 +1204,7 @@ int movecwd(const struct vol *vol, struct dir *dir)
     LOG(log_debug, logtype_afpd, "movecwd(to: did: %u, \"%s\")",
         ntohl(dir->d_did), cfrombstr(dir->d_fullpath));
 
-    if ((ret = lchdir(cfrombstr(dir->d_fullpath))) != 0 ) {
+    if ((ret = ochdir(cfrombstr(dir->d_fullpath), vol_syml_opt(vol))) != 0 ) {
         LOG(log_debug, logtype_afpd, "movecwd(\"%s\"): %s",
             cfrombstr(dir->d_fullpath), strerror(errno));
         if (ret == 1) {
@@ -1581,10 +1408,6 @@ int getdirparams(const struct vol *vol,
                 memcpy( data, ad_entry( &ad, ADEID_FINDERI ), 32 );
             } else { /* no appledouble */
                 memset( data, 0, 32 );
-                /* set default view -- this also gets done in ad_open() */
-                ashort = htons(FINDERINFO_CLOSEDVIEW);
-                memcpy(data + FINDERINFO_FRVIEWOFF, &ashort, sizeof(ashort));
-
                 /* dot files are by default visible */
                 if (invisible_dots(vol, cfrombstr(dir->d_u_name))) {
                     ashort = htons(FINDERINFO_INVISIBLE);
@@ -1785,7 +1608,7 @@ int afp_setdirparams(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U_
     /*
      * If ibuf is odd, make it even.
      */
-    if ((u_long)ibuf & 1 ) {
+    if ((intptr_t)ibuf & 1 ) {
         ibuf++;
     }
 
@@ -1994,9 +1817,12 @@ int setdirparams(struct vol *vol, struct path *path, u_int16_t d_bitmap, char *b
         case DIRPBIT_FINFO :
             if (isad) {
                 /* Fixes #2802236 */
-                u_int16_t *fflags = (u_int16_t *)(finder_buf + FINDERINFO_FRFLAGOFF);
-                *fflags &= htons(~FINDERINFO_ISHARED);
+                uint16_t fflags;
+                memcpy(&fflags, finder_buf + FINDERINFO_FRFLAGOFF, sizeof(uint16_t));
+                fflags &= htons(~FINDERINFO_ISHARED);
+                memcpy(finder_buf + FINDERINFO_FRFLAGOFF, &fflags, sizeof(uint16_t));
                 /* #2802236 end */
+
                 if (  dir->d_did == DIRDID_ROOT ) {
                     /*
                      * Alright, we admit it, this is *really* sick!
@@ -2267,7 +2093,7 @@ int afp_createdir(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf, size_
         return err;
     }
 
-    if (of_stat(s_path) < 0) {
+    if (of_stat(vol, s_path) < 0) {
         return AFPERR_MISC;
     }
 
@@ -2335,10 +2161,10 @@ int renamedir(const struct vol *vol,
             /* this needs to copy and delete. bleah. that means we have
              * to deal with entire directory hierarchies. */
             if ((err = copydir(vol, dirfd, src, dst)) < 0) {
-                deletedir(-1, dst);
+                deletedir(vol, -1, dst);
                 return err;
             }
-            if ((err = deletedir(dirfd, src)) < 0)
+            if ((err = deletedir(vol, dirfd, src)) < 0)
                 return err;
             break;
         default :
@@ -2650,34 +2476,9 @@ int afp_mapname(AFPObj *obj _U_, char *ibuf, size_t ibuflen _U_, char *rbuf, siz
 */
 int afp_closedir(AFPObj *obj _U_, char *ibuf _U_, size_t ibuflen _U_, char *rbuf _U_, size_t *rbuflen)
 {
-#if 0
-    struct vol   *vol;
-    struct dir   *dir;
-    u_int16_t    vid;
-    u_int32_t    did;
-#endif /* 0 */
-
     *rbuflen = 0;
 
     /* do nothing as dids are static for the life of the process. */
-#if 0
-    ibuf += 2;
-
-    memcpy(&vid,  ibuf, sizeof( vid ));
-    ibuf += sizeof( vid );
-    if (( vol = getvolbyvid( vid )) == NULL ) {
-        return( AFPERR_PARAM );
-    }
-
-    memcpy( &did, ibuf, sizeof( did ));
-    ibuf += sizeof( did );
-    if (( dir = dirlookup( vol, did )) == NULL ) {
-        return( AFPERR_PARAM );
-    }
-
-    /* dir_remove -- deletedid */
-#endif /* 0 */
-
     return AFP_OK;
 }
 
@@ -2717,7 +2518,7 @@ int afp_opendir(AFPObj *obj _U_, char *ibuf, size_t ibuflen  _U_, char *rbuf, si
         return path_error(path, AFPERR_NOOBJ);
     }
 
-    if ( !path->st_valid && of_stat(path ) < 0 ) {
+    if ( !path->st_valid && of_stat(vol, path) < 0 ) {
         return( AFPERR_NOOBJ );
     }
     if ( path->st_errno ) {
