@@ -9,22 +9,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-
-/* STDC check */
-#if STDC_HEADERS
 #include <string.h>
-#else /* STDC_HEADERS */
-#ifndef HAVE_STRCHR
-#define strchr index
-#define strrchr index
-#endif /* HAVE_STRCHR */
-char *strchr (), *strrchr ();
-
-#ifndef HAVE_MEMCPY
-#define memcpy(d,s,n) bcopy ((s), (d), (n))
-#define memmove(d,s,n) bcopy ((s), (d), (n))
-#endif /* ! HAVE_MEMCPY */
-#endif /* STDC_HEADERS */
 
 #include <utime.h>
 #include <errno.h>
@@ -115,7 +100,7 @@ void *get_finderinfo(const struct vol *vol, const char *upath, struct adouble *a
         }
     }
 
-    if (islink){
+    if (islink && !vol_syml_opt(vol)) {
         u_int16_t linkflag;
         memcpy(&linkflag, (char *)data + FINDERINFO_FRFLAGOFF, 2);
         linkflag |= htons(FINDERINFO_ISALIAS);
@@ -206,8 +191,7 @@ char *set_name(const struct vol *vol, char *data, cnid_t pid, char *name, cnid_t
  *
  * 1. Get the objects CNID as stored in its adouble file
  * 2. Get the objects CNID from the database
- * 3. If there's a problem with a "dbd" database, fallback to "tdb" in memory
- * 4. In case 2 and 3 differ, store 3 in the adouble file
+ * 3. Store in the adouble file
  *
  * @param vol    (rw) volume
  * @param adp    (rw) adouble struct of object upath, might be NULL
@@ -223,11 +207,9 @@ uint32_t get_id(struct vol *vol,
                 const char *upath,
                 const int len) 
 {
-    static int first = 1;       /* mark if this func is called the first time */
     u_int32_t adcnid;
     u_int32_t dbcnid = CNID_INVALID;
 
-restart:
     if (vol->v_cdb != NULL) {
         /* prime aint with what we think is the cnid, set did to zero for
            catching moved files */
@@ -239,6 +221,14 @@ restart:
             switch (errno) {
             case CNID_ERR_CLOSE: /* the db is closed */
                 break;
+            case CNID_ERR_DB:
+                LOG(log_error, logtype_afpd,
+                    "get_id: Connection to the CNID backend DB failed. "
+                    "This is now treated as a fatal error. "
+                    "Is the cnid_metad process running? "
+                    "Please escalate to the upstream project "
+                    "if you suspect this is a bug");
+                exit(EXITERR_SYS);
             case CNID_ERR_PARAM:
                 LOG(log_error, logtype_afpd, "get_id: Incorrect parameters passed to cnid_add");
                 afp_errno = AFPERR_PARAM;
@@ -247,41 +237,11 @@ restart:
                 afp_errno = AFPERR_PARAM;
                 goto exit;
             default:
-                /* Close CNID backend if "dbd" and switch to temp in-memory "tdb" */
-                /* we have to do it here for "dbd" because it uses "lazy opening" */
-                /* In order to not end in a loop somehow with goto restart below  */
-                /*  */
-                if (first && (strcmp(vol->v_cnidscheme, "dbd") == 0)) { /* (3) */
-                    cnid_close(vol->v_cdb);
-                    free(vol->v_cnidscheme);
-                    vol->v_cnidscheme = strdup("tdb");
-
-                    int flags = CNID_FLAG_MEMORY;
-                    if ((vol->v_flags & AFPVOL_NODEV)) {
-                        flags |= CNID_FLAG_NODEV;
-                    }
-                    LOG(log_error, logtype_afpd, "Reopen volume %s using in memory temporary CNID DB.",
-                        vol->v_path);
-                    vol->v_cdb = cnid_open(vol->v_path, vol->v_umask, "tdb", flags, NULL, NULL);
-                    if (vol->v_cdb) {
-                        vol->v_flags &= ~AFPVOL_CACHE;
-                        if (!(vol->v_flags & AFPVOL_TM)) {
-                            vol->v_flags |= AFPVOL_RO;
-                            setmessage("Something wrong with the volume's CNID DB, using temporary CNID DB instead."
-                                       "Check server messages for details. Switching to read-only mode.");
-                            kill(getpid(), SIGUSR2);
-                        }
-                        goto restart; /* now try again with the temp CNID db */
-                    } else {
-                        setmessage("Something wrong with the volume's CNID DB, using temporary CNID DB failed too!"
-                                   "Check server messages for details, can't recover from this state!");
-                    }
-                }
                 afp_errno = AFPERR_MISC;
                 goto exit;
             }
         }
-        else if (adp && (adcnid != dbcnid)) { /* 4 */
+        else if (adp && (adcnid != dbcnid)) { /* 3 */
             /* Update the ressource fork. For a folder adp is always null */
             LOG(log_debug, logtype_afpd, "get_id(%s/%s): calling ad_setid(old: %u, new: %u)",
                 getcwdpath(), upath, htonl(adcnid), htonl(dbcnid));
@@ -292,7 +252,6 @@ restart:
     }
 
 exit:
-    first = 0;
     return dbcnid;
 }
              
@@ -539,7 +498,7 @@ int getmetadata(struct vol *vol,
             }
             break;
         case FILPBIT_EXTDFLEN:
-            aint = htonl(st->st_size >> 32);
+            aint = htonl((uint64_t)st->st_size >> 32);
             memcpy(data, &aint, sizeof( aint ));
             data += sizeof( aint );
             aint = htonl(st->st_size);
@@ -549,7 +508,7 @@ int getmetadata(struct vol *vol,
         case FILPBIT_EXTRFLEN:
             aint = 0;
             if (adp) 
-                aint = htonl(adp->ad_rlen >> 32);
+                aint = htonl((uint64_t)adp->ad_rlen >> 32);
             memcpy(data, &aint, sizeof( aint ));
             data += sizeof( aint );
             if (adp) 
@@ -701,7 +660,7 @@ int afp_createfile(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U_,
     upath = s_path->u_name;
     
     /* if upath is deleted we already in trouble anyway */
-    if ((of = of_findname(s_path))) {
+    if ((of = of_findname(vol, s_path))) {
         adp = of->of_ad;
     } else {
         ad_init(&ad, vol->v_adouble, vol->v_ad_options);
@@ -777,11 +736,6 @@ int afp_createfile(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U_,
 createfile_done:
     curdir->d_offcnt++;
 
-#ifdef DROPKLUDGE
-    if (vol->v_flags & AFPVOL_DROPBOX) {
-        retvalue = matchfile2dirperms(upath, vol, did);
-    }
-#endif /* DROPKLUDGE */
 
     setvoltime(obj, vol );
 
@@ -830,7 +784,7 @@ int afp_setfilparams(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U_
         return( AFPERR_NOOBJ );
     }
 
-    if ((u_long)ibuf & 1 ) {
+    if ((intptr_t)ibuf & 1 ) {
         ibuf++;
     }
 
@@ -854,10 +808,12 @@ int setfilparams(struct vol *vol,
     struct extmap	*em;
     int			bit, isad = 1, err = AFP_OK;
     char                *upath;
-    u_char              achar, *fdType, xyy[4]; /* uninitialized, OK 310105 */
-    u_int16_t		ashort, bshort, oshort;
+    u_char              achar, xyy[4];
+    u_char              *fdType = NULL;
+    u_int16_t		ashort = 0;
+    u_int16_t		bshort, oshort;
     u_int32_t		aint;
-    u_int32_t		upriv;
+    u_int32_t		upriv = 0;
     u_int16_t           upriv_bit = 0;
     
     struct utimbuf	ut;
@@ -871,10 +827,11 @@ int setfilparams(struct vol *vol,
     u_int16_t           bitmap = f_bitmap;
     u_int32_t           cdate,bdate;
     u_char              finder_buf[32];
+    int fp;
+    ssize_t len;
+    char symbuf[MAXPATHLEN+1];
 
-#ifdef DEBUG
     LOG(log_debug9, logtype_afpd, "begin setfilparams:");
-#endif /* DEBUG */
 
     adp = of_ad(vol, path, &ad);
     upath = path->u_name;
@@ -892,7 +849,9 @@ int setfilparams(struct vol *vol,
         }
         switch(  bit ) {
         case FILPBIT_ATTR :
-            change_mdate = 1;
+            /* Preserve file dates during a GS/OS folder copy */
+            if (afp_version >= 30) 
+                change_mdate = 1;
             memcpy(&ashort, buf, sizeof( ashort ));
             buf += sizeof( ashort );
             break;
@@ -912,29 +871,32 @@ int setfilparams(struct vol *vol,
             break;
         case FILPBIT_FINFO :
             change_mdate = 1;
-            memcpy(finder_buf, buf, 32 );
-            if (memcmp(buf,"slnkrhap",8)==0 && !S_ISLNK(path->st.st_mode)){
-            // SLFINFO
-                int fp;
-                ssize_t len;
-                int erc=1;
-                char buf[PATH_MAX+1];
-                if ((fp=open(path->u_name,O_RDONLY))>=0){
-                    if ((len=read(fp,buf,PATH_MAX+1))){
-                        if (unlink(path->u_name)==0){
-                            buf[len]=0;
-                            erc = symlink(buf, path->u_name);
-                            if (!erc)
-                                of_stat(path);
-                        }
-                    }
-                    close(fp);
-                }
-                if (erc!=0){
-                    err=AFPERR_BITMAP;
+            if (memcmp(buf,"slnkrhap",8) == 0
+                && !(S_ISLNK(path->st.st_mode))
+                && !(vol->v_flags & AFPVOL_FOLLOWSYM)) {
+                /* request to turn this into a symlink */
+                if ((fp = open(path->u_name, O_RDONLY)) == -1) {
+                    err = AFPERR_MISC;
                     goto setfilparam_done;
                 }
+                len = read(fp, symbuf, MAXPATHLEN);
+                close(fp);
+                if (!(len > 0)) {
+                    err = AFPERR_MISC;
+                    goto setfilparam_done;
+                }
+                if (unlink(path->u_name) != 0) {
+                    err = AFPERR_MISC;
+                    goto setfilparam_done;
+                }
+                symbuf[len] = 0;
+                if (symlink(symbuf, path->u_name) != 0) {
+                    err = AFPERR_MISC;
+                    goto setfilparam_done;
+                }
+                of_stat(vol, path);
             }
+            memcpy(finder_buf, buf, 32 );
             buf += 32;
             break;
         case FILPBIT_UNIXPR :
@@ -1100,9 +1062,8 @@ setfilparam_done:
         setdirparams(vol, &Cur_Path, bitmap, (char *)&newdate);
     }
 
-#ifdef DEBUG
     LOG(log_debug9, logtype_afpd, "end setfilparams:");
-#endif /* DEBUG */
+
     return err;
 }
 
@@ -1327,9 +1288,6 @@ int afp_copyfile(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U_, si
         goto copy_exit;
     }
 
-#ifdef FORCE_UIDGID
-    /* FIXME svid != dvid && dvid's user can't read svid */
-#endif
     if (NULL == ( d_vol = getvolbyvid( dvid )) ) {
         retvalue = AFPERR_PARAM;
         goto copy_exit;
@@ -1374,11 +1332,6 @@ int afp_copyfile(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U_, si
     }
     curdir->d_offcnt++;
 
-#ifdef DROPKLUDGE
-    if (vol->v_flags & AFPVOL_DROPBOX) {
-        retvalue=matchfile2dirperms(upath, vol, ddid); /* FIXME sdir or ddid */
-    }
-#endif /* DROPKLUDGE */
 
     setvoltime(obj, d_vol );
 
@@ -1393,9 +1346,7 @@ static int copy_all(const int dfd, const void *buf,
 {
     ssize_t cc;
 
-#ifdef DEBUG
     LOG(log_debug9, logtype_afpd, "begin copy_all:");
-#endif /* DEBUG */
 
     while (buflen > 0) {
         if ((cc = write(dfd, buf, buflen)) < 0) {
@@ -1409,9 +1360,7 @@ static int copy_all(const int dfd, const void *buf,
         buflen -= cc;
     }
 
-#ifdef DEBUG
     LOG(log_debug9, logtype_afpd, "end copy_all:");
-#endif /* DEBUG */
 
     return 0;
 }
@@ -1441,35 +1390,6 @@ static int copy_fork(int eid, struct adouble *add, struct adouble *ads)
     if ((off_t)-1 == lseek(dfd, ad_getentryoff(add, eid), SEEK_SET))
     	return -1;
     	
-#if 0 /* ifdef SENDFILE_FLAVOR_LINUX */
-    /* doesn't work With 2.6 FIXME, only check for EBADFD ? */
-    off_t   offset = 0;
-    size_t  size;
-    struct stat         st;
-    #define BUF 128*1024*1024
-
-    if (fstat(sfd, &st) == 0) {
-        
-        while (1) {
-            if ( offset >= st.st_size) {
-               return 0;
-            }
-            size = (st.st_size -offset > BUF)?BUF:st.st_size -offset;
-            if ((cc = sys_sendfile(dfd, sfd, &offset, size)) < 0) {
-                switch (errno) {
-                case ENOSYS:
-                case EINVAL:  /* there's no guarantee that all fs support sendfile */
-                    goto no_sendfile;
-                default:
-                    return -1;
-                }
-            }
-        }
-    }
-    no_sendfile:
-    lseek(sfd, offset, SEEK_SET);
-#endif 
-
     while (1) {
         if ((cc = read(sfd, filebuf, sizeof(filebuf))) < 0) {
             if (errno == EINTR)
@@ -1807,7 +1727,7 @@ static int reenumerate_loop(struct dirent *de, char *mname _U_, void *data)
     cnid_t        did  = param->did;
     cnid_t	  aint;
     
-    if ( lstat(de->d_name, &path.st)<0 )
+    if (ostat(de->d_name, &path.st, vol_syml_opt(vol)) < 0)
         return 0;
     
     /* update or add to cnid */
@@ -1852,7 +1772,7 @@ reenumerate_id(struct vol *vol, char *name, struct dir *dir)
     }
     
     /* FIXME use of_statdir ? */
-    if (lstat(name, &st)) {
+    if (ostat(name, &st, vol_syml_opt(vol))) {
 	return -1;
     }
 
@@ -1931,7 +1851,7 @@ retry:
 
     memset(&path, 0, sizeof(path));
     path.u_name = upath;
-    if ( of_stat(&path) < 0 ) {
+    if (of_stat(vol, &path) < 0 ) {
 #ifdef ESTALE
         /* with nfs and our working directory is deleted */
 	if (errno == ESTALE) {
@@ -2026,7 +1946,7 @@ int afp_deleteid(AFPObj *obj _U_, char *ibuf, size_t ibuflen _U_, char *rbuf _U_
     }
 
     err = AFP_OK;
-    if ((movecwd(vol, dir) < 0) || (lstat(upath, &st) < 0)) {
+    if ((movecwd(vol, dir) < 0) || (ostat(upath, &st, vol_syml_opt(vol)) < 0)) {
         switch (errno) {
         case EACCES:
         case EPERM:
@@ -2062,7 +1982,7 @@ delete:
 }
 
 /* ------------------------------ */
-static struct adouble *find_adouble(struct path *path, struct ofork **of, struct adouble *adp)
+static struct adouble *find_adouble(const struct vol *vol, struct path *path, struct ofork **of, struct adouble *adp)
 {
     int             ret;
 
@@ -2089,7 +2009,7 @@ static struct adouble *find_adouble(struct path *path, struct ofork **of, struct
         return NULL;
     }
     
-    if ((*of = of_findname(path))) {
+    if ((*of = of_findname(vol, path))) {
         /* reuse struct adouble so it won't break locks */
         adp = (*of)->of_ad;
     }
@@ -2118,7 +2038,7 @@ int afp_exchangefiles(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U
     char		*spath, temp[17], *p;
     char                *supath, *upath;
     struct path         *path;
-    int                 err;
+    int                 err, fd;
     struct adouble	ads;
     struct adouble	add;
     struct adouble	*adsp = NULL;
@@ -2127,7 +2047,7 @@ int afp_exchangefiles(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U
     struct ofork	*d_of = NULL;
     int                 crossdev;
     
-    int                 slen, dlen;
+    unsigned int        slen, dlen;
     u_int32_t		sid, did;
     u_int16_t		vid;
 
@@ -2180,7 +2100,7 @@ int afp_exchangefiles(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U
     }
     
     ad_init(&ads, vol->v_adouble, vol->v_ad_options);
-    if (!(adsp = find_adouble( path, &s_of, &ads))) {
+    if (!(adsp = find_adouble(vol, path, &s_of, &ads))) {
         return afp_errno;
     }
 
@@ -2213,7 +2133,7 @@ int afp_exchangefiles(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U
     }
 
     ad_init(&add, vol->v_adouble, vol->v_ad_options);
-    if (!(addp = find_adouble( path, &d_of, &add))) {
+    if (!(addp = find_adouble(vol, path, &d_of, &add))) {
         err = afp_errno;
         goto err_exchangefile;
     }
@@ -2237,10 +2157,11 @@ int afp_exchangefiles(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U
      * NOTE: the temp file will be in the dest file's directory. it
      * will also be inaccessible from AFP. */
     memcpy(temp, APPLETEMP, sizeof(APPLETEMP));
-    if (!mktemp(temp)) {
+    if ((fd = mkstemp(temp)) == -1) {
         err = AFPERR_MISC;
         goto err_exchangefile;
     }
+    close(fd);
     
     if (crossdev) {
         /* FIXME we need to close fork for copy, both s_of and d_of are null */
@@ -2271,10 +2192,10 @@ int afp_exchangefiles(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U
     if (did)
         cnid_delete(vol->v_cdb, did);
 
-    if ((did && ( (crossdev && lstat( upath, &srcst) < 0) || 
+    if ((did && ( (crossdev && ostat(upath, &srcst, vol_syml_opt(vol)) < 0) || 
                 cnid_update(vol->v_cdb, did, &srcst, curdir->d_did,upath, dlen) < 0))
        ||
-       (sid && ( (crossdev && lstat(p, &destst) < 0) ||
+        (sid && ( (crossdev && ostat(p, &destst, vol_syml_opt(vol)) < 0) ||
                 cnid_update(vol->v_cdb, sid, &destst, sdir->d_did,supath, slen) < 0))
     ) {
         switch (errno) {
