@@ -1,6 +1,4 @@
 /*
- * $Id: fork.c,v 1.73 2010-03-30 12:55:26 franklahm Exp $
- *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * All Rights Reserved.  See COPYRIGHT.
  */
@@ -36,16 +34,6 @@
 #include "desktop.h"
 #include "volume.h"
 
-#ifdef DEBUG1
-#define Debug(a) ((a)->options.flags & OPTION_DEBUG)
-#else
-#define Debug(a) (0)
-#endif
-
-#ifdef AFS
-struct ofork *writtenfork;
-#endif
-
 static int getforkparams(struct ofork *ofork, u_int16_t bitmap, char *buf, size_t *buflen)
 {
     struct path         path;
@@ -112,7 +100,7 @@ static off_t get_off_t(char **ibuf, int is64)
     if (is64) {
         memcpy(&temp, *ibuf, sizeof( temp ));
         *ibuf += sizeof(temp);
-        ret = ntohl(temp)| (ret << 32);
+        ret = ntohl(temp)| ((unsigned long long) ret << 32);
     }
     else {
     	ret = (int)ret;	/* sign extend */
@@ -128,7 +116,7 @@ static int set_off_t(off_t offset, char *rbuf, int is64)
 
     ret = 0;
     if (is64) {
-        temp = htonl(offset >> 32);
+        temp = htonl((uint64_t) offset >> 32);
         memcpy(rbuf, &temp, sizeof( temp ));
         rbuf += sizeof(temp);
         ret = sizeof( temp );
@@ -331,7 +319,7 @@ int afp_openfork(AFPObj *obj _U_, char *ibuf, size_t ibuflen _U_, char *rbuf, si
        ad_open so that we can keep file locks together.
        FIXME: add the fork we are opening? 
     */
-    if ((opened = of_findname(s_path))) {
+    if ((opened = of_findname(vol, s_path))) {
         adsame = opened->of_ad;
     }
 
@@ -522,6 +510,8 @@ openfork_err:
 int afp_setforkparams(AFPObj *obj _U_, char *ibuf, size_t ibuflen, char *rbuf _U_, size_t *rbuflen)
 {
     struct ofork	*ofork;
+    struct vol      *vol;
+    struct dir      *dir;
     off_t		size;
     u_int16_t		ofrefnum, bitmap;
     int                 err;
@@ -544,6 +534,16 @@ int afp_setforkparams(AFPObj *obj _U_, char *ibuf, size_t ibuflen, char *rbuf _U
         return( AFPERR_PARAM );
     }
 
+    vol = ofork->of_vol;
+    if ((dir = dirlookup(vol, ofork->of_did)) == NULL) {
+        LOG(log_error, logtype_afpd, "%s: bad fork did",  of_name(ofork));
+        return AFPERR_MISC;
+    }
+    if (movecwd(vol, dir) != 0) {
+        LOG(log_error, logtype_afpd, "%s: bad fork directory", dir->d_fullpath);
+        return AFPERR_MISC;
+    }
+
     if (ofork->of_vol->v_flags & AFPVOL_RO)
         return AFPERR_VLOCK;
 
@@ -617,12 +617,6 @@ int afp_setforkparams(AFPObj *obj _U_, char *ibuf, size_t ibuflen, char *rbuf _U
     } else
         return AFPERR_BITMAP;
 
-#ifdef AFS
-    if ( flushfork( ofork ) < 0 ) {
-        LOG(log_error, logtype_afpd, "afp_setforkparams(%s): flushfork: %s", of_name(ofork), strerror(errno) );
-    }
-#endif /* AFS */
-
     return( AFP_OK );
 
 afp_setfork_err:
@@ -940,7 +934,7 @@ static int read_fork(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf, si
         /* due to the nature of afp packets, we have to exit if we get
            an error. we can't do this with translation on. */
 #ifdef WITH_SENDFILE 
-        if (!(xlate || Debug(obj) )) {
+        if (!(xlate || obj->options.flags & OPTION_DEBUG)) {
             int fd;
                         
             fd = ad_readfile_init(ofork->of_ad, eid, &offset, 0);
@@ -1210,10 +1204,6 @@ static int write_fork(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf, s
         goto afp_write_err;
     }
 
-#ifdef AFS
-    writtenfork = ofork;
-#endif /* AFS */
-
     if ( ofork->of_flags & AFPFORK_DATA) {
         eid = ADEID_DFORK;
         xlate = (ofork->of_vol->v_flags & AFPVOL_CRLF) ? crlf(ofork) : 0;
@@ -1267,12 +1257,11 @@ static int write_fork(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf, s
             return( AFPERR_PARAM );
         }
 
-#ifdef DEBUG1
         if (obj->options.flags & OPTION_DEBUG) {
-            printf("(write) len: %d\n", *rbuflen);
+            printf("(write) len: %ld\n", (unsigned long) *rbuflen);
             bprint(rbuf, *rbuflen);
         }
-#endif
+
         if ((cc = write_file(ofork, eid, offset, rbuf, *rbuflen,
                              xlate)) < 0) {
             *rbuflen = 0;
