@@ -1,6 +1,4 @@
 /*
- * $Id: messages.c,v 1.23 2009-11-24 15:44:40 didg Exp $
- *
  * Copyright (c) 1997 Adrian Sun (asun@zoology.washington.edu)
  * All Rights Reserved.  See COPYRIGHT.
  */
@@ -9,9 +7,7 @@
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
-#ifdef HAVE_UNISTD_H
 #include <unistd.h>
-#endif /* HAVE_UNISTD_H */
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
@@ -24,7 +20,6 @@
 
 #include "misc.h"
 
-
 #define MAXMESGSIZE 199
 
 /* this is only used by afpd children, so it's okay. */
@@ -59,9 +54,7 @@ void readmessage(AFPObj *obj)
 
     sprintf(filename, "%s/message.%d", SERVERTEXT, getpid());
 
-#ifdef DEBUG
     LOG(log_debug9, logtype_afpd, "Reading file %s ", filename);
-#endif 
 
     message=fopen(filename, "r");
     if (message==NULL) {
@@ -102,13 +95,13 @@ void readmessage(AFPObj *obj)
         if (rc < 0) {
             LOG(log_error, logtype_afpd, "Error deleting %s: %s", filename, strerror(rc));
         }
-#ifdef DEBUG
+
         else {
-            LOG(log_debug9, logtype_afpd, "Deleted %s", filename);
+	    LOG(log_debug9, logtype_afpd, "Deleted %s", filename);
         }
 
-        LOG(log_debug9, logtype_afpd, "Set server message to \"%s\"", servermesg);
-#endif
+	LOG(log_debug9, logtype_afpd, "Set server message to \"%s\"", servermesg);
+
     }
     free(filename);
 #endif /* SERVERTEXT */
