@@ -17,20 +17,7 @@
 #ifdef HAVE_STRINGS_H
 #include <strings.h>
 #endif
-/* STDC check */
-#if STDC_HEADERS
 #include <string.h>
-#else /* STDC_HEADERS */
-#ifndef HAVE_STRCHR
-#define strchr index
-#define strrchr index
-#endif /* HAVE_STRCHR */
-char *strchr (), *strrchr ();
-#ifndef HAVE_MEMCPY
-#define memcpy(d,s,n) bcopy ((s), (d), (n))
-#define memmove(d,s,n) bcopy ((s), (d), (n))
-#endif /* ! HAVE_MEMCPY */
-#endif /* STDC_HEADERS */
 #include <sys/param.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -66,8 +53,7 @@ char *strchr (), *strrchr ();
 #include "fork.h"
 #include "hash.h"
 #include "acls.h"
-
-extern int afprun(int root, char *cmd, int *outfd);
+#include "auth.h"
 
 #ifndef MIN
 #define MIN(a, b) ((a) < (b) ? (a) : (b))
@@ -99,23 +85,11 @@ static void             free_extmap(void);
 #define VOLOPT_LIMITSIZE  8  /* Limit the size of the volume */
 /* Usable slot: 9 */
 #define VOLOPT_VETO          10  /* list of veto filespec */
-#define VOLOPT_PREEXEC       11  /* preexec command */
-#define VOLOPT_ROOTPREEXEC   12  /* root preexec command */
-#define VOLOPT_POSTEXEC      13  /* postexec command */
-#define VOLOPT_ROOTPOSTEXEC  14  /* root postexec command */
 #define VOLOPT_ENCODING      15  /* mac encoding (pre OSX)*/
 #define VOLOPT_MACCHARSET    16
 #define VOLOPT_CNIDSCHEME    17
 #define VOLOPT_ADOUBLE       18  /* adouble version */
 
-#ifdef FORCE_UIDGID
-#warning UIDGID
-#include "uid.h"
-
-#define VOLOPT_FORCEUID      19  /* force uid for username x */
-#define VOLOPT_FORCEGID      20  /* force gid for group x */
-#endif /* FORCE_UIDGID */
-
 #define VOLOPT_UMASK         21
 #define VOLOPT_ALLOWED_HOSTS 22
 #define VOLOPT_DENIED_HOSTS  23
@@ -326,7 +300,7 @@ static char *volxlate(AFPObj *obj,
                 q = obj->options.server;
             } else
                 q = obj->options.hostname;
-        } else if (obj->username && is_var(p, "$u")) {
+        } else if ((strlen(obj->username) > 0) && is_var(p, "$u")) {
             if (afpmaster && xlatevolname)
                 return NULL;
             char* sep = NULL;
@@ -410,9 +384,7 @@ static void volset(struct vol_option *options, struct vol_option *save,
         volname[vlen] = 0;
         return;
     }
-#if 0
     LOG(log_debug, logtype_afpd, "Parsing volset %s", val);
-#endif
     if (optionok(tmp, "allow:", val)) {
         setoption(options, save, VOLOPT_ALLOW, val);
 
@@ -452,10 +424,6 @@ static void volset(struct vol_option *options, struct vol_option *save,
 #if AD_VERSION == AD_VERSION2
         else if (strcasecmp(val + 1, "v2") == 0)
             options[VOLOPT_ADOUBLE].i_value = AD_VERSION2;
-        else if (strcasecmp(val + 1, "osx") == 0)
-            options[VOLOPT_ADOUBLE].i_value = AD_VERSION2_OSX;
-        else if (strcasecmp(val + 1, "sfm") == 0)
-            options[VOLOPT_ADOUBLE].i_value = AD_VERSION1_SFM;
 #endif
     } else if (optionok(tmp, "options:", val)) {
         char *p;
@@ -482,19 +450,8 @@ static void volset(struct vol_option *options, struct vol_option *save,
                 options[VOLOPT_FLAGS].i_value |= AFPVOL_USEDOTS | AFPVOL_INV_DOTS;
             else if (strcasecmp(p, "limitsize") == 0)
                 options[VOLOPT_FLAGS].i_value |= AFPVOL_LIMITSIZE;
-            /* support for either "dropbox" or "dropkludge" */
-            else if (strcasecmp(p, "dropbox") == 0)
-                options[VOLOPT_FLAGS].i_value |= AFPVOL_DROPBOX;
-            else if (strcasecmp(p, "dropkludge") == 0)
-                options[VOLOPT_FLAGS].i_value |= AFPVOL_DROPBOX;
             else if (strcasecmp(p, "nofileid") == 0)
                 options[VOLOPT_FLAGS].i_value |= AFPVOL_NOFILEID;
-            else if (strcasecmp(p, "nostat") == 0)
-                options[VOLOPT_FLAGS].i_value |= AFPVOL_NOSTAT;
-            else if (strcasecmp(p, "preexec_close") == 0)
-                options[VOLOPT_PREEXEC].i_value = 1;
-            else if (strcasecmp(p, "root_preexec_close") == 0)
-                options[VOLOPT_ROOTPREEXEC].i_value = 1;
             else if (strcasecmp(p, "upriv") == 0)
                 options[VOLOPT_FLAGS].i_value |= AFPVOL_UNIX_PRIV;
             else if (strcasecmp(p, "nodev") == 0)
@@ -513,6 +470,8 @@ static void volset(struct vol_option *options, struct vol_option *save,
                 options[VOLOPT_FLAGS].i_value |= AFPVOL_NONETIDS;
             else if (strcasecmp(p, "noacls") == 0)
                 options[VOLOPT_FLAGS].i_value &= ~AFPVOL_ACLS;
+            else if (strcasecmp(p, "followsymlinks") == 0)
+                options[VOLOPT_FLAGS].i_value |= AFPVOL_FOLLOWSYM;
             p = strtok(NULL, ",");
         }
 
@@ -542,27 +501,6 @@ static void volset(struct vol_option *options, struct vol_option *save,
     } else if (optionok(tmp, "password:", val)) {
         setoption(options, save, VOLOPT_PASSWORD, val);
 
-#ifdef FORCE_UIDGID
-
-        /* this code allows forced uid/gid per volume settings */
-    } else if (optionok(tmp, "forceuid:", val)) {
-        setoption(options, save, VOLOPT_FORCEUID, val);
-    } else if (optionok(tmp, "forcegid:", val)) {
-        setoption(options, save, VOLOPT_FORCEGID, val);
-
-#endif /* FORCE_UIDGID */
-    } else if (optionok(tmp, "root_preexec:", val)) {
-        setoption(options, save, VOLOPT_ROOTPREEXEC, val);
-
-    } else if (optionok(tmp, "preexec:", val)) {
-        setoption(options, save, VOLOPT_PREEXEC, val);
-
-    } else if (optionok(tmp, "root_postexec:", val)) {
-        setoption(options, save, VOLOPT_ROOTPOSTEXEC, val);
-
-    } else if (optionok(tmp, "postexec:", val)) {
-        setoption(options, save, VOLOPT_POSTEXEC, val);
-
     } else if (optionok(tmp, "allowed_hosts:", val)) {
         setoption(options, save, VOLOPT_ALLOWED_HOSTS, val);
 
@@ -602,8 +540,7 @@ static void showvol(const ucs2_t *name)
 /* ------------------------------- */
 static int creatvol(AFPObj *obj, struct passwd *pwd,
                     char *path, char *name,
-                    struct vol_option *options,
-                    const int user /* user defined volume */
+                    struct vol_option *options
     )
 {
     struct vol  *volume;
@@ -628,7 +565,7 @@ static int creatvol(AFPObj *obj, struct passwd *pwd,
 
     /* suffix for mangling use (lastvid + 1)   */
     /* because v_vid has not been decided yet. */
-    suffixlen = sprintf(suffix, "%c%X", MANGLE_CHAR, lastvid + 1 );
+    suffixlen = snprintf(suffix, sizeof(suffix), "%c%X", MANGLE_CHAR, lastvid + 1);
 
     vlen = strlen( name );
 
@@ -701,13 +638,12 @@ static int creatvol(AFPObj *obj, struct passwd *pwd,
         if ((utf8_encoding() && (strcasecmp_w(volume->v_u8mname, u8mtmpname) == 0))
              ||
             (!utf8_encoding() && (strcasecmp_w(volume->v_macname, mactmpname) == 0))) {
-            LOG (log_error, logtype_afpd,
-                 "Duplicate volume name, check AppleVolumes files: previous: \"%s\", new: \"%s\"",
-                 volume->v_localname, name);
             if (volume->v_deleted) {
                 volume->v_new = hide = 1;
-            }
-            else {
+            } else {
+                LOG (log_error, logtype_afpd,
+                     "Duplicate volume name, check AppleVolumes files: previous: \"%s\", new: \"%s\"",
+                     volume->v_localname, name);
                 return -1;  /* Won't be able to access it, anyway... */
             }
         }
@@ -778,6 +714,8 @@ static int creatvol(AFPObj *obj, struct passwd *pwd,
             volume->v_ad_options |= ADVOL_INVDOTS;
         if ((volume->v_flags & AFPVOL_NOADOUBLE))
             volume->v_ad_options |= ADVOL_NOADOUBLE;
+        if ((volume->v_flags & AFPVOL_FOLLOWSYM))
+            volume->v_ad_options |= ADVOL_FOLLO_SYML;
 
         if (options[VOLOPT_PASSWORD].c_value)
             volume->v_password = strdup(options[VOLOPT_PASSWORD].c_value);
@@ -847,34 +785,6 @@ static int creatvol(AFPObj *obj, struct passwd *pwd,
         if ((volume->v_flags & AFPVOL_EILSEQ))
             volume->v_utom_flags |= CONV__EILSEQ;
 
-#ifdef FORCE_UIDGID
-        if (options[VOLOPT_FORCEUID].c_value) {
-            volume->v_forceuid = strdup(options[VOLOPT_FORCEUID].c_value);
-        } else {
-            volume->v_forceuid = NULL; /* set as null so as to return 0 later on */
-        }
-
-        if (options[VOLOPT_FORCEGID].c_value) {
-            volume->v_forcegid = strdup(options[VOLOPT_FORCEGID].c_value);
-        } else {
-            volume->v_forcegid = NULL; /* set as null so as to return 0 later on */
-        }
-#endif
-        if (!user) {
-            if (options[VOLOPT_PREEXEC].c_value)
-                volume->v_preexec = volxlate(obj, NULL, MAXPATHLEN, options[VOLOPT_PREEXEC].c_value, pwd, path, name);
-            volume->v_preexec_close = options[VOLOPT_PREEXEC].i_value;
-
-            if (options[VOLOPT_POSTEXEC].c_value)
-                volume->v_postexec = volxlate(obj, NULL, MAXPATHLEN, options[VOLOPT_POSTEXEC].c_value, pwd, path, name);
-
-            if (options[VOLOPT_ROOTPREEXEC].c_value)
-                volume->v_root_preexec = volxlate(obj, NULL, MAXPATHLEN, options[VOLOPT_ROOTPREEXEC].c_value, pwd, path,  name);
-            volume->v_root_preexec_close = options[VOLOPT_ROOTPREEXEC].i_value;
-
-            if (options[VOLOPT_ROOTPOSTEXEC].c_value)
-                volume->v_root_postexec = volxlate(obj, NULL, MAXPATHLEN, options[VOLOPT_ROOTPOSTEXEC].c_value, pwd, path,  name);
-        }
     }
     volume->v_dperm |= volume->v_perm;
     volume->v_fperm |= volume->v_perm;
@@ -1066,8 +976,12 @@ static void setextmap(char *ext, char *type, char *creator, int user)
     }
 
     if ( em->em_ext == NULL ) {
-        if (!(Extmap  = realloc( Extmap, sizeof( struct extmap ) * (cnt +2))) ) {
+        void *Extmap_check = realloc(Extmap, sizeof(struct extmap) * (cnt + 2));
+        if (Extmap_check) {
+            Extmap = Extmap_check;
+        } else {
             LOG(log_error, logtype_afpd, "setextmap: realloc: %s", strerror(errno) );
+            free(Extmap);
             return;
         }
         (Extmap +cnt +1)->em_ext = NULL;
@@ -1295,7 +1209,7 @@ static int readvolfile(AFPObj *obj, struct afp_volume_name *p1, char *p2, int us
             /* send path through variable substitution */
             if (*path != '~') /* need to copy path to tmp */
                 strcpy(tmp, path);
-            if (!pwent && obj->username)
+            if (!pwent && (strlen(obj->username) > 0))
                 pwent = getpwnam(obj->username);
 
             if (volxlate(obj, path, sizeof(path) - 1, tmp, pwent, NULL, NULL) == NULL)
@@ -1342,7 +1256,7 @@ static int readvolfile(AFPObj *obj, struct afp_volume_name *p1, char *p2, int us
                 if (volxlate(obj, tmp, sizeof(tmp) - 1, volname, pwent, path, NULL) == NULL)
                     continue;
 
-                creatvol(obj, pwent, path, tmp, options, p2 != NULL);
+                creatvol(obj, pwent, path, tmp, options);
             }
             volfree(options, default_options);
             break;
@@ -1383,10 +1297,6 @@ static void volume_free(struct vol *vol)
     free(vol->v_cnidscheme);
     free(vol->v_dbpath);
     free(vol->v_gvs);
-#ifdef FORCE_UIDGID
-    free(vol->v_forceuid);
-    free(vol->v_forcegid);
-#endif /* FORCE_UIDGID */
     if (vol->v_uuid)
         free(vol->v_uuid);
 }
@@ -1477,6 +1387,8 @@ EC_CLEANUP:
     if (file)
         fclose(file);
     LOG(log_debug, logtype_afpd, "get_tm_bandsize(\"%s\"): bandsize: %lld", path, bandsize);
+    if (ret != 0)
+        return -1;
     return bandsize;
 }
 
@@ -1540,7 +1452,6 @@ static int get_tm_used(struct vol * restrict vol)
     DIR *dir = NULL;
     const struct dirent *entry;
     const char *p;
-    struct stat st;
     long int links;
     time_t now = time(NULL);
 
@@ -1598,28 +1509,18 @@ static int getvolspace(struct vol *vol,
                        u_int32_t *bfree, u_int32_t *btotal,
                        VolSpace *xbfree, VolSpace *xbtotal, u_int32_t *bsize)
 {
-    int         spaceflag, rc;
+    int         rc;
     u_int32_t   maxsize;
-    VolSpace    used;
 #ifndef NO_QUOTA_SUPPORT
     VolSpace    qfree, qtotal;
+    int         spaceflag = AFPVOL_GVSMASK & vol->v_flags;
 #endif
 
-    spaceflag = AFPVOL_GVSMASK & vol->v_flags;
     /* report up to 2GB if afp version is < 2.2 (4GB if not) */
     maxsize = (vol->v_flags & AFPVOL_A2VOL) ? 0x01fffe00 :
         (((afp_version < 22) || (vol->v_flags & AFPVOL_LIMITSIZE))
          ? 0x7fffffffL : 0xffffffffL);
 
-#ifdef AFS
-    if ( spaceflag == AFPVOL_NONE || spaceflag == AFPVOL_AFSGVS ) {
-        if ( afs_getvolspace( vol, xbfree, xbtotal, bsize ) == AFP_OK ) {
-            vol->v_flags = ( ~AFPVOL_GVSMASK & vol->v_flags ) | AFPVOL_AFSGVS;
-            goto getvolspace_done;
-        }
-    }
-#endif
-
     if (( rc = ustatfs_getvolspace( vol, xbfree, xbtotal, bsize)) != AFP_OK ) {
         return( rc );
     }
@@ -1696,13 +1597,13 @@ static void vol_setdate(u_int16_t id, struct adouble *adp, time_t date)
 static int getvolparams( u_int16_t bitmap, struct vol *vol, struct stat *st, char *buf, size_t *buflen)
 {
     struct adouble  ad;
-    int         bit = 0, isad = 1;
-    u_int32_t       aint;
-    u_short     ashort;
-    u_int32_t       bfree, btotal, bsize;
-    VolSpace            xbfree, xbtotal; /* extended bytes */
-    char        *data, *nameoff = NULL;
-    char                *slash;
+    int bit = 0, isad = 1;
+    u_int32_t aint;
+    u_int16_t ashort;
+    u_int32_t bfree = 0, btotal = 0, bsize = 0;
+    VolSpace xbfree = 0, xbtotal = 0; /* extended bytes */
+    char *data = NULL, *nameoff = NULL;
+    char *slash = NULL;
 
     LOG(log_debug, logtype_afpd, "getvolparams: Volume '%s'", vol->v_localname);
 
@@ -1843,21 +1744,13 @@ static int getvolparams( u_int16_t bitmap, struct vol *vol, struct stat *st, cha
 #ifndef NO_LARGE_VOL_SUPPORT
         case VOLPBIT_XBFREE :
             xbfree = hton64( xbfree );
-#if defined(__GNUC__) && defined(HAVE_GCC_MEMCPY_BUG)
-            bcopy(&xbfree, data, sizeof(xbfree));
-#else /* __GNUC__ && HAVE_GCC_MEMCPY_BUG */
             memcpy(data, &xbfree, sizeof( xbfree ));
-#endif /* __GNUC__ && HAVE_GCC_MEMCPY_BUG */
             data += sizeof( xbfree );
             break;
 
         case VOLPBIT_XBTOTAL :
             xbtotal = hton64( xbtotal );
-#if defined(__GNUC__) && defined(HAVE_GCC_MEMCPY_BUG)
-            bcopy(&xbtotal, data, sizeof(xbtotal));
-#else /* __GNUC__ && HAVE_GCC_MEMCPY_BUG */
             memcpy(data, &xbtotal, sizeof( xbtotal ));
-#endif /* __GNUC__ && HAVE_GCC_MEMCPY_BUG */
             data += sizeof( xbfree );
             break;
 #endif /* ! NO_LARGE_VOL_SUPPORT */
@@ -1932,7 +1825,8 @@ static int stat_vol(u_int16_t bitmap, struct vol *vol, char *rbuf, size_t *rbufl
 /* ------------------------------- */
 void load_volumes(AFPObj *obj)
 {
-    struct passwd   *pwent;
+    int ret;
+    struct passwd *pwent = NULL;
 
     if (Volumes) {
         int changed = 0;
@@ -1958,9 +1852,20 @@ void load_volumes(AFPObj *obj)
         LOG(log_debug, logtype_afpd, "load_volumes: AFP MASTER");
     } else {
         LOG(log_debug, logtype_afpd, "load_volumes: user: %s", obj->username);
+        if ((pwent = getpwnam(obj->username))) {
+            if (seteuid(0) < 0)
+                LOG(log_error, logtype_afpd, "could not seteuid(%i)", 0);
+            ret = set_groups(obj, pwent);
+            if (seteuid(obj->uid) < 0)
+                LOG(log_error, logtype_afpd, "could not seteuid(%i)", obj->uid);
+            if (ret != 0) {
+                LOG(log_error, logtype_afpd, "load_volumes: set_groups: %s", strerror(errno));
+                return;
+            }
+        }
+        LOG(log_debug, logtype_afpd, "login: supplementary groups: %s", print_groups(ngroups, groups));
     }
 
-    pwent = getpwnam(obj->username);
     if ( (obj->options.flags & OPTION_USERVOLFIRST) == 0 ) {
         readvolfile(obj, &obj->options.systemvol, NULL, 0, pwent);
     }
@@ -1983,7 +1888,7 @@ void load_volumes(AFPObj *obj)
              readvolfile(obj, &obj->options.uservol, ".applevolumes", 1, pwent) < 0 &&
              obj->options.defaultvol.name != NULL ) {
             if (readvolfile(obj, &obj->options.defaultvol, NULL, 1, pwent) < 0)
-                creatvol(obj, pwent, pwent->pw_dir, NULL, NULL, 1);
+                creatvol(obj, pwent, pwent->pw_dir, NULL, NULL);
         }
     }
     if ( obj->options.flags & OPTION_USERVOLFIRST ) {
@@ -2017,8 +1922,8 @@ int afp_getsrvrparms(AFPObj *obj, char *ibuf _U_, size_t ibuflen _U_, char *rbuf
     load_volumes(obj);
 
     data = rbuf + 5;
-    for ( vcnt = 0, volume = Volumes; volume; volume = volume->v_next ) {
-        if (!(volume->v_flags & AFPVOL_NOSTAT)) {
+    for ( vcnt = 0, volume = Volumes; volume && vcnt < 255; volume = volume->v_next ) {
+        if (!(volume->v_flags)) {
             struct maccess ma;
 
             if ( stat( volume->v_path, &st ) < 0 ) {
@@ -2047,6 +1952,14 @@ int afp_getsrvrparms(AFPObj *obj, char *ibuf _U_, size_t ibuflen _U_, char *rbuf
         if (len == (size_t)-1)
             continue;
 
+        /*
+         * There seems to be an undocumented limit on how big our reply can get
+         * before the client chokes and closes the connection.
+         * Testing with 10.8.4 found the limit at ~4600 bytes. Go figure. 
+         */
+        if (((data + len + 3) - rbuf) > 4600)
+            break;
+
         /* set password bit if there's a volume password */
         *data = (volume->v_password) ? AFPSRVR_PASSWD : 0;
 
@@ -2132,28 +2045,6 @@ static int volume_openDB(struct vol *volume)
         volume->v_cnidserver ? volume->v_cnidserver : Cnid_srv,
         volume->v_cnidport ? volume->v_cnidport : Cnid_port);
 
-#if 0
-/* Found this in branch dir-rewrite, maybe we want to use it sometimes */
-
-    /* Legacy pre 2.1 way of sharing eg CD-ROM */
-    if (strcmp(volume->v_cnidscheme, "last") == 0) {
-        /* "last" is gone. We support it by switching to in-memory "tdb" */
-        volume->v_cnidscheme = strdup("tdb");
-        flags |= CNID_FLAG_MEMORY;
-    }
-
-    /* New way of sharing CD-ROM */
-    if (volume->v_flags & AFPVOL_CDROM) {
-        flags |= CNID_FLAG_MEMORY;
-        if (strcmp(volume->v_cnidscheme, "tdb") != 0) {
-            free(volume->v_cnidscheme);
-            volume->v_cnidscheme = strdup("tdb");
-            LOG(log_info, logtype_afpd, "Volume %s is ejectable, switching to scheme %s.",
-                volume->v_path, volume->v_cnidscheme);
-        }
-    }
-#endif
-
     volume->v_cdb = cnid_open(volume->v_path,
                               volume->v_umask,
                               volume->v_cnidscheme,
@@ -2161,24 +2052,10 @@ static int volume_openDB(struct vol *volume)
                               volume->v_cnidserver ? volume->v_cnidserver : Cnid_srv,
                               volume->v_cnidport ? volume->v_cnidport : Cnid_port);
 
-    if ( ! volume->v_cdb && ! (flags & CNID_FLAG_MEMORY)) {
-        /* The first attempt failed and it wasn't yet an attempt to open in-memory */
+    if ( ! volume->v_cdb ) {
         LOG(log_error, logtype_afpd, "Can't open volume \"%s\" CNID backend \"%s\" ",
             volume->v_path, volume->v_cnidscheme);
-        LOG(log_error, logtype_afpd, "Reopen volume %s using in memory temporary CNID DB.",
-            volume->v_path);
-        flags |= CNID_FLAG_MEMORY;
-        volume->v_cdb = cnid_open (volume->v_path, volume->v_umask, "tdb", flags, NULL, NULL);
-#ifdef SERVERTEXT
-        /* kill ourself with SIGUSR2 aka msg pending */
-        if (volume->v_cdb) {
-            setmessage("Something wrong with the volume's CNID DB, using temporary CNID DB instead."
-                       "Check server messages for details!");
-            kill(getpid(), SIGUSR2);
-            /* deactivate cnid caching/storing in AppleDouble files */
-            volume->v_flags &= ~AFPVOL_CACHE;
-        }
-#endif
+        return -1;
     }
 
     return (!volume->v_cdb)?-1:0;
@@ -2193,8 +2070,9 @@ static int volume_openDB(struct vol *volume)
 static void check_ea_sys_support(struct vol *vol)
 {
     uid_t process_uid = 0;
-    char eaname[] = {"org.netatalk.supports-eas.XXXXXX"};
+    char eaname[] = {"/tmp/org.netatalk.supports-eas.XXXXXX"};
     const char *eacontent = "yes";
+    int fd;
 
     if (vol->v_vfs_ea == AFPVOL_EA_AUTO) {
 
@@ -2204,12 +2082,14 @@ static void check_ea_sys_support(struct vol *vol)
             return;
         }
 
-        mktemp(eaname);
+        fd = mkstemp(eaname);
+        close(fd);
 
         process_uid = geteuid();
         if (process_uid)
             if (seteuid(0) == -1) {
                 LOG(log_error, logtype_afpd, "check_ea_sys_support: can't seteuid(0): %s", strerror(errno));
+                unlink(eaname);
                 exit(EXITERR_SYS);
             }
 
@@ -2225,9 +2105,12 @@ static void check_ea_sys_support(struct vol *vol)
         if (process_uid) {
             if (seteuid(process_uid) == -1) {
                 LOG(log_error, logtype_afpd, "can't seteuid back %s", strerror(errno));
+                unlink(eaname);
                 exit(EXITERR_SYS);
             }
         }
+
+        unlink(eaname);
     }
 }
 
@@ -2289,7 +2172,7 @@ int afp_openvol(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf, size_t
     }
 
     if ( volume == NULL ) {
-        return AFPERR_PARAM;
+        return AFPERR_NOOBJ;
     }
 
     /* check for a volume password */
@@ -2299,29 +2182,9 @@ int afp_openvol(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf, size_t
 
     if (( volume->v_flags & AFPVOL_OPEN  ) ) {
         /* the volume is already open */
-#ifdef FORCE_UIDGID
-        set_uidgid ( volume );
-#endif
         return stat_vol(bitmap, volume, rbuf, rbuflen);
     }
 
-    if (volume->v_root_preexec) {
-        if ((ret = afprun(1, volume->v_root_preexec, NULL)) && volume->v_root_preexec_close) {
-            LOG(log_error, logtype_afpd, "afp_openvol(%s): root preexec : %d", volume->v_path, ret );
-            return AFPERR_MISC;
-        }
-    }
-
-#ifdef FORCE_UIDGID
-    set_uidgid ( volume );
-#endif
-
-    if (volume->v_preexec) {
-        if ((ret = afprun(0, volume->v_preexec, NULL)) && volume->v_preexec_close) {
-            LOG(log_error, logtype_afpd, "afp_openvol(%s): preexec : %d", volume->v_path, ret );
-            return AFPERR_MISC;
-        }
-    }
 
     if ( stat( volume->v_path, &st ) < 0 ) {
         return AFPERR_PARAM;
@@ -2477,13 +2340,6 @@ static void closevol(struct vol *vol)
         cnid_close(vol->v_cdb);
         vol->v_cdb = NULL;
     }
-
-    if (vol->v_postexec) {
-        afprun(0, vol->v_postexec, NULL);
-    }
-    if (vol->v_root_postexec) {
-        afprun(1, vol->v_root_postexec, NULL);
-    }
 }
 
 /* ------------------------- */
@@ -2559,9 +2415,6 @@ struct vol *getvolbyvid(const u_int16_t vid )
         return( NULL );
     }
 
-#ifdef FORCE_UIDGID
-    set_uidgid ( vol );
-#endif /* FORCE_UIDGID */
 
     current_vol = vol;
 
@@ -2954,11 +2807,16 @@ static char *get_vol_uuid(const AFPObj *obj, const char *volname)
         fprintf(fp, "# and stores UUIDs for TM volumes.\n\n");
     } else {
         fseek(fp, -1L, SEEK_END);
-        if(fgetc(fp) != '\n') fputc('\n', fp); /* last char is \n? */
+        if (fgetc(fp) != '\n')
+        {
+            /* Required - C11 7.21.5.3p7 */
+            fseek(fp, 0L, SEEK_END);
+            fputc('\n', fp); /* last char is \n? */
+        }
     }                    
     
     /* generate uuid and write to file */
-    atalk_uuid_t id;
+    atalk_uuid_t id = "\0";
     const char *cp;
     randombytes((void *)id, 16);
     cp = uuid_bin2string(id);
