@@ -1,5 +1,5 @@
 /*
- * $Id: config.c,v 1.13.6.5.2.6 2008/12/03 19:17:27 didg Exp $
+ * $Id: config.c,v 1.20 2009-10-29 11:35:58 didg Exp $
  *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * All Rights Reserved. See COPYRIGHT.
@@ -15,10 +15,6 @@
 #include <sys/ioctl.h>
 #include <atalk/logger.h>
 #include <sys/param.h>
-#ifdef TRU64
-#include <sys/mbuf.h>
-#include <net/route.h>
-#endif /* TRU64 */
 #include <net/if.h>
 #include <netatalk/at.h>
 #include <netatalk/endian.h>
@@ -30,21 +26,7 @@
 #include <string.h>
 #include <errno.h>
 #include <ctype.h>
-
-/* STDC check */
-#if STDC_HEADERS
 #include <string.h>
-#else /* STDC_HEADERS */
-#ifndef HAVE_STRCHR
-#define strchr index
-#define strrchr index
-#endif /* HAVE_STRCHR */
-char *strchr (), *strrchr ();
-#ifndef HAVE_MEMCPY
-#define memcpy(d,s,n) bcopy ((s), (d), (n))
-#define memmove(d,s,n) bcopy ((s), (d), (n))
-#endif /* ! HAVE_MEMCPY */
-#endif /* STDC_HEADERS */
 
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
@@ -67,11 +49,18 @@ char *strchr (), *strrchr ();
 #define IFF_SLAVE 0
 #endif /* IFF_SLAVE */
 
-int	router(), dontroute(), seed(), phase(), net(), addr(), zone(), noallmulti();
+int router(struct interface *iface, char **av);
+int dontroute(struct interface *iface, char **av);
+int seed(struct interface *iface, char **av);
+int phase(struct interface *iface, char **av); 
+int net(struct interface *iface, char **av);
+int addr(struct interface *iface, char **av); 
+int zone(struct interface *iface, char **av);
+int noallmulti(struct interface *iface, char **av);
 
 static const struct param {
     char	*p_name;
-    int		(*p_func)();
+    int		(*p_func)(struct interface *iface, char **av);
 } params[] = {
     { "router", router },
     { "dontroute", dontroute },
@@ -85,7 +74,7 @@ static const struct param {
 
 #define ARGV_CHUNK_SIZE 128
 #define MAXLINELEN 2048
-char **parseline(const char *line)
+static char **parseline(const char *line)
 {
     const char	 *p;
     int		  argc = 0;
@@ -165,7 +154,7 @@ char **parseline(const char *line)
     return argv;
 }
 
-void freeline( char **argv )
+static void freeline( char **argv )
 {
     char **tmp = argv;
 
@@ -178,8 +167,7 @@ void freeline( char **argv )
     }
 }
 
-int writeconf( cf )
-    char	*cf;
+int writeconf(char *cf)
 {
     struct stat		st;
     char		*path, *p, newpath[ MAXPATHLEN ], line[ MAXLINELEN ];
@@ -321,8 +309,7 @@ int writeconf( cf )
  * zone for an interface is the first zone encountered for that
  * interface.
  */
-int readconf( cf )
-    char		*cf;
+int readconf(char *cf)
 {
     struct ifreq	ifr;
     struct interface	*iface, *niface;
@@ -474,9 +461,7 @@ read_conf_err:
     return -1;
 }
 
-int noallmulti( iface, av )
-    struct interface	*iface;
-    char		**av _U_;
+int noallmulti( struct interface *iface, char **av _U_)
 {
     /* Linux specific, no effect on other platforms */
     iface->i_flags &= !IFACE_ALLMULTI;
@@ -485,9 +470,7 @@ int noallmulti( iface, av )
 }
 	
 /*ARGSUSED*/
-int router( iface, av )
-    struct interface	*iface;
-    char		**av _U_;
+int router(struct interface *iface, char **av _U_)
 {
     /* make sure "-router" and "-dontroute" aren't both on the same line. */
     if (iface->i_flags & IFACE_DONTROUTE) {
@@ -509,9 +492,7 @@ int router( iface, av )
 }
 
 /*ARGSUSED*/
-int dontroute( iface, av )
-    struct interface	*iface;
-    char		**av _U_;
+int dontroute(struct interface *iface, char **av _U_)
 {
     /* make sure "-router" and "-dontroute" aren't both on the same line. */
     if (iface->i_flags & IFACE_RSEED) {
@@ -524,9 +505,7 @@ int dontroute( iface, av )
 }
 
 /*ARGSUSED*/
-int seed( iface, av )
-    struct interface	*iface;
-    char		**av _U_;
+int seed( struct interface *iface, char **av _U_)
 {
     /*
      * Check to be sure "-seed" is before "-zone". we keep the old
@@ -542,9 +521,7 @@ int seed( iface, av )
     return( 1 );
 }
 
-int phase( iface, av )
-    struct interface	*iface;
-    char		**av;
+int phase(struct interface *iface, char **av)
 {
     int			n;
     char		*pnum;
@@ -570,9 +547,7 @@ int phase( iface, av )
     return( 2 );
 }
 
-int net( iface, av )
-    struct interface	*iface;
-    char		**av;
+int net(struct interface *iface, char **av)
 {
     char		*nrange;
     char		*stop;
@@ -583,7 +558,7 @@ int net( iface, av )
 	return -1;
     }
 
-    if (( stop = strchr( nrange, '-' )) != 0 ) {
+    if (( stop = strchr( nrange, '-' )) != NULL ) {
 	stop++;
     }
     net = atoi( nrange );
@@ -598,7 +573,7 @@ int net( iface, av )
     }
 
     if ( iface->i_flags & IFACE_PHASE1 ) {
-	if ( stop != 0 ) {
+	if ( stop != NULL ) {
 	    fprintf( stderr, "Phase 1 doesn't use an address range.\n" );
 	    return -1;
 	}
@@ -611,7 +586,7 @@ int net( iface, av )
 	iface->i_rt->rt_firstnet = iface->i_rt->rt_lastnet = htons( net );
     } else if ( iface->i_flags & IFACE_PHASE2 ) {
 	iface->i_rt->rt_firstnet = htons( net );
-	if ( stop != 0 ) {
+	if ( stop != NULL ) {
 	    net = atoi( stop );
 	    if ( net < 0 || net >= 0xffff ) {
 		fprintf( stderr, "Bad network: %d\n", net );
@@ -640,9 +615,7 @@ int net( iface, av )
     return( 2 );
 }
 
-int addr( iface, av )
-    struct interface	*iface;
-    char		**av;
+int addr(struct interface *iface, char **av)
 {
     if ( av[ 0 ] == NULL ) {
 	fprintf( stderr, "No address.\n" );
@@ -676,9 +649,7 @@ int addr( iface, av )
     return( 2 );
 }
 
-int zone( iface, av )
-    struct interface	*iface;
-    char		**av;
+int zone(struct interface *iface, char **av)
 {
     struct ziptab	*zt;
     char		*zname;
@@ -689,7 +660,7 @@ int zone( iface, av )
     }
 
     /* codepage conversion */
-    if ((size_t)(-1) == convert_string_allocate(CH_UNIX, CH_MAC, av[0], strlen(av[0]), &zname)) {
+    if ((size_t)(-1) == convert_string_allocate(CH_UNIX, CH_MAC, av[0], -1, &zname)) {
 	zname = strdup(av[0]);
     }
 
@@ -724,7 +695,7 @@ int zone( iface, av )
  * Get the configuration from the kernel. Only called if there's no
  * configuration.
  */
-int getifconf()
+int getifconf(void)
 {
     struct interface	*iface, *niface;
     struct ifreq        ifr;
@@ -804,8 +775,7 @@ int getifconf()
  * the interface structure and have it updated nicely.
  */
 
-struct interface *newiface( name )
-    const char		*name;
+struct interface *newiface( const char *name)
 {
     struct interface	*niface;
 
@@ -816,11 +786,9 @@ struct interface *newiface( name )
     strlcpy( niface->i_name, name, sizeof(niface->i_name));
 #ifdef BSD4_4
     niface->i_addr.sat_len = sizeof( struct sockaddr_at );
-#endif /* BSD4_4 */
-    niface->i_addr.sat_family = AF_APPLETALK;
-#ifdef BSD4_4
     niface->i_caddr.sat_len = sizeof( struct sockaddr_at );
 #endif /* BSD4_4 */
+    niface->i_addr.sat_family = AF_APPLETALK;
     niface->i_caddr.sat_family = AF_APPLETALK;
 #ifdef linux
     niface->i_flags = IFACE_ALLMULTI;
@@ -829,7 +797,7 @@ struct interface *newiface( name )
 }
 
 #ifdef __svr4__
-int plumb()
+int plumb(void)
 {
     struct interface	*iface;
     char		device[ MAXPATHLEN + 1], *p, *t;
@@ -883,6 +851,8 @@ int plumb()
 	}
 
 	LOG(log_info, logtype_atalkd, "plumbed %s%d", device, ppa );
+
+    close(fd);
     }
 
     return( 0 );
