@@ -1,6 +1,4 @@
 /*
- * $Id: main.c,v 1.17.8.6.2.3 2009/01/19 02:25:57 didg Exp $
- *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * All Rights Reserved. See COPYRIGHT.
  */
@@ -16,26 +14,10 @@
 #include <sys/resource.h>
 #include <sys/ioctl.h>
 
-/* POSIX.1 check */
 #include <sys/types.h>
-#ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
-#endif /* HAVE_SYS_WAIT_H */
-#ifndef WEXITSTATUS 
-#define WEXITSTATUS(stat_val) ((unsigned)(stat_val) >> 8)
-#endif /* ! WEXITSTATUS */
-#ifndef WIFEXITED
-#define WIFEXITED(stat_val) (((stat_val) & 255) == 0)
-#endif /* ! WIFEXITED */
-#ifndef WIFSTOPPED
-#define WIFSTOPPED(status) (((status) & 0xff) == 0x7f)
-#endif
 
 #include <errno.h>
-#ifdef TRU64
-#include <sys/mbuf.h>
-#include <net/route.h>
-#endif /* TRU64 */
 #include <net/if.h>
 #include <net/route.h>
 
@@ -55,6 +37,7 @@
 #include <atalk/compat.h>
 #include <atalk/zip.h>
 #include <atalk/rtmp.h>
+#include <atalk/nbp.h>
 #include <atalk/ddp.h>
 #include <atalk/atp.h>
 #include <atalk/paths.h>
@@ -70,39 +53,22 @@
 #include "list.h"
 #include "rtmp.h"
 #include "zip.h"
+#include "nbp.h"
 #include "atserv.h"
 #include "main.h"
 
 /* Forward Declarations */
 int ifconfig(const char *iname, unsigned long cmd, struct sockaddr_at *sa);
 
-/* FIXME/SOCKLEN_T: socklen_t is a unix98 feature */
-#ifndef SOCKLEN_T
-#define SOCKLEN_T unsigned int
-#endif /* SOCKLEN_T */
-
-#ifndef WEXITSTATUS
-#define WEXITSTATUS(x)	((x).w_retcode)
-#endif /* WEXITSTATUS */
-
-/* linux has a special ioctl for appletalk device destruction.  as of
- * 2.1.57, SIOCDIFADDR works w/ linux. okay, we need to deal with the
- * fact that SIOCDIFADDR may be defined on linux despite the fact that
- * it doesn't work. */
-#if !defined(SIOCDIFADDR) && defined(SIOCATALKDIFADDR)
-#define SIOCDIFADDR SIOCATALKDIFADDR
-#endif
-
 #define elements(a)	(sizeof(a)/sizeof((a)[0]))
 
 #define PKTSZ	1024
 
-extern int	rtmp_packet();
-extern int	nbp_packet();
-extern int	aep_packet();
-extern int	zip_packet();
+extern int aep_packet(struct atport *ap, struct sockaddr_at *from, char *data, int len);
 
 int		rtfd;
+int 		transition = 0;
+int		stabletimer, newrtmpdata = 0;
 
 static struct atserv	atserv[] = {
     { "rtmp",		1,	rtmp_packet },		/* 0 */
@@ -114,17 +80,17 @@ static int		atservNATSERV = elements( atserv );
 
 struct interface	*interfaces = NULL, *ciface = NULL;
 
-int		debug = 0, quiet = 0, chatty = 0;
-char		*configfile = NULL;
-int		ziptimeout = 0, transition = 0;
-int		stabletimer, stable = 0, newrtmpdata = 0, noparent = 0;
-static int	ninterfaces;
-int		defphase = IFACE_PHASE2;
-int		nfds = 0;
-fd_set		fds;
-char		Packet[ PKTSZ ];
-char		*version = VERSION;
-static char     *pidfile = _PATH_ATALKDLOCK;
+int			debug = 0;
+static char		*configfile = NULL;
+static int		ziptimeout = 0;
+static int		stable = 0, noparent = 0;
+static int		ninterfaces;
+static int		defphase = IFACE_PHASE2;
+static int		nfds = 0;
+static fd_set		fds;
+static char		Packet[ PKTSZ ];
+static char		*version = VERSION;
+static char     	*pidfile = _PATH_ATALKDLOCK;
 
 
 /* from config.c */
@@ -133,6 +99,8 @@ int readconf( char * );
 int getifconf( void );
 int writeconf( char * );
 
+static void consistency( void );
+
 /* this is the messiest of the bunch as atalkd can exit pretty much
  * everywhere. we delete interfaces here instead of in as_down. */
 static void atalkd_exit(const int i)
@@ -142,12 +110,10 @@ static void atalkd_exit(const int i)
 
   for (iface = interfaces; iface; iface = iface->i_next) {
     if (ifconfig( iface->i_name, SIOCDIFADDR, &iface->i_addr)) {
-#ifdef SIOCATALKDIFADDR
-#if (SIOCDIFADDR != SIOCATALKDIFADDR)
+#if defined(__linux__)
       if (!ifconfig(iface->i_name, SIOCATALKDIFADDR, &iface->i_addr)) 
 	continue;
-#endif /* SIOCDIFADDR != SIOCATALKDIFADDR */
-#endif /* SIOCATALKIFADDR */
+#endif /* __linux__ */
       LOG(log_error, logtype_atalkd, "difaddr(%u.%u): %s", 
 	      ntohs(iface->i_addr.sat_addr.s_net), 
 	      iface->i_addr.sat_addr.s_node, strerror(errno));
@@ -157,12 +123,36 @@ static void atalkd_exit(const int i)
         ifsetallmulti(iface->i_name, 0);
 #endif /* linux */
   }
-#endif /* SOPCDOFADDR */
+#endif /* SIOCDIFADDR */
 
   server_unlock(pidfile);
   exit(i);
 }
 
+/* XXX need better error handling for gone interfaces, delete routes and so on 
+ * moreover there's no way to put an interface back short of restarting atalkd
+ * thus after the first time, silently fail
+*/
+static ssize_t sendto_iface(struct interface *iface, int sockfd, const void *buf, size_t len, 
+                       const struct sockaddr_at	 *dest_addr)
+{
+    ssize_t ret = sendto( sockfd, buf, len, 0, (struct sockaddr *)dest_addr, sizeof( struct sockaddr_at ));
+
+    if (ret < 0 ) {
+        if (!(iface->i_flags & IFACE_ERROR)) {
+            LOG(log_error, logtype_atalkd, "as_timer sendto %u.%u (%u): %s",
+				    ntohs( dest_addr->sat_addr.s_net ),
+				    dest_addr->sat_addr.s_node,
+				    ntohs( iface->i_rt->rt_firstnet ),
+				    strerror(errno) );
+        }
+        iface->i_flags |= IFACE_ERROR;
+    }
+    else {
+        iface->i_flags &= ~IFACE_ERROR;
+    }
+    return ret;
+}
 
 static void as_timer(int sig _U_)
 {
@@ -182,6 +172,7 @@ static void as_timer(int sig _U_)
     ap=zap=rap=NULL;
 
     memset(&sat, 0, sizeof( struct sockaddr_at ));
+
     for ( iface = interfaces; iface; iface = iface->i_next ) {
 	if ( iface->i_flags & IFACE_LOOPBACK ) {
 	    continue;
@@ -195,8 +186,7 @@ static void as_timer(int sig _U_)
 	    }
 	}
 
-	if (( iface->i_flags & ( IFACE_ADDR|IFACE_CONFIG|IFACE_NOROUTER )) ==
-		IFACE_ADDR ) {
+	if (( iface->i_flags & ( IFACE_ADDR|IFACE_CONFIG|IFACE_NOROUTER )) == IFACE_ADDR ) {
 	    if ( iface->i_time < 3 ) {
 		if ( iface->i_flags & IFACE_PHASE1 ) {
 		  if (rtmp_request( iface ) < 0) {
@@ -220,10 +210,8 @@ static void as_timer(int sig _U_)
 			 * No seed info, and we've got multiple interfaces.
 			 * Wait forever.
 			 */
-			LOG(log_info, logtype_atalkd,
-				"as_timer multiple interfaces, no seed" );
-			LOG(log_info, logtype_atalkd, "as_timer can't configure %s",
-				iface->i_name );
+			LOG(log_info, logtype_atalkd, "as_timer multiple interfaces, no seed" );
+			LOG(log_info, logtype_atalkd, "as_timer can't configure %s", iface->i_name );
 			LOG(log_info, logtype_atalkd, "as_timer waiting for router" );
 			iface->i_time = 0;
 			continue;
@@ -234,8 +222,7 @@ static void as_timer(int sig _U_)
 			 */
 			iface->i_flags |= IFACE_CONFIG;
 			for ( zt = iface->i_czt; zt; zt = zt->zt_next ) {
-			    if (addzone( iface->i_rt, zt->zt_len, 
-					 zt->zt_name) < 0) {
+			    if (addzone( iface->i_rt, zt->zt_len, zt->zt_name) < 0) {
 			      LOG(log_error, logtype_atalkd, "addzone: %s", strerror(errno));
 			      atalkd_exit(1);
 			    }
@@ -245,26 +232,20 @@ static void as_timer(int sig _U_)
 			    iface->i_rt->rt_flags |= RTMPTAB_HASZONES;
 			}
 			if ( iface->i_flags & IFACE_PHASE1 ) {
-			    LOG(log_info, logtype_atalkd,
-				    "as_timer configured %s phase 1 from seed",
-				    iface->i_name );
+			    LOG(log_info, logtype_atalkd, "as_timer configured %s phase 1 from seed", iface->i_name );
 			    setaddr( iface, IFACE_PHASE1,
 				    iface->i_caddr.sat_addr.s_net,
 				    iface->i_addr.sat_addr.s_node,
 				    iface->i_caddr.sat_addr.s_net,
 				    iface->i_caddr.sat_addr.s_net );
 			} else {
-			    LOG(log_info, logtype_atalkd,
-				    "as_timer configured %s phase 2 from seed",
-				    iface->i_name );
+			    LOG(log_info, logtype_atalkd, "as_timer configured %s phase 2 from seed", iface->i_name );
 			}
 
 			if ( looproute( iface, RTMP_ADD )) { /* -1 or 1 */
-			    LOG(log_error, logtype_atalkd,
-				    "as_timer: can't route %u.%u to loop: %s",
-				    ntohs( iface->i_addr.sat_addr.s_net ),
-				    iface->i_addr.sat_addr.s_node,
-				    strerror(errno) );
+			    LOG(log_error, logtype_atalkd, "as_timer: can't route %u.%u to loop: %s", 
+			            ntohs( iface->i_addr.sat_addr.s_net ),
+				    iface->i_addr.sat_addr.s_node, strerror(errno) );
 			    atalkd_exit( 1 );
 			}
 			if ( iface == ciface ) {
@@ -280,18 +261,14 @@ static void as_timer(int sig _U_)
 		    LOG(log_info, logtype_atalkd, "config for no router" );
 		      
 		    if ( iface->i_flags & IFACE_PHASE2 ) {
-			iface->i_rt->rt_firstnet = 0;
+			iface->i_rt->rt_firstnet = htons( OS_STARTUP_FIRSTNET );
 			iface->i_rt->rt_lastnet = htons( STARTUP_LASTNET );
-			setaddr( iface, IFACE_PHASE2,
-				iface->i_addr.sat_addr.s_net,
-				iface->i_addr.sat_addr.s_node,
-				0, htons( STARTUP_LASTNET ));
+			setaddr( iface, IFACE_PHASE2, iface->i_addr.sat_addr.s_net, iface->i_addr.sat_addr.s_node,
+				htons( OS_STARTUP_FIRSTNET ), htons( STARTUP_LASTNET ));
 		    }
 		    if ( looproute( iface, RTMP_ADD ) ) { /* -1 or 1 */
-			LOG(log_error, logtype_atalkd,
-				"as_timer: can't route %u.%u to loopback: %s",
-				ntohs( iface->i_addr.sat_addr.s_net ),
-				iface->i_addr.sat_addr.s_node,
+			LOG(log_error, logtype_atalkd, "as_timer: can't route %u.%u to loopback: %s",
+				ntohs( iface->i_addr.sat_addr.s_net ), iface->i_addr.sat_addr.s_node,
 				strerror(errno) );
 			atalkd_exit( 1 );
 		    }
@@ -323,14 +300,12 @@ static void as_timer(int sig _U_)
 	     * our zone has gone away.
 	     */
 	    if ( ++gate->g_state >= RTMPTAB_BAD ) {
-		LOG(log_info, logtype_atalkd, "as_timer gateway %u.%u down",
-			ntohs( gate->g_sat.sat_addr.s_net ),
+		LOG(log_info, logtype_atalkd, "as_timer gateway %u.%u down", ntohs( gate->g_sat.sat_addr.s_net ),
 			gate->g_sat.sat_addr.s_node );
 		rtmp = gate->g_rt;
 		while ( rtmp ) {
 		    frtmp = rtmp->rt_next;
-		    if ( rtmp->rt_hops == RTMPHOPS_POISON ||
-			    rtmp->rt_iprev == 0 ) {
+		    if ( rtmp->rt_hops == RTMPHOPS_POISON || rtmp->rt_iprev == NULL ) {
 			rtmp_free( rtmp );
 		    } else {
 			rtmp->rt_hops = RTMPHOPS_POISON;
@@ -344,13 +319,13 @@ static void as_timer(int sig _U_)
 		    }
 		    rtmp = frtmp;
 		}
-		if ( gate->g_rt == 0 ) {
-		    if ( gate->g_prev == 0 ) {
+		if ( gate->g_rt == NULL ) {
+		    if ( gate->g_prev == NULL ) {
 			gate->g_iface->i_gate = gate->g_next;
 		    } else {
 			gate->g_prev->g_next = gate->g_next;
 		    }
-		    if ( gate->g_next != 0 ) {
+		    if ( gate->g_next != NULL ) {
 			gate->g_next->g_prev = gate->g_prev;
 		    }
 		    fgate = gate;	/* can't free here, just mark it */
@@ -365,8 +340,7 @@ static void as_timer(int sig _U_)
 		 * if we're not a seed router.
 		 */
 
-		if ( gate->g_iface->i_gate == 0 && 
-		     ((iface->i_flags & IFACE_SEED) == 0)) {
+		if ( gate->g_iface->i_gate == NULL && ((iface->i_flags & IFACE_SEED) == 0)) {
 		    gate->g_iface->i_flags |= IFACE_NOROUTER;
 		    gate->g_iface->i_flags &= ~IFACE_CONFIG;
 
@@ -423,7 +397,7 @@ static void as_timer(int sig _U_)
 		 */
 		if ( rtmp->rt_state >= RTMPTAB_BAD ) {
 		    frtmp = rtmp->rt_next;
-		    if ( rtmp->rt_iprev == 0 ) {	/* not in use */
+		    if ( rtmp->rt_iprev == NULL ) {	/* not in use */
 			rtmp_free( rtmp );
 		    } else {				/* in use */
 			if ( rtmp->rt_hops == RTMPHOPS_POISON ) {
@@ -445,8 +419,7 @@ static void as_timer(int sig _U_)
 		/*
 		 * Do ZIP lookups.
 		 */
-		if ( rtmp->rt_iprev &&
-			( rtmp->rt_flags & RTMPTAB_HASZONES ) == 0 ) {
+		if ( rtmp->rt_iprev && ( rtmp->rt_flags & RTMPTAB_HASZONES ) == 0 ) {
 		    if ( data + sizeof( u_short ) > end || n == 255 ) {
 			/* send what we've got */
 			zh.zh_op = ZIPOP_QUERY;
@@ -456,11 +429,7 @@ static void as_timer(int sig _U_)
 			*data++ = DDPTYPE_ZIP;
 			memcpy( data, &zh, sizeof( struct ziphdr ));
 
-			if ( sendto( zap->ap_fd, packet, cc, 0,
-				(struct sockaddr *)&sat,
-				sizeof( struct sockaddr_at )) < 0 ) {
-			    LOG(log_error, logtype_atalkd, "as_timer sendto: %s", strerror(errno) );
-			}
+			sendto_iface(iface,  zap->ap_fd, packet, cc, &sat);
 			sentzipq = 1;
 
 			n = 0;
@@ -478,8 +447,7 @@ static void as_timer(int sig _U_)
 		     * ask about, and warn that we can't get it's zone.
 		     */
 		    if ( rtmp->rt_nzq++ == 3 ) {
-			LOG(log_info, logtype_atalkd, "as_timer can't get zone for %u",
-				ntohs( rtmp->rt_firstnet ));
+			LOG(log_info, logtype_atalkd, "as_timer can't get zone for %u", ntohs( rtmp->rt_firstnet ));
 		    }
 		    if ( rtmp->rt_nzq > 3 ) {
 			if ( ziptimeout ) {
@@ -506,10 +474,7 @@ static void as_timer(int sig _U_)
 		*data++ = DDPTYPE_ZIP;
 		memcpy( data, &zh, sizeof( struct ziphdr ));
 
-		if ( sendto( zap->ap_fd, packet, cc, 0, (struct sockaddr *)&sat,
-			sizeof( struct sockaddr_at )) < 0 ) {
-		    LOG(log_error, logtype_atalkd, "as_timer sendto: %s", strerror(errno) );
-		}
+		sendto_iface( iface, zap->ap_fd, packet, cc, &sat);
 	    }
 	}
 	if ( fgate ) {
@@ -575,31 +540,33 @@ static void as_timer(int sig _U_)
 		 */
 		for ( rtmp = iface2->i_rt; rtmp; rtmp = rtmp->rt_inext ) {
 		    /* don't broadcast routes we have no zone for */
-		    if ( rtmp->rt_zt == 0 ||
+		    if ( rtmp->rt_zt == NULL ||
 			    ( rtmp->rt_flags & RTMPTAB_ZIPQUERY ) ||
 			    ( rtmp->rt_flags & RTMPTAB_HASZONES ) == 0 ) {
 			continue;
 		    }
 
+		    /* split horizon */
+
+		    /* HACK: If we only have one interface configured and are using
+      		     * the -router switch to seed that interface, we need not
+	     	     * worry about split horizon. Otherwise interface won't
+	    	     * transmit RTMP broadcasts.
+		     */
+		    if (!(iface->i_flags & IFACE_RSEED) && (rtmp->rt_iface == iface)) {
+		        continue;
+		    }
+
 		    if ((( rtmp->rt_flags & RTMPTAB_EXTENDED ) &&
 			    data + 2 * SZ_RTMPTUPLE > end ) ||
 			    data + SZ_RTMPTUPLE > end ) {
-			if ( sendto( rap->ap_fd, packet, data - packet, 0,
-				(struct sockaddr *)&sat,
-				sizeof( struct sockaddr_at )) < 0 ) {
-			    LOG(log_error, logtype_atalkd, "as_timer sendto %u.%u (%u): %s",
-				    ntohs( sat.sat_addr.s_net ),
-				    sat.sat_addr.s_node,
-				    ntohs( iface->i_rt->rt_firstnet ),
-				    strerror(errno) );
-			}
+
+			sendto_iface(iface,rap->ap_fd, packet, data - packet, &sat);
 
 			if ( iface->i_flags & IFACE_PHASE2 ) {
-			    data = packet + 1 + sizeof( struct rtmp_head ) +
-				    2 * SZ_RTMPTUPLE;
+			    data = packet + 1 + sizeof( struct rtmp_head ) + 2 * SZ_RTMPTUPLE;
 			} else {
-			    data = packet + 1 + sizeof( struct rtmp_head ) +
-				    SZ_RTMPTUPLE;
+			    data = packet + 1 + sizeof( struct rtmp_head ) + SZ_RTMPTUPLE;
 			}
 			n = 0;
 		    }
@@ -624,15 +591,7 @@ static void as_timer(int sig _U_)
 
 	    /* send rest */
 	    if ( n ) {
-		if ( sendto( rap->ap_fd, packet, data - packet, 0,
-			(struct sockaddr *)&sat,
-			sizeof( struct sockaddr_at )) < 0 ) {
-		    LOG(log_error, logtype_atalkd, "as_timer sendto %u.%u (%u): %s",
-			    ntohs( sat.sat_addr.s_net ),
-			    sat.sat_addr.s_node,
-			    ntohs( iface->i_rt->rt_firstnet ),
-			    strerror(errno) );
-		}
+		sendto_iface(iface, rap->ap_fd, packet, data - packet, &sat);
 	    }
 	}
     }
@@ -672,16 +631,15 @@ static void as_timer(int sig _U_)
 	}
     }
 
-#ifdef DEBUG
-    consistency();
-#endif /* DEBUG */
+    if ( debug )
+        consistency();
+
 }
 
-#ifdef DEBUG
 /*
 * Consistency check...
 */
-consistency()
+static void consistency()
 {
     struct rtmptab	*rtmp;
     struct list		*lr, *lz;
@@ -692,8 +650,8 @@ consistency()
 	    rtmp = (struct rtmptab *)lr->l_data;
 	    if ( rtmp->rt_iprev == 0 && rtmp->rt_gate != 0 ) {
 		LOG(log_error, logtype_atalkd, "%.*s has %u-%u (unused)",
-			zt->zt_len, zt->zt_name, ntohs( rtmp->rt_firstnet ),
-			ntohs( rtmp->rt_lastnet ));
+		    zt->zt_len, zt->zt_name, ntohs( rtmp->rt_firstnet ),
+		    ntohs( rtmp->rt_lastnet ));
 		atalkd_exit(1);
 	    }
 	    for ( lz = rtmp->rt_zt; lz; lz = lz->l_next ) {
@@ -703,20 +661,17 @@ consistency()
 	    }
 	    if ( lz == 0 ) {
 		LOG(log_error, logtype_atalkd, "no map from %u-%u to %.*s", 
-			ntohs( rtmp->rt_firstnet ),
-			ntohs( rtmp->rt_lastnet ),
-			zt->zt_len, zt->zt_name );
+		    ntohs( rtmp->rt_firstnet ),
+		    ntohs( rtmp->rt_lastnet ),
+		    zt->zt_len, zt->zt_name );
 		atalkd_exit(1);
 	    }
 	}
     }
 }
-#endif /* DEBUG */
 
-#if !defined( ibm032 ) && !defined( _IBMR2 )
-    void
-#endif /* ! ibm032 && ! _IBMR2 */
-as_debug()
+static void
+as_debug(int sig _U_)
 {
     struct interface	*iface;
     struct list		*l;
@@ -817,10 +772,8 @@ as_debug()
 /*
  * Called when SIGTERM is recieved.  Remove all routes and then exit.
  */
-#if !defined( ibm032 ) && !defined( _IBMR2 )
-    void
-#endif /* ! ibm032 && ! _IBMR2 */
-as_down()
+static void
+as_down(int sig _U_)
 {
     struct interface	*iface;
     struct gate		*gate;
@@ -853,13 +806,8 @@ as_down()
     atalkd_exit( 0 );
 }
 
-int main( ac, av )
-    int		ac;
-    char	**av;
+int main( int ac, char **av)
 {
-    extern char         *optarg;
-    extern int          optind;
-
     struct sockaddr_at	sat;
     struct sigaction	sv;
     struct itimerval	it;
@@ -870,10 +818,10 @@ int main( ac, av )
     struct atport	*ap;
     fd_set		readfds;
     int			i, c;
-    SOCKLEN_T 		fromlen;
+    socklen_t 		fromlen;
     char		*prog;
 
-    while (( c = getopt( ac, av, "12qsdtf:P:v" )) != EOF ) {
+    while (( c = getopt( ac, av, "12df:P:tv" )) != EOF ) {
 	switch ( c ) {
 	case '1' :
 	    defphase = IFACE_PHASE1;
@@ -891,29 +839,21 @@ int main( ac, av )
 	    configfile = optarg;
 	    break;
 
-	case 'q' :	/* don't seed */
-	    quiet++;
-	    break;
-
-	case 's' :	/* seed */
-	    chatty++;
+	case 'P' :	/* pid file */
+	    pidfile = optarg;
 	    break;
 
 	case 't' :	/* transition */
 	    transition++;
 	    break;
 
-	case 'P' :	/* pid file */
-	    pidfile = optarg;
-	    break;
-
 	case 'v' :	/* version */
 	    printf( "atalkd (version %s)\n", version );
 	    exit ( 1 );
 	    break;
 
 	default :
-	    fprintf( stderr, "Unknown option -- '%c'\n", c );
+	    fprintf( stderr, "Usage:\tatalkd -1 -2 [-f configfile] [-P pidfile] -t -d\n" );
 	    exit( 1 );
 	}
     }
@@ -950,6 +890,8 @@ int main( ac, av )
      */
     if ( readconf( configfile ) < 0 && getifconf() < 0 ) {
 	fprintf( stderr, "%s: can't get interfaces, exiting.\n", prog );
+	if (interfaces != NULL)
+		free(interfaces);
 	exit( 1 );
     }
 
@@ -970,6 +912,12 @@ int main( ac, av )
      * broken.
      */
     for (iface = interfaces; iface; iface = iface->i_next) {
+        if ( (ninterfaces == IFBASE) &&
+                (iface->i_flags & IFACE_SEED) &&
+                ((iface->i_flags & IFACE_RSEED) == 0) ) {
+            fprintf( stderr, "%s: use the -router switch to seed a single interface.\n", prog );
+            exit( 1 );
+        }
 	/* Apply the default phase */
 	if (( iface->i_flags & IFACE_PHASE1 ) == 0 &&
 		( iface->i_flags & IFACE_PHASE2 ) == 0 ) {
@@ -1044,10 +992,8 @@ int main( ac, av )
 #ifdef SIOCDIFADDR
     for (iface = interfaces; iface; iface = iface->i_next) {
       if (ifconfig(iface->i_name, SIOCDIFADDR, &iface->i_addr)) {
-#ifdef SIOCATALKDIFADDR
-#if (SIOCDIFADDR != SIOCATALKDIFADDR)
+#if defined(__linux__)
 	ifconfig(iface->i_name, SIOCATALKDIFADDR, &iface->i_addr);
-#endif /* SIOCDIFADDR != SIOCATALKDIFADDR */
 #endif /* SIOCATALKDIFADDR */
       }
     }
@@ -1089,12 +1035,8 @@ int main( ac, av )
       exit( 0 );
     }
 
-#ifdef ultrix
-    openlog( prog, LOG_PID );
-#else /* ultrix */
     set_processname(prog);
     syslog_setup(log_debug, logtype_default, logoption_pid, logfacility_daemon );
-#endif /* ultrix */
 
     LOG(log_info, logtype_atalkd, "restart (%s)", version );
 
@@ -1166,10 +1108,6 @@ int main( ac, av )
 
     sigemptyset( &signal_set );
     sigaddset(&signal_set, SIGALRM);
-#if 0
-    /* don't block SIGTERM */
-    sigaddset(&signal_set, SIGTERM);
-#endif
     sigaddset(&signal_set, SIGUSR1);
 
     for (;;) {
@@ -1194,15 +1132,15 @@ int main( ac, av )
 			    LOG(log_error, logtype_atalkd, "recvfrom: %s", strerror(errno) );
 			    continue;
 			}
-#ifdef DEBUG1
+
 			if ( debug ) {
 			    printf( "packet from %u.%u on %s (%x) %d (%d)\n",
-				    ntohs( sat.sat_addr.s_net ),
-				    sat.sat_addr.s_node, iface->i_name,
-				    iface->i_flags, ap->ap_port, ap->ap_fd );
+				ntohs( sat.sat_addr.s_net ),
+				sat.sat_addr.s_node, iface->i_name,
+				iface->i_flags, ap->ap_port, ap->ap_fd );
 			    bprint( Packet, c );
 			}
-#endif 
+
 			if (sigprocmask(SIG_BLOCK, &signal_set, &old_set) < 0) {
 			    LOG(log_error, logtype_atalkd, "sigprocmask: %s", strerror(errno) );
 			    atalkd_exit( 1 );
@@ -1213,9 +1151,9 @@ int main( ac, av )
 			  atalkd_exit(1);
 			}
 
-#ifdef DEBUG
-			consistency();
-#endif 
+			if ( debug )
+			    consistency();
+
 			if (sigprocmask(SIG_SETMASK, &old_set, NULL) < 0) {
 			    LOG(log_error, logtype_atalkd, "sigprocmask old set: %s", strerror(errno) );
 			    atalkd_exit( 1 );
@@ -1233,10 +1171,9 @@ int main( ac, av )
  * and rtmp_packet()) to set the initial "bootstrapping" address
  * on an interface.
  */
-void bootaddr( iface )
-    struct interface	*iface;
+void bootaddr(struct interface *iface)
 {
-    if ( iface == 0 ) {
+    if ( iface == NULL ) {
 	return;
     }
 
@@ -1247,8 +1184,14 @@ void bootaddr( iface )
     }
 
     if ( iface->i_flags & IFACE_PHASE1 ) {
+#ifndef __NetBSD__
 	setaddr( iface, IFACE_PHASE1, 0,
 		iface->i_caddr.sat_addr.s_node, 0, 0 );
+#else
+       setaddr( iface, IFACE_PHASE1, iface->i_caddr.sat_addr.s_net,
+		iface->i_caddr.sat_addr.s_node,
+		iface->i_rt->rt_firstnet, iface->i_rt->rt_lastnet );
+#endif
 
 	if ( iface->i_flags & IFACE_LOOPBACK ) {
 	    iface->i_flags |= IFACE_CONFIG | IFACE_ADDR;
@@ -1350,10 +1293,7 @@ smaller net range.", iface->i_name, ntohs(first), ntohs(last), strerror(errno));
 
     /* open ports */
     i = 1; /* enable broadcasts */
-#if 0
-    /* useless message, no? */
-    LOG(log_info, logtype_atalkd, "setsockopt incompatible w/ Solaris STREAMS module.");
-#endif /* __svr4__ */
+
     for ( ap = iface->i_ports; ap; ap = ap->ap_next ) {
 	if (( ap->ap_fd = socket( AF_APPLETALK, SOCK_DGRAM, 0 )) < 0 ) {
 	    LOG(log_error, logtype_atalkd, "socket: %s", strerror(errno) );
@@ -1381,11 +1321,9 @@ smaller net range.", iface->i_name, ntohs(first), ntohs(last), strerror(errno));
 	    /* remove all interfaces if we have a problem with bind */
 	    for (iface = interfaces; iface; iface = iface->i_next) {
 	      if (ifconfig( iface->i_name, SIOCDIFADDR, &iface->i_addr )) {
-#ifdef SIOCATALKDIFADDR
-#if (SIOCDIFADDR != SIOCATALKDIFADDR)
+#if defined(__linux__)
 		ifconfig( iface->i_name, SIOCATALKDIFADDR, &iface->i_addr );
-#endif /* SIOCDIFADDR != SIOCATALKDIFADDR */
-#endif /* SIOCATALKDIFADDR */
+#endif /* __linux__ */
 	      }
 	    }
 #endif /* SIOCDIFADDR */
@@ -1406,9 +1344,7 @@ smaller net range.", iface->i_name, ntohs(first), ntohs(last), strerror(errno));
     nfds++;
 }
 
-int ifsetallmulti ( iname, set )
-const char		*iname;
-int set;
+int ifsetallmulti (const char *iname, int set)
 {
     int sock;
     struct ifreq ifr;
@@ -1443,10 +1379,7 @@ int set;
     return (0);
 }
 
-int ifconfig( iname, cmd, sa )
-    const char		*iname;
-    unsigned long	cmd;
-    struct sockaddr_at	*sa;
+int ifconfig( const char *iname, unsigned long cmd, struct sockaddr_at *sa)
 {
     struct ifreq	ifr;
     int			s;
@@ -1469,8 +1402,7 @@ int ifconfig( iname, cmd, sa )
     return( 0 );
 }
 
-void dumpconfig( iface )
-    struct interface	*iface;
+void dumpconfig( struct interface *iface)
 {
     struct list		*l;
 
@@ -1504,81 +1436,3 @@ void dumpconfig( iface )
     }
     printf( "\n" );
 }
-
-#ifdef DEBUG
-void dumproutes()
-{
-    struct interface	*iface;
-    struct rtmptab	*rtmp;
-    struct list		*l;
-    struct ziptab	*zt;
-
-    for ( iface = interfaces; iface; iface = iface->i_next ) {
-	for ( rtmp = iface->i_rt; rtmp; rtmp = rtmp->rt_inext ) {
-	    if ( rtmp->rt_gate == 0 ) {
-		if ( rtmp->rt_flags & RTMPTAB_EXTENDED ) {
-		    printf( "%u-%u", ntohs( rtmp->rt_firstnet ),
-			    ntohs( rtmp->rt_lastnet ));
-		} else {
-		    printf( "%u", ntohs( rtmp->rt_firstnet ));
-		}
-	    } else {
-		if ( rtmp->rt_flags & RTMPTAB_EXTENDED ) {
-		    printf( "%u.%u for %u-%u",
-			    ntohs( rtmp->rt_gate->g_sat.sat_addr.s_net ),
-			    rtmp->rt_gate->g_sat.sat_addr.s_node,
-			    ntohs( rtmp->rt_firstnet ),
-			    ntohs( rtmp->rt_lastnet ));
-		} else {
-		    printf( "%u.%u for %u",
-			    ntohs( rtmp->rt_gate->g_sat.sat_addr.s_net ),
-			    rtmp->rt_gate->g_sat.sat_addr.s_node,
-			    ntohs( rtmp->rt_firstnet ));
-		}
-	    }
-
-	    if ( rtmp->rt_iprev == 0 && rtmp != iface->i_rt ) {
-		printf( " *" );
-	    }
-
-	    for ( l = rtmp->rt_zt; l; l = l->l_next ) {
-		zt = (struct ziptab *)l->l_data;
-		printf( " %.*s", zt->zt_len, zt->zt_name );
-	    }
-
-	    printf( "\n" );
-	}
-    }
-
-    printf( "\n" );
-    fflush( stdout );
-}
-
-void dumpzones()
-{
-    struct interface	*iface;
-    struct rtmptab	*rtmp;
-    struct list		*l;
-    struct ziptab	*zt;
-
-    for ( zt = ziptab; zt; zt = zt->zt_next ) {
-	printf( "%.*s", zt->zt_len, zt->zt_name );
-	for ( l = zt->zt_rt; l; l = l->l_next ) {
-	    rtmp = (struct rtmptab *)l->l_data;
-	    if ( rtmp->rt_flags & RTMPTAB_EXTENDED ) {
-		printf( " %u-%u", ntohs( rtmp->rt_firstnet ),
-			ntohs( rtmp->rt_lastnet ));
-	    } else {
-		printf( " %u", ntohs( rtmp->rt_firstnet ));
-	    }
-	    if ( rtmp->rt_iprev == 0 && rtmp->rt_gate != 0 ) {
-		printf( "*" );
-	    }
-	}
-	printf( "\n" );
-    }
-
-    printf( "\n" );
-    fflush( stdout );
-}
-#endif /* DEBUG */
