@@ -1,6 +1,4 @@
 /*
- * $Id: nbp.c,v 1.13 2009-10-14 02:24:05 didg Exp $
- *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * All Rights Reserved. See COPYRIGHT.
  */
@@ -17,10 +15,6 @@
 #include <sys/param.h>
 #include <sys/socket.h>
 #include <sys/ioctl.h>
-#ifdef TRU64
-#include <sys/mbuf.h>
-#include <net/route.h>
-#endif /* TRU64 */
 #include <net/if.h>
 #include <netatalk/at.h>
 #include <atalk/ddp.h>
@@ -79,6 +73,9 @@ int nbp_packet(struct atport *ap, struct sockaddr_at *from, char *data, int len)
     int         n, i, cc, locallkup;
     u_char      tmplen;
 
+    /* initialize per valgrind */
+    memset(&sat, 0, sizeof (struct sockaddr_at));
+
     end = data + len;
     if ( data >= end ) {
         LOG(log_info, logtype_atalkd, "nbp_packet malformed packet" );
