@@ -1,6 +1,4 @@
 /*
- * $Id: rtmp.c,v 1.17 2009-12-08 03:21:16 didg Exp $
- *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * All Rights Reserved. See COPYRIGHT.
  */
@@ -17,10 +15,6 @@
 #include <sys/param.h>
 #include <sys/socket.h>
 #include <sys/ioctl.h>
-#ifdef TRU64
-#include <sys/mbuf.h>
-#include <net/route.h>
-#endif /* TRU64 */
 #include <net/if.h>
 #include <net/route.h>
 #include <netatalk/endian.h>
@@ -43,6 +37,8 @@
 #include "route.h"
 #include "main.h"
 
+extern int debug;
+
 void rtmp_delzonemap(struct rtmptab *rtmp)
 {
     struct list		*lz, *flz, *lr, *flr;
@@ -797,11 +793,9 @@ int rtmp_packet(struct atport *ap, struct sockaddr_at *from, char *data, int len
 		LOG(log_error, logtype_atalkd, "as_timer sendto: %s", strerror(errno) );
 	    }
 	} else if ( *data == 2 || *data == 3 ) {
-#ifdef DEBUG
-	    printf( "rtmp_packet rdr (%d) from %u.%u\n",
+            LOG(log_debug, logtype_atalkd, "rtmp_packet rdr (%d) from %u.%u\n",
 		    *data, ntohs( from->sat_addr.s_net ),
 		    from->sat_addr.s_node );
-#endif /* DEBUG */
 	} else {
 	    LOG(log_info, logtype_atalkd, "rtmp_packet unknown request from %u.%u",
 		    ntohs( from->sat_addr.s_net ), from->sat_addr.s_node );
