@@ -70,19 +70,12 @@
 #include <pwd.h>
 #include <grp.h>
 
-/* FIXME */
-#ifdef linux
-#ifndef USE_SETRESUID
-#define USE_SETRESUID 1
+#if defined(__linux__)
 #define SWITCH_TO_GID(gid)  ((setresgid(gid,gid,gid) < 0 || setgid(gid) < 0) ? -1 : 0)
 #define SWITCH_TO_UID(uid)  ((setresuid(uid,uid,uid) < 0 || setuid(uid) < 0) ? -1 : 0)
-#endif  /* USE_SETRESUID */
-#else   /* ! linux */
-#ifndef USE_SETEUID
-#define USE_SETEUID 1
+#else
 #define SWITCH_TO_GID(gid)  ((setegid(gid) < 0 || setgid(gid) < 0) ? -1 : 0)
 #define SWITCH_TO_UID(uid)  ((setuid(uid) < 0 || seteuid(uid) < 0 || setuid(uid) < 0) ? -1 : 0)
-#endif  /* USE_SETEUID */
 #endif  /* linux */
 
 #include <atalk/util.h>
@@ -162,6 +155,7 @@ static int maybe_start_dbd(char *dbdpn, struct volinfo *volinfo)
     char buf1[8];
     char buf2[8];
     char *volpath = volinfo->v_path;
+    int ret;
 
     LOG(log_debug, logtype_cnid, "maybe_start_dbd: Volume: \"%s\"", volpath);
 
@@ -205,7 +199,7 @@ static int maybe_start_dbd(char *dbdpn, struct volinfo *volinfo)
                 if (up->count > MAXSPAWN) {
                     /* ...and already tried to fork too often */
                     LOG(log_maxdebug, logtype_cnid, "maybe_start_dbd: respawning too fast");
-                    return -1; /* just exit, dont sleep, because we might have work to do for another client  */
+                    return -1; /* just exit, don't sleep, because we might have work to do for another client  */
                 }
             } else {
                 /* out of respawn too fast windows reset the count */
@@ -219,7 +213,7 @@ static int maybe_start_dbd(char *dbdpn, struct volinfo *volinfo)
         if (up->count > MAXSPAWN) {
             /* We spawned too fast. From now until the first time we tried + TESTTIME seconds
                we will just return -1 above */
-            LOG(log_info, logtype_cnid, "maybe_start_dbd: reached MAXSPAWN threshhold");
+            LOG(log_info, logtype_cnid, "maybe_start_dbd: reached MAXSPAWN threshold");
        }
     }
 
@@ -238,7 +232,6 @@ static int maybe_start_dbd(char *dbdpn, struct volinfo *volinfo)
         return -1;
     }
     if (pid == 0) {
-        int ret;
         /*
          *  Child. Close descriptors and start the daemon. If it fails
          *  just log it. The client process will fail connecting
@@ -266,9 +259,11 @@ static int maybe_start_dbd(char *dbdpn, struct volinfo *volinfo)
         } else {
             ret = execlp(dbdpn, dbdpn, volpath, buf1, buf2, logconfig, NULL);
         }
-        /* Yikes! We're still here, so exec failed... */
-        LOG(log_error, logtype_cnid, "Fatal error in exec: %s", strerror(errno));
-        daemon_exit(0);
+        if (ret) {
+            /* Yikes! We're still here, so exec failed... */
+            LOG(log_error, logtype_cnid, "Fatal error in exec: %s", strerror(errno));
+            daemon_exit(0);
+        }
     }
     /*
      *  Parent.
@@ -557,7 +552,7 @@ int main(int argc, char *argv[])
     while (1) {
         rqstfd = usockfd_check(srvfd, &set);
         /* Collect zombie processes and log what happened to them */
-        if (sigchild) while ((pid = waitpid(-1, &status, WNOHANG)) > 0) {
+        while ((pid = waitpid(-1, &status, WNOHANG)) > 0) {
             for (i = 0; i < maxvol; i++) {
                 if (srv[i].pid == pid) {
                     srv[i].pid = 0;
