@@ -14,7 +14,7 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/cdefs.h>
+#include <sys/types.h>
 #include <unistd.h>
 
 #include <db.h>
@@ -111,7 +111,7 @@ static int dbif_init_rootinfo(DBD *dbd, int version)
     if (dbif_put(dbd, DBIF_CNID, &key, &data, 0) < 0)
         return -1;
     if (dbif_txn_commit(dbd) != 1) {
-        LOG(log_error, logtype_cnid, "dbif_init_rootinfo: cant commit txn");
+        LOG(log_error, logtype_cnid, "dbif_init_rootinfo: can't commit txn");
         return -1;
     }
 
@@ -363,7 +363,7 @@ int get_lock(int cmd, const char *dbpath)
     case LOCK_EXCL:
     case LOCK_SHRD:
         if (lockfd == -1) {
-            if ( (strlen(dbpath) + strlen(LOCKFILENAME+1)) > (PATH_MAX - 1) ) {
+            if ( (strlen(dbpath) + (strlen(LOCKFILENAME) + 1)) > (PATH_MAX - 1) ) {
                 LOG(log_error, logtype_cnid, ".AppleDB pathname too long");
                 return -1;
             }
@@ -421,8 +421,10 @@ DBD *dbif_init(const char *envhome, const char *filename)
 
     /* filename == NULL means in memory db */
     if (filename) {
-        if (! envhome)
+        if (! envhome) {
+            free(dbd);
             return NULL;
+        }
 
         dbd->db_envhome = strdup(envhome);
         if (NULL == dbd->db_envhome) {
@@ -671,7 +673,7 @@ int dbif_open(DBD *dbd, struct db_param *dbp, int reindex)
                                       dbd->db_table[i].type,
                                       dbd->db_table[i].openflags,
                                       0664) < 0) {
-            LOG(log_error, logtype_cnid, "Cant open database");
+            LOG(log_error, logtype_cnid, "Can't open database");
             return -1;
         }
 
@@ -808,7 +810,7 @@ int dbif_env_remove(const char *path)
     LOG(log_debug, logtype_cnid, "Trying to remove BerkeleyDB environment");
 
     if (get_lock(LOCK_EXCL, path) != LOCK_EXCL) {
-        LOG(log_warning, logtype_cnid, "CNID db \"%s\" in use, can't remove BerkeleyDB environment", path);
+        LOG(log_debug, logtype_cnid, "CNID db \"%s\" in use, can't remove BerkeleyDB environment", path);
         return 0;
     }
     
@@ -900,30 +902,6 @@ int dbif_get(DBD *dbd, const int dbi, DBT *key, DBT *val, u_int32_t flags)
         return 1;
 }
 
-/* search by secondary return primary */
-int dbif_pget(DBD *dbd, const int dbi, DBT *key, DBT *pkey, DBT *val, u_int32_t flags)
-{
-    int ret;
-
-    ret = dbd->db_table[dbi].db->pget(dbd->db_table[dbi].db,
-                                      dbd->db_txn,
-                                      key,
-                                      pkey,
-                                      val,
-                                      flags);
-
-    if (ret == DB_NOTFOUND || ret == DB_SECONDARY_BAD) {
-        return 0;
-    }
-    if (ret) {
-        LOG(log_error, logtype_cnid, "error retrieving value from %s: %s",
-            dbd->db_table[dbi].name, db_strerror(ret));
-        return -1;
-   } else
-        return 1;
-}
-
-/* -------------------------- */
 int dbif_put(DBD *dbd, const int dbi, DBT *key, DBT *val, u_int32_t flags)
 {
     int ret;
