@@ -1,6 +1,4 @@
 /*
- * $Id: usockfd.h,v 1.5 2009-11-05 14:38:07 franklahm Exp $
- *
  * Copyright (C) Joerg Lenneis 2003
  * All Rights Reserved.  See COPYING.
  */
@@ -13,9 +11,9 @@
 #include <atalk/cnid_dbd_private.h>
 
 
-extern int      usockfd_create  (char *, mode_t, int);
-extern int      tsockfd_create  (char *, char *, int);
-extern int      usockfd_check   (int, const sigset_t *);
+int      usockfd_create  (char *, mode_t, int);
+int      tsockfd_create  (char *, char *, int);
+int      usockfd_check   (int, const sigset_t *);
 
 #ifndef OSSH_ALIGNBYTES
 #define OSSH_ALIGNBYTES (sizeof(int) - 1)
