@@ -1,6 +1,4 @@
 /*
- * $Id: lp.c,v 1.14.8.4.2.9 2009/07/20 09:03:54 franklahm Exp $
- *
  * Copyright (c) 1990,1994 Regents of The University of Michigan.
  * All Rights Reserved.  See COPYRIGHT.
  *
@@ -50,9 +48,7 @@
 #include <sys/socket.h>
 #include <sys/stat.h>
 #include <ctype.h>
-#ifdef HAVE_UNISTD_H
 #include <unistd.h>
-#endif /* HAVE_UNISTD_H */
 
 #include <sys/file.h>
 #include <sys/un.h>
@@ -92,11 +88,11 @@ int lp_disconn_inet( int );
 int lp_conn_unix();
 int lp_disconn_unix( int );
 
-char	hostname[ MAXHOSTNAMELEN ];
+static char hostname[ MAXHOSTNAMELEN ];
 
 extern struct sockaddr_at *sat;
 
-struct lp {
+static struct lp {
     int			lp_flags;
     FILE		*lp_stream;
     int			lp_seq;
@@ -127,7 +123,7 @@ static void convert_octal (char *string, charset_t dest)
     char temp[4];
     long int ch;
 
-    q=p=string;
+    q=p=(unsigned char *)string;
     while ( *p != '\0' ) {
         ch = 0;
         if ( *p == '\\' ) {
@@ -164,7 +160,7 @@ static void translate(charset_t from, charset_t dest, char **option)
     if (*option != NULL) {
         convert_octal(*option, from);
         if (from) {
-             if ((size_t) -1 != (convert_string_allocate(from, dest, *option, strlen(*option), &translated)) ) {
+             if ((size_t) -1 != (convert_string_allocate(from, dest, *option, -1, &translated)) ) {
                  free (*option);
                  *option = translated;
              }
@@ -187,27 +183,23 @@ static void lp_setup_comments (charset_t dest)
     }
 
     if (lp.lp_job) {
-#ifdef DEBUG1
-        LOG(log_debug, logtype_papd, "job: %s", lp.lp_job );
-#endif
+        LOG(log_debug9, logtype_papd, "job: %s", lp.lp_job );
         translate(from, dest, &lp.lp_job);
     }
     if (lp.lp_created_for) {
-#ifdef DEBUG1
-        LOG(log_debug, logtype_papd, "for: %s", lp.lp_created_for );
-#endif
+        LOG(log_debug9, logtype_papd, "for: %s", lp.lp_created_for );
         translate(from, dest, &lp.lp_created_for);
     }
     if (lp.lp_person) {
-#ifdef DEBUG1
-       LOG(log_debug, logtype_papd, "person: %s", lp.lp_person );
-#endif
-       translate(from, dest, &lp.lp_person);
+        LOG(log_debug9, logtype_papd, "person: %s", lp.lp_person );
+        translate(from, dest, &lp.lp_person);
     }
 }
 
 #define is_var(a, b) (strncmp((a), (b), 2) == 0)
 
+#if 0
+/* removed, it's not used and a pain to get it right from a security POV */
 static size_t quote(char *dest, char *src, const size_t bsize, size_t len)
 {
 size_t used = 0;
@@ -234,7 +226,6 @@ size_t used = 0;
     return used;
 }
 
-
 static char* pipexlate(char *src)
 {
     char *p, *q, *dest; 
@@ -302,10 +293,9 @@ static char* pipexlate(char *src)
     }
     return destbuf;
 }
+#endif
 
-
-void lp_person( person )
-    char	*person;
+void lp_person(char *person)
 {
     if ( lp.lp_person != NULL ) {
 	free( lp.lp_person );
@@ -318,7 +308,7 @@ void lp_person( person )
 }
 
 #ifdef ABS_PRINT
-int lp_pagecost()
+int lp_pagecost(void)
 {
     char	cost[ 22 ];
     char	balance[ 22 ];
@@ -335,8 +325,7 @@ int lp_pagecost()
 }
 #endif /* ABS_PRINT */
 
-void lp_host( host )
-    char	*host;
+void lp_host( char *host)
 {
     if ( lp.lp_host != NULL ) {
 	free( lp.lp_host );
@@ -354,22 +343,17 @@ void lp_host( host )
  * This should be added.
  */
 
-void lp_job( job )
-    char	*job;
+void lp_job(char *job)
 {
     if ( lp.lp_job != NULL ) {
 	free( lp.lp_job );
     }
 
     lp.lp_job = strdup(job);
-#ifdef DEBUG
-    LOG(log_debug, logtype_papd, "job: %s", lp.lp_job );
-#endif
-    
+    LOG(log_debug9, logtype_papd, "job: %s", lp.lp_job );
 }
 
-void lp_for ( lpfor )
-	char	*lpfor;
+void lp_for (char *lpfor)
 {
     if ( lp.lp_created_for != NULL ) {
 	free( lp.lp_created_for );
@@ -379,15 +363,17 @@ void lp_for ( lpfor )
 }
 
 
-static int lp_init( out, sat )
-    struct papfile	*out;
-    struct sockaddr_at	*sat;
+static int lp_init(struct papfile *out, struct sockaddr_at *sat)
 {
     int		authenticated = 0;
 #ifndef HAVE_CUPS
     int		fd, n, len;
     char	*cp, buf[ BUFSIZ ];
     struct stat	st;
+    struct flock lock;
+    lock.l_start = 0;
+    lock.l_len = 0;
+    lock.l_whence = SEEK_SET;
 #endif /* HAVE_CUPS */
 #ifdef ABS_PRINT
     char	cost[ 22 ];
@@ -496,13 +482,12 @@ static int lp_init( out, sat )
 	    return( -1 );
 	}
 
-#ifndef SOLARIS /* flock is unsupported, I doubt this stuff works anyway with newer solaris so ignore for now */
-	if ( flock( fd, LOCK_EX ) < 0 ) {
+	lock.l_type = F_WRLCK;
+	if (( fcntl(fd, F_SETLK, &lock)) < 0 ) {
 	    LOG(log_error, logtype_papd, "lp_init: can't lock .seq" );
 	    spoolerror( out, NULL );
 	    return( -1 );
 	}
-#endif
 
 	n = 0;
 	if (( len = read( fd, buf, sizeof( buf ))) < 0 ) {
@@ -523,7 +508,8 @@ static int lp_init( out, sat )
 	n = ( n + 1 ) % 1000;
 	sprintf( buf, "%03d\n", n );
 	lseek( fd, 0L, 0 );
-	write( fd, buf, strlen( buf ));
+	if (write(fd, buf, strlen(buf)) < 0)
+		LOG(log_error, logtype_papd, "write failed (%s)", strerror(errno));
 	close( fd );
 #else
 
@@ -544,17 +530,13 @@ static int lp_init( out, sat )
     return( 0 );
 }
 
-int lp_open( out, sat )
-    struct papfile	*out;
-    struct sockaddr_at	*sat;
+int lp_open(struct papfile *out, struct sockaddr_at *sat)
 {
     char	name[ MAXPATHLEN ];
     int		fd;
     struct passwd	*pwent;
 
-#ifdef DEBUG
-    LOG (log_debug, logtype_papd, "lp_open");
-#endif
+    LOG (log_debug9, logtype_papd, "lp_open");
 
     if ( lp.lp_flags & LP_JOBPENDING ) {
 	lp_print();
@@ -638,15 +620,13 @@ int lp_open( out, sat )
 	    spoolerror( out, NULL );
 	    return( -1 );
 	}
-#ifdef DEBUG        
-        LOG(log_debug, logtype_papd, "lp_open: opened %s", name );
-#endif	
+	LOG(log_debug9, logtype_papd, "lp_open: opened %s", name );
     }
     lp.lp_flags |= LP_OPEN;
     return( 0 );
 }
 
-int lp_close()
+int lp_close(void)
 {
     if (( lp.lp_flags & LP_INIT ) == 0 || ( lp.lp_flags & LP_OPEN ) == 0 ) {
 	return 0;
@@ -660,10 +640,7 @@ int lp_close()
 
 
 
-int lp_write(in, buf, len )
-    struct papfile *in;
-    char	*buf;
-    size_t	len;
+int lp_write(struct papfile *in, char *buf, size_t len)
 {
 #define BUFSIZE 32768
     static char tempbuf[BUFSIZE];
@@ -694,9 +671,7 @@ int lp_write(in, buf, len )
             tempbuf2[len] = 0;
             tbuf = tempbuf2;
             last_line_translated = 1;
-#ifdef DEBUG
-            LOG(log_debug, logtype_papd, "lp_write: %s", tbuf );
-#endif
+            LOG(log_debug9, logtype_papd, "lp_write: %s", tbuf );
         }
         else {
             LOG(log_error, logtype_papd, "lp_write: conversion buffer too small" );
@@ -715,10 +690,8 @@ int lp_write(in, buf, len )
      * we store the start of the print job in a buffer.
      * %%EndComment triggers writing to file */
     if (( lp.lp_flags & LP_OPEN ) == 0 ) {
-#ifdef DEBUG
-        LOG(log_debug, logtype_papd, "lp_write: writing to temporary buffer" );
-#endif
-    	if ((bufpos+len) > BUFSIZE) {
+        LOG(log_debug9, logtype_papd, "lp_write: writing to temporary buffer" );
+        if ((bufpos+len) > BUFSIZE) {
             LOG(log_error, logtype_papd, "lp_write: temporary buffer too small" );
             /* FIXME: call lp_open here? abort isn't nice... */
             abort();
@@ -746,7 +719,7 @@ int lp_write(in, buf, len )
     return( 0 );
 }
 
-int lp_cancel()
+int lp_cancel(void)
 {
     char	name[ MAXPATHLEN ];
     char	letter;
@@ -775,7 +748,7 @@ int lp_cancel()
  *
  * XXX piped?
  */
-int lp_print()
+int lp_print(void)
 {
 #ifndef HAVE_CUPS
     char		buf[ MAXPATHLEN ];
@@ -899,7 +872,7 @@ int lp_disconn_unix( int fd )
     return( close( fd ));
 }
 
-int lp_conn_unix()
+int lp_conn_unix(void)
 {
     int			s;
     struct sockaddr_un	saun;
@@ -926,7 +899,7 @@ int lp_disconn_inet( int fd )
     return( close( fd ));
 }
 
-int lp_conn_inet()
+int lp_conn_inet(void)
 {
     int			privfd, port = IPPORT_RESERVED - 1;
     struct sockaddr_in	sin;
@@ -970,8 +943,7 @@ int lp_conn_inet()
     return( privfd );
 }
 
-int lp_rmjob( job )
-    int		job;
+int lp_rmjob( int job)
 {
     char	buf[ 1024 ];
     int		n, s;
@@ -1010,8 +982,7 @@ char	*tag_files = "files: ";
 char	*tag_size = "size: ";
 char	*tag_status = "status: ";
 
-int lp_queue( out )
-    struct papfile	*out;
+int lp_queue( struct papfile *out)
 {
     char			buf[ 1024 ], *start, *stop, *p, *q;
     int				linelength, crlflength;
