@@ -1,6 +1,4 @@
 /*
- * $Id: magics.c,v 1.15 2009-10-13 22:55:37 didg Exp $
- *
  * Copyright (c) 1990,1994 Regents of The University of Michigan.
  * All Rights Reserved.  See COPYRIGHT.
  */
@@ -46,6 +44,10 @@ int ps( struct papfile *infile, struct papfile *outfile, struct sockaddr_at *sat
 		}
 		state = 1;
 	}	
+	if ( infile->pf_state & PF_QUERY )
+	{
+		infile->pf_state |= PF_BOT;
+	}
 	if ( (comment = compeek()) ) {
 	    switch( (*comment->c_handler)( infile, outfile, sat )) {
 	    case CH_DONE :
@@ -74,6 +76,7 @@ int ps( struct papfile *infile, struct papfile *outfile, struct sockaddr_at *sat
 		return( 0 );
 
 	    case -1 :
+		spoolreply( outfile, "Processing..." );
 		return( 0 );
 	    }
 
@@ -82,6 +85,10 @@ int ps( struct papfile *infile, struct papfile *outfile, struct sockaddr_at *sat
 		    compush( comment );
 		    continue;	/* top of for (;;) */
 		}
+		else {
+		    CONSUME( infile, linelength + crlflength );
+		    continue; /* clear out the input queue if client sent data before magic string */
+		}
 #if 0
 		infile->pf_state &= ~PF_BOT;
 
@@ -98,6 +105,7 @@ int ps( struct papfile *infile, struct papfile *outfile, struct sockaddr_at *sat
 	    CONSUME( infile, linelength + crlflength );
 	}
     }
+    return 0;
 }
 
 int cm_psquery( struct papfile *in, struct papfile *out, struct sockaddr_at *sat _U_)
@@ -107,6 +115,12 @@ int cm_psquery( struct papfile *in, struct papfile *out, struct sockaddr_at *sat
     int			linelength, crlflength;
 
     for (;;) {
+	if ( in->pf_state & PF_QUERY )
+	{
+	    /* handle eof at end of query job */
+	    compop();
+	    return (CH_DONE);
+	}
 	switch ( markline( in, &start, &linelength, &crlflength )) {
 	case 0 :
 	    /* eof on infile */
@@ -115,6 +129,7 @@ int cm_psquery( struct papfile *in, struct papfile *out, struct sockaddr_at *sat
 	    return( CH_DONE );
 
 	case -1 :
+	    spoolreply( out, "Processing..." );
 	    return( CH_MORE );
 
         case -2 :
@@ -181,6 +196,11 @@ int cm_psswitch(struct papfile *in, struct papfile *out, struct sockaddr_at *sat
     char		*start, *stop, *p;
     int			linelength, crlflength;
 
+	if ( in->pf_state & PF_QUERY )
+	{
+	    /*handle eof at end of query job */
+	    in->pf_state &= ~PF_QUERY;
+	}
     switch ( markline( in, &start, &linelength, &crlflength )) {
     case 0 :
 	/* eof on infile */
