@@ -1,6 +1,4 @@
 /*
- * $Id: main.c,v 1.18.6.2.2.7 2009/02/03 08:25:00 didg Exp $
- *
  * Copyright (c) 1990,1995 Regents of The University of Michigan.
  * All Rights Reserved.  See COPYRIGHT.
  */
@@ -31,30 +29,14 @@
 #endif /* ! WIFEXITED */
 
 #include <errno.h>
-
-/* STDC check */
-#if STDC_HEADERS
 #include <string.h>
-#else /* STDC_HEADERS */
-#ifndef HAVE_STRCHR
-#define strchr index
-#define strrchr index
-#endif /* HAVE_STRCHR */
-char *strchr (), *strrchr ();
-#ifndef HAVE_MEMCPY
-#define memcpy(d,s,n) bcopy ((s), (d), (n))
-#define memmove(d,s,n) bcopy ((s), (d), (n))
-#endif /* ! HAVE_MEMCPY */
-#endif /* STDC_HEADERS */
 
 #include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <netdb.h>
 #include <fcntl.h>
-#ifdef HAVE_UNISTD_H
 #include <unistd.h>
-#endif /* HAVE_UNISTD_H */
 
 #include <netatalk/endian.h>
 #include <netatalk/at.h>
@@ -73,7 +55,16 @@ char *strchr (), *strrchr ();
 #include "print_cups.h"
 
 
-#define PIPED_STATUS	"status: print spooler processing job"
+/* This maps to TReq and TResp, per "Inside AppleTalk" 10-13 */
+
+typedef struct __attribute__((__packed__)) {
+	u_int8_t user_bytes[4];
+	u_int8_t data_bytes[4];
+	u_int8_t buf_len;
+	char buf[255];
+} rbuf_t;
+
+static rbuf_t r_buf;
 
 struct printer	defprinter;
 struct printer	*printers = NULL;
@@ -82,7 +73,7 @@ int		debug = 0;
 static char	*conffile = _PATH_PAPDCONF;
 char		*printcap = _PATH_PAPDPRINTCAP;
 unsigned char	connid, quantum, sock, oquantum = PAP_MAXQUANTUM;
-char		*cannedstatus = PIPED_STATUS;
+char		*cannedstatus = "status: print spooler processing job";
 struct printer	*printer = NULL;
 char		*version = VERSION;
 static char	*pidfile = _PATH_PAPDLOCK;
@@ -91,7 +82,7 @@ char		*uamlist;
 char		*uampath = _PATH_PAPDUAMPATH;
 
 /* Prototypes for locally used functions */
-int getstatus( struct printer *pr, char *buf );
+int getstatus( struct printer *pr, rbuf_t *buf );
 int rprintcap( struct printer *pr );
 static void getprinters( char *cf );
 
@@ -104,11 +95,8 @@ static void papd_exit(const int i)
   exit(i);
 }
 
-#if !defined( ibm032 ) && !defined( _IBMR2 )
-    void
-#endif /* ! ibm032 && ! _IBMR2 */
-die( n )
-    int			n;
+static void
+die(int n)
 {
     struct printer	*pr;
     struct at_addr	addr;
@@ -136,15 +124,13 @@ die( n )
     papd_exit( n );
 }
 
-#if !defined( ibm032 ) && !defined( _IBMR2 )
-    void
-#endif /* ! ibm032 && ! _IBMR2 */
-reap()
+static void
+reap(int sig _U_)
 {
     int		status;
     int		pid;
 
-    while (( pid = wait3( &status, WNOHANG, 0 )) > 0 ) {
+    while (( pid = wait3( &status, WNOHANG, NULL )) > 0 ) {
 	if ( WIFEXITED( status )) {
 	    if ( WEXITSTATUS( status )) {
 		LOG(log_error, logtype_papd, "child %d exited with %d", pid,
@@ -164,11 +150,7 @@ reap()
     return;
 }
 
-char		rbuf[ 255 + 1 + 8 ];
-
-int main( ac, av )
-    int		ac;
-    char	**av;
+int main(int ac, char **av)
 {
     extern char         *optarg;
 
@@ -188,7 +170,7 @@ int main( ac, av )
 	perror( "gethostname" );
 	exit( 1 );
     }
-    if (( p = strchr( hostname, '.' )) != 0 ) {
+    if (( p = strchr( hostname, '.' )) != NULL ) {
 	*p = '\0';
     }
     if (( defprinter.p_name = (char *)malloc( strlen( hostname ) + 1 ))
@@ -265,9 +247,8 @@ int main( ac, av )
       exit(0);
     }      
 
-#ifdef DEBUG1
-    fault_setup(NULL);
-#endif
+    if ( debug ) 
+	fault_setup(NULL);
 
     /*
      * Start logging.
@@ -277,13 +258,9 @@ int main( ac, av )
     } else {
 	p++;
     }
-#ifdef ultrix
-    openlog( p, LOG_PID );
-#else /* ultrix */
     set_processname(p);
     syslog_setup(log_debug, logtype_default, logoption_ndelay | logoption_pid |
                debug ? logoption_perror : 0, logfacility_lpr );
-#endif /* ultrix */
 
     LOG(log_info, logtype_papd, "restart (%s)", version );
 #ifdef HAVE_CUPS
@@ -298,7 +275,7 @@ int main( ac, av )
 	}
 
 	if (!(pr->p_flags & P_CUPS)) {
-		if ((size_t)-1 != convert_string_allocate(CH_UNIX, CH_MAC, pr->p_name, strlen(pr->p_name), &atname)) {
+		if ((size_t)-1 != convert_string_allocate(CH_UNIX, CH_MAC, pr->p_name, -1, &atname)) {
 			pr->p_u_name = pr->p_name;
 			pr->p_name = atname;
 		}
@@ -329,7 +306,7 @@ int main( ac, av )
     sv.sa_handler = die;
     sigemptyset( &sv.sa_mask );
     sv.sa_flags = SA_RESTART;
-    if ( sigaction( SIGTERM, &sv, 0 ) < 0 ) {
+    if ( sigaction( SIGTERM, &sv, NULL ) < 0 ) {
 	LOG(log_error, logtype_papd, "sigaction: %s", strerror(errno) );
 	papd_exit( 1 );
     }
@@ -337,7 +314,7 @@ int main( ac, av )
     sv.sa_handler = reap;
     sigemptyset( &sv.sa_mask );
     sv.sa_flags = SA_RESTART;
-    if ( sigaction( SIGCHLD, &sv, 0 ) < 0 ) {
+    if ( sigaction( SIGCHLD, &sv, NULL ) < 0 ) {
 	LOG(log_error, logtype_papd, "sigaction: %s", strerror(errno) );
 	papd_exit( 1 );
     }
@@ -355,7 +332,7 @@ int main( ac, av )
 	for ( pr = printers; pr; pr = pr->p_next ) {
 	    FD_SET( atp_fileno( pr->p_atp ), &fdset );
 	}
-	if (( c = select( FD_SETSIZE, &fdset, 0, 0, 0 )) < 0 ) {
+	if (( c = select( FD_SETSIZE, &fdset, NULL, NULL, NULL )) < 0 ) {
 	    if ( errno == EINTR ) {
 		continue;
 	    }
@@ -394,14 +371,17 @@ int main( ac, av )
 		    connid = (unsigned char)cbuf[ 0 ];
 		    sock = (unsigned char)cbuf[ 4 ];
 		    quantum = (unsigned char)cbuf[ 5 ];
-		    rbuf[ 0 ] = cbuf[ 0 ];
-		    rbuf[ 1 ] = PAP_OPENREPLY;
-		    rbuf[ 2 ] = rbuf[ 3 ] = 0;
+		    r_buf.user_bytes[0] = connid;
+		    r_buf.user_bytes[1] = PAP_OPENREPLY;
+		    r_buf.user_bytes[2] = 0;
+		    r_buf.user_bytes[3] = 0;
 
 		    if (( pr->p_flags & P_SPOOLED ) && rprintcap( pr ) != 0 ) {
 			LOG(log_error, logtype_papd, "printcap problem: %s",
 				pr->p_printer );
-			rbuf[ 2 ] = rbuf[ 3 ] = 0xff;
+			/* "IAT", 10-16 */
+			r_buf.user_bytes[2] = 0xff;
+			r_buf.user_bytes[3] = 0xff;
 			err = 1;
 		    }
 
@@ -410,13 +390,13 @@ int main( ac, av )
 		    * If cups is not accepting jobs, we return
 		    * 0xffff to indicate we're busy
 		    */
-#ifdef DEBUG
-                    LOG(log_debug, logtype_papd, "CUPS: PAP_OPEN");
-#endif
+		    LOG(log_debug9, logtype_papd, "CUPS: PAP_OPEN");
+
 		    if ( (pr->p_flags & P_SPOOLED) && (cups_get_printer_status ( pr ) == 0)) {
                         LOG(log_error, logtype_papd, "CUPS_PAP_OPEN: %s is not accepting jobs",
                                 pr->p_printer );
-                        rbuf[ 2 ] = rbuf[ 3 ] = 0xff;
+			r_buf.user_bytes[2] = 0xff;
+			r_buf.user_bytes[3] = 0xff;
                         err = 1;
                     }
 #endif /* HAVE_CUPS */
@@ -429,18 +409,21 @@ int main( ac, av )
 		    if (( atp = atp_open( ATADDR_ANYPORT, 
 					  &pr->p_addr)) == NULL ) {
 			LOG(log_error, logtype_papd, "atp_open: %s", strerror(errno) );
-			rbuf[ 2 ] = rbuf[ 3 ] = 0xff;  /* printer busy */
-			rbuf[ 4 ] = 0; /* FIXME is it right? */
+			r_buf.user_bytes[2] = 0xff;
+			r_buf.user_bytes[3] = 0xff;
+			/* FIXME is it right? */
+			r_buf.data_bytes[0] = 0x00;
 			err = 1;
 		    }
 		    else {
-		       rbuf[ 4 ] = atp_sockaddr( atp )->sat_port;
+		       r_buf.data_bytes[0] = atp_sockaddr( atp )->sat_port;
                     }
-		    rbuf[ 5 ] = oquantum;
-		    rbuf[ 6 ] = rbuf[ 7 ] = 0;
+		    r_buf.data_bytes[1] = oquantum;
+		    r_buf.data_bytes[2] = 0;
+		    r_buf.data_bytes[3] = 0;
 
-		    iov.iov_base = rbuf;
-		    iov.iov_len = 8 + getstatus( pr, &rbuf[ 8 ] );
+		    iov.iov_base = &r_buf;
+		    iov.iov_len = 8 + getstatus( pr, &r_buf );
 		    atpb.atp_sresiov = &iov;
 		    atpb.atp_sresiovcnt = 1;
 		    /*
@@ -485,12 +468,12 @@ int main( ac, av )
 			sv.sa_handler = SIG_DFL;
 			sigemptyset( &sv.sa_mask );
 			sv.sa_flags = SA_RESTART;
-			if ( sigaction( SIGTERM, &sv, 0 ) < 0 ) {
+			if ( sigaction( SIGTERM, &sv, NULL ) < 0 ) {
 			    LOG(log_error, logtype_papd, "sigaction: %s", strerror(errno) );
 			    exit( 1 );
 			}
 			
-			if ( sigaction( SIGCHLD, &sv, 0 ) < 0 ) {
+			if ( sigaction( SIGCHLD, &sv, NULL ) < 0 ) {
 			    LOG(log_error, logtype_papd, "sigaction: %s", strerror(errno) );
 			    exit( 1 );
                         }
@@ -515,14 +498,17 @@ int main( ac, av )
 		    break;
 
 		case PAP_SENDSTATUS :
-		    rbuf[ 0 ] = 0;
-		    rbuf[ 1 ] = PAP_STATUS;
-		    rbuf[ 2 ] = rbuf[ 3 ] = 0;
-		    rbuf[ 4 ] = rbuf[ 5 ] = 0;
-		    rbuf[ 6 ] = rbuf[ 7 ] = 0;
+		    r_buf.user_bytes[0] = 0;
+		    r_buf.user_bytes[1] = PAP_STATUS;
+		    r_buf.user_bytes[2] = 0;
+		    r_buf.user_bytes[3] = 0;
+		    r_buf.data_bytes[0] = 0;
+		    r_buf.data_bytes[1] = 0;
+		    r_buf.data_bytes[2] = 0;
+		    r_buf.data_bytes[3] = 0;
 
-		    iov.iov_base = rbuf;
-		    iov.iov_len = 8 + getstatus( pr, &rbuf[ 8 ] );
+		    iov.iov_base = &r_buf;
+		    iov.iov_len = 8 + getstatus( pr, &r_buf );
 		    atpb.atp_sresiov = &iov;
 		    atpb.atp_sresiovcnt = 1;
 		    /*
@@ -567,58 +553,61 @@ int main( ac, av )
  * We assume buf is big enough for 255 bytes of data and a length byte.
  */
 
-int getstatus( pr, buf )
-    struct printer	*pr;
-    char		*buf;
+int getstatus(struct printer *pr, rbuf_t *buf)
 {
-
 #ifdef HAVE_CUPS
     if ( pr->p_flags & P_PIPED ) {
-	*buf = strlen( cannedstatus );
-	strncpy( &buf[ 1 ], cannedstatus, *buf );
-	return( *buf + 1 );
+	buf->buf_len = strlen(cannedstatus);
+	snprintf(buf->buf, 254, "%s", cannedstatus);
+	return (buf->buf_len + 1);
     } else {
 	cups_get_printer_status( pr );
-	*buf = strlen ( pr->p_status );
-	strncpy ( &buf[1], pr->p_status, *buf);
-	return ( *buf + 1);
+	buf->buf_len = strlen(pr->p_status);
+	snprintf(buf->buf, 254, "%s", pr->p_status);
+	return (buf->buf_len + 1);
     }
 #else
-
-    char		path[ MAXPATHLEN ];
-    int			fd = -1, rc;
+    char path[MAXPATHLEN];
+    char getstatus_buffer[255];
+    char *temp = NULL;
+    FILE *fd = NULL;
+    int rc;
 
     if ( pr->p_flags & P_SPOOLED && ( pr->p_spool != NULL )) {
-	strcpy( path, pr->p_spool );
-	strcat( path, "/status" );
-	fd = open( path, O_RDONLY);
+	snprintf(path, MAXPATHLEN - 1, "%s/status", pr->p_spool);
+	fd = fopen(path, O_RDONLY);
     }
 
-    if (( pr->p_flags & P_PIPED ) || ( fd < 0 )) {
-	*buf = strlen( cannedstatus );
-	strncpy( &buf[ 1 ], cannedstatus, *buf );
-	return( *buf + 1 );
+    if ((pr->p_flags & P_PIPED) || (fd == NULL)) {
+	buf->buf_len = strlen(cannedstatus);
+	snprintf(buf->buf, 254, "%s", cannedstatus);
+	return (buf->buf_len + 1);
     } else {
-	if (( rc = read( fd, &buf[ 1 ], 255 )) < 0 ) {
-	    rc = 0;
+	rc = fread(getstatus_buffer, 255, sizeof(unsigned char), fd);
+	fclose(fd);
+
+	if (rc > 0) {
+		if (temp != NULL) {
+			temp[strcspn(temp, "\n")] = '\0';
+			snprintf(buf->buf, 254, "%s", temp);
+			buf->buf_len = strlen(buf->buf);
+		}
+	} else {
+		snprintf(buf->buf, 254, "%s", "thisisanemptystring");
+		buf->buf_len = 0;
 	}
-	close( fd );
-	if ( rc && buf[ rc ] == '\n' ) {	/* remove trailing newline */
-	    rc--;
-	}
-	*buf = rc;
-	return( rc + 1 );
+
+	return (buf->buf_len + 1);
     }
 #endif /* HAVE_CUPS */
 }
 
-char	*pgetstr();
+char	*pgetstr(char *id, char **area);
 char	*getpname(char **area, int bufsize);
 
 #define PF_CONFBUFFER	1024
 
-static void getprinters( cf )
-    char	*cf;
+static void getprinters( char *cf)
 {
     char		buf[ PF_CONFBUFFER ], area[ PF_CONFBUFFER ], *a, *p, *name, *type, *zone;
     struct printer	*pr;
@@ -804,8 +793,7 @@ static void getprinters( cf )
     }
 }
 
-int rprintcap( pr )
-    struct printer	*pr;
+int rprintcap( struct printer *pr)
 {
 
 #ifdef HAVE_CUPS
@@ -942,17 +930,6 @@ int rprintcap( pr )
 	    strcpy( pr->p_lock, p );
 	}
 
-#ifdef KRB
-	/*
-	 * Must Kerberos authenticate?
-	 */
-	if ( pgetflag( "ka" ) == 1 ) {
-	    pr->p_flags |= P_KRB;
-	} else {
-	    pr->p_flags &= ~P_KRB;
-	}
-#endif /* KRB */
-
 	endprent();
     }
 #endif /* HAVE_CUPS */
