@@ -1,6 +1,4 @@
 /*
- * $Id: showppd.c,v 1.5.10.1 2004/06/09 01:25:54 bfernhomberg Exp $
- *
  * Copyright (c) 1995 Regents of The University of Michigan.
  * All Rights Reserved.  See COPYRIGHT.
  */
@@ -26,9 +24,7 @@ extern struct ppd_font		*ppd_fonts;
 extern struct ppd_feature	ppd_features[];
 
 
-int main( ac, av )
-    int		ac;
-    char	**av;
+int main(int ac, char **av)
 {
     struct ppd_feature	*pfe;
     struct ppd_font	*pfo;
