@@ -1,8 +1,6 @@
 /*
- * $Id: uams_dhx_pam.c,v 1.24.6.5.2.4 2008/12/03 19:28:23 didg Exp $
- *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
- * Copyright (c) 1999 Adrian Sun (asun@u.washington.edu) 
+ * Copyright (c) 1999 Adrian Sun (asun@u.washington.edu)
  * All Rights Reserved.  See COPYRIGHT.
  */
 
@@ -16,9 +14,7 @@
 #include <string.h>
 #include <atalk/logger.h>
 
-#ifdef HAVE_UNISTD_H
 #include <unistd.h>
-#endif /* HAVE_UNISTD_H */
 #include <errno.h>
 #ifdef HAVE_SECURITY_PAM_APPL_H
 #include <security/pam_appl.h>
@@ -72,15 +68,19 @@ static const u_int8_t g = 0x07;
  * and the server_login function
  */
 static pam_handle_t *pamh = NULL; 
-static char *PAM_username;
-static char *PAM_password;
+static unsigned char *PAM_username;
+static unsigned char *PAM_password;
 
 /* PAM conversation function
  * Here we assume (for now, at least) that echo on means login name, and
  * echo off means password.
  */
 static int PAM_conv (int num_msg,
+#if !defined(__svr4__)
                      const struct pam_message **msg,
+#else
+                     struct pam_message **msg,
+#endif
                      struct pam_response **resp,
                      void *appdata_ptr _U_) {
   int count = 0;
@@ -114,7 +114,7 @@ static int PAM_conv (int num_msg,
 
     switch (msg[count]->msg_style) {
     case PAM_PROMPT_ECHO_ON:
-      if (!(string = COPY_STRING(PAM_username))) {
+      if (!(string = COPY_STRING((const char *)PAM_username))) {
     /* Log Entry */
            LOG(log_info, logtype_uams, "uams_dhx_pam.c :PAM: username failure -- %s",
 		  strerror(errno));
@@ -123,7 +123,7 @@ static int PAM_conv (int num_msg,
       }
       break;
     case PAM_PROMPT_ECHO_OFF:
-      if (!(string = COPY_STRING(PAM_password))) {
+      if (!(string = COPY_STRING((const char *)PAM_password))) {
     /* Log Entry */
            LOG(log_info, logtype_uams, "uams_dhx_pam.c :PAM: passwd failure: --: %s",
 		  strerror(errno));
@@ -184,12 +184,13 @@ static struct pam_conv PAM_conversation = {
 };
 
 
-static int dhx_setup(void *obj, char *ibuf, int ibuflen _U_, 
-		     char *rbuf, int *rbuflen)
+static int dhx_setup(void *obj, unsigned char *ibuf, size_t ibuflen _U_, 
+		     unsigned char *rbuf, size_t *rbuflen)
 {
     u_int16_t sessid;
-    int i;
+    size_t i;
     BIGNUM *bn, *gbn, *pbn;
+    const BIGNUM *pub_key;
     DH *dh;
 
     /* get the client's public key */
@@ -233,9 +234,16 @@ static int dhx_setup(void *obj, char *ibuf, int ibuflen _U_,
       return AFPERR_PARAM;
     }
 
+    if (!DH_set0_pqg(dh, pbn, NULL, gbn)) {
+      BN_free(pbn);
+      BN_free(gbn);
+    /* Log Entry */
+      LOG(log_info, logtype_uams, "uams_dhx_pam.c :PAM DH_set0_pqg() mysteriously failed  -- %s", strerror(errno));
+    /* Log Entry */
+      goto pam_fail;
+    }
+
     /* generate key and make sure that we have enough space */
-    dh->p = pbn;
-    dh->g = gbn;
     if (DH_generate_key(dh) == 0) {
 	unsigned long dherror;
 	char errbuf[256];
@@ -249,16 +257,17 @@ static int dhx_setup(void *obj, char *ibuf, int ibuflen _U_,
 	ERR_free_strings();
 	goto pam_fail;
     }
-    if (BN_num_bytes(dh->pub_key) > KEYSIZE) {
+    DH_get0_key(dh, &pub_key, NULL);
+    if (BN_num_bytes(pub_key) > KEYSIZE) {
 	LOG(log_info, logtype_uams, "uams_dhx_pam.c :PAM: Err Generating Key -- Not enough Space? -- %s", strerror(errno));
 	goto pam_fail;
     }
 
     /* figure out the key. store the key in rbuf for now. */
-    i = DH_compute_key(rbuf, bn, dh);
+    i = DH_compute_key((unsigned char *)rbuf, bn, dh);
     
     /* set the key */
-    CAST_set_key(&castkey, i, rbuf);
+    CAST_set_key(&castkey, i, (unsigned char *)rbuf);
     
     /* session id. it's just a hashed version of the object pointer. */
     sessid = dhxhash(obj);
@@ -267,7 +276,7 @@ static int dhx_setup(void *obj, char *ibuf, int ibuflen _U_,
     *rbuflen += sizeof(sessid);
     
     /* public key */
-    BN_bn2bin(dh->pub_key, rbuf); 
+    BN_bn2bin(pub_key, (unsigned char *)rbuf); 
     rbuf += KEYSIZE;
     *rbuflen += KEYSIZE;
 
@@ -319,13 +328,13 @@ pam_fail:
 }
 
 /* -------------------------------- */
-static int login(void *obj, char *username, int ulen,  struct passwd **uam_pwd _U_,
-		     char *ibuf, int ibuflen,
-		     char *rbuf, int *rbuflen)
+static int login(void *obj, unsigned char *username, int ulen,
+	struct passwd **uam_pwd _U_, unsigned char *ibuf, size_t ibuflen,
+	unsigned char *rbuf, size_t *rbuflen)
 {
-    if (( dhxpwd = uam_getname(obj, username, ulen)) == NULL ) {
-        LOG(log_info, logtype_uams, "uams_dhx_pam.c: unknown username");
-	return AFPERR_PARAM;
+    if (( dhxpwd = uam_getname(obj, (char *)username, ulen)) == NULL ) {
+        LOG(log_info, logtype_uams, "uams_dhx_pam.c: unknown username [%s]", username);
+        return AFPERR_NOTAUTH;
     }
 
     PAM_username = username;
@@ -337,11 +346,11 @@ static int login(void *obj, char *username, int ulen,  struct passwd **uam_pwd _
 /* dhx login: things are done in a slightly bizarre order to avoid
  * having to clean things up if there's an error. */
 static int pam_login(void *obj, struct passwd **uam_pwd,
-		     char *ibuf, int ibuflen,
-		     char *rbuf, int *rbuflen)
+		     unsigned char *ibuf, size_t ibuflen,
+		     unsigned char *rbuf, size_t *rbuflen)
 {
-    char *username;
-    int len, ulen;
+    unsigned char *username;
+    size_t len, ulen;
 
     *rbuflen = 0;
 
@@ -371,11 +380,11 @@ static int pam_login(void *obj, struct passwd **uam_pwd,
 
 /* ----------------------------- */
 static int pam_login_ext(void *obj, char *uname, struct passwd **uam_pwd,
-		     char *ibuf, int ibuflen,
-		     char *rbuf, int *rbuflen)
+		     unsigned char *ibuf, size_t ibuflen,
+		     unsigned char *rbuf, size_t *rbuflen)
 {
-    char *username;
-    int len, ulen;
+    unsigned char *username;
+    size_t len, ulen;
     u_int16_t  temp16;
 
     *rbuflen = 0;
@@ -407,10 +416,10 @@ static int pam_login_ext(void *obj, char *uname, struct passwd **uam_pwd,
 /* -------------------------------- */
 
 static int pam_logincont(void *obj, struct passwd **uam_pwd,
-			 char *ibuf, int ibuflen _U_, 
-			 char *rbuf, int *rbuflen)
+			 unsigned char *ibuf, size_t ibuflen _U_, 
+			 unsigned char *rbuf, size_t *rbuflen)
 {
-    char *hostname;
+    const char *hostname;
     BIGNUM *bn1, *bn2, *bn3;
     u_int16_t sessid;
     int err, PAM_error;
@@ -476,8 +485,8 @@ static int pam_logincont(void *obj, struct passwd **uam_pwd,
     PAM_password = rbuf;
 
     err = AFPERR_NOTAUTH;
-    PAM_error = pam_start("netatalk", PAM_username, &PAM_conversation,
-			  &pamh);
+    PAM_error = pam_start("netatalk", (const char *)PAM_username,
+			&PAM_conversation, &pamh);
     if (PAM_error != PAM_SUCCESS) {
     /* Log Entry */
            LOG(log_info, logtype_uams, "uams_dhx_pam.c :PAM: PAM_Error: %s",
@@ -554,7 +563,7 @@ logincont_err:
 }
 
 /* logout */
-static void pam_logout() {
+static void pam_logout(void) {
     pam_close_session(pamh, 0);
     pam_end(pamh, 0);
     pamh = NULL;
@@ -563,9 +572,9 @@ static void pam_logout() {
 
 /* change pw for dhx needs a couple passes to get everything all
  * right. basically, it's like the login/logincont sequence */
-static int pam_changepw(void *obj, char *username,
-			struct passwd *pwd _U_, char *ibuf, int ibuflen,
-			char *rbuf, int *rbuflen)
+static int pam_changepw(void *obj, unsigned char *username,
+			struct passwd *pwd _U_, unsigned char *ibuf,
+			size_t ibuflen, unsigned char *rbuf, size_t *rbuflen)
 {
     BIGNUM *bn1, *bn2, *bn3;
 
@@ -575,6 +584,10 @@ static int pam_changepw(void *obj, char *username,
     u_int16_t sessid;
     int PAM_error;
 
+    if (ibuflen < sizeof(sessid)) {
+      return AFPERR_PARAM;
+    }
+
     /* grab the id */
     memcpy(&sessid, ibuf, sizeof(sessid));
     ibuf += sizeof(sessid);
@@ -668,7 +681,7 @@ static int pam_changepw(void *obj, char *username,
     ibuf[PASSWDLEN + PASSWDLEN] = '\0';
     PAM_password = ibuf + PASSWDLEN;
 
-    PAM_error = pam_start("netatalk", username, &PAM_conversation,
+    PAM_error = pam_start("netatalk", (char *)username, &PAM_conversation,
 			  &lpamh);
     if (PAM_error != PAM_SUCCESS) {
     /* Log Entry */
