@@ -1,6 +1,4 @@
 /*
- * $Id: uams_gss.c,v 1.12 2010-03-30 10:25:49 franklahm Exp $
- *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * Copyright (c) 1999 Adrian Sun (asun@u.washington.edu)
  * Copyright (c) 2003 The Reed Institute
@@ -14,24 +12,8 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#ifdef HAVE_UNISTD_H
 #include <unistd.h>
-#endif /* HAVE_UNISTD_H */
-
-/* STDC check */
-#if STDC_HEADERS
 #include <string.h>
-#else /* STDC_HEADERS */
-#ifndef HAVE_STRCHR
-#define strchr index
-#define strrchr index
-#endif /* HAVE_STRCHR */
-char *strchr (), *strrchr ();
-#ifndef HAVE_MEMCPY
-#define memcpy(d,s,n) bcopy ((s), (d), (n))
-#define memmove(d,s,n) bcopy ((s), (d), (n))
-#endif /* ! HAVE_MEMCPY */
-#endif /* STDC_HEADERS */
 
 #include <errno.h>
 #include <atalk/logger.h>
@@ -64,8 +46,10 @@ char *strchr (), *strrchr ();
 /* We work around something I don't entirely understand... */
 /* BF: This is a Heimdal/MIT compatibility fix */
 #ifndef HAVE_GSS_C_NT_HOSTBASED_SERVICE
+#ifndef GSS_C_NT_HOSTBASED_SERVICE
 #define GSS_C_NT_HOSTBASED_SERVICE gss_nt_service_name
 #endif
+#endif
 
 #ifdef MIN
 #undef MIN
@@ -77,13 +61,10 @@ static void log_status( char *s, OM_uint32 major_status,
                         OM_uint32 minor_status )
 {
     gss_buffer_desc msg = GSS_C_EMPTY_BUFFER;
-    OM_uint32 min_status, maj_status;
+    OM_uint32 min_status;
     OM_uint32 maj_ctx = 0, min_ctx = 0;
 
     while (1) {
-        maj_status = gss_display_status( &min_status, major_status,
-                                         GSS_C_GSS_CODE, GSS_C_NULL_OID,
-                                         &maj_ctx, &msg );
         LOG(log_info, logtype_uams, "uams_gss.c :do_gss_auth: %s %.*s (error %s)", s,
             (int)msg.length, msg.value, strerror(errno));
         gss_release_buffer(&min_status, &msg);
@@ -92,9 +73,6 @@ static void log_status( char *s, OM_uint32 major_status,
             break;
     }
     while (1) {
-        maj_status = gss_display_status( &min_status, minor_status,
-                                         GSS_C_MECH_CODE, GSS_C_NULL_OID, // gss_mech_krb5,
-                                         &min_ctx, &msg );
         LOG(log_info, logtype_uams, "uams_gss.c :do_gss_auth: %s %.*s (error %s)", s,
             (int)msg.length, msg.value, strerror(errno));
         gss_release_buffer(&min_status, &msg);
@@ -107,7 +85,6 @@ static void log_status( char *s, OM_uint32 major_status,
 
 static void log_ctx_flags( OM_uint32 flags )
 {
-#ifdef DEBUG1
     if (flags & GSS_C_DELEG_FLAG)
         LOG(log_debug, logtype_uams, "uams_gss.c :context flag: GSS_C_DELEG_FLAG" );
     if (flags & GSS_C_MUTUAL_FLAG)
@@ -120,7 +97,6 @@ static void log_ctx_flags( OM_uint32 flags )
         LOG(log_debug, logtype_uams, "uams_gss.c :context flag: GSS_C_CONF_FLAG" );
     if (flags & GSS_C_INTEG_FLAG)
         LOG(log_debug, logtype_uams, "uams_gss.c :context flag: GSS_C_INTEG_FLAG" );
-#endif
 }
 
 static void log_principal(gss_name_t server_name)
@@ -303,7 +279,7 @@ static int wrap_sessionkey(gss_ctx_id_t context, struct session_info *sinfo)
     /* store the wrapped session key in afpd's session_info struct */
     if ( NULL == (sinfo->cryptedkey = malloc ( wrap_buff.length )) ) {
         LOG(log_error, logtype_uams,
-            "wrap_sessionkey: out of memory tyring to allocate %u bytes",
+            "wrap_sessionkey: out of memory trying to allocate %u bytes",
             wrap_buff.length);
         ret = 1;
     } else {
