@@ -1,6 +1,4 @@
 /*
- * $Id: atp_sresp.c,v 1.6 2009-10-13 22:55:37 didg Exp $
- *
  * Copyright (c) 1990,1991 Regents of The University of Michigan.
  * All Rights Reserved.
  *
@@ -52,8 +50,8 @@ int atp_sresp(
 {
     int			i;
     u_int8_t		ctrlinfo;
-    struct atpbuf	*resp_buf;
-    struct atpbuf	*save_buf; /* uninitialized, OK 310105 */
+    struct atpbuf	*resp_buf = NULL;
+    struct atpbuf	*save_buf = NULL;
 
 #ifdef EBUG
     atp_print_bufuse( ah, "atp_sresp" );
