@@ -1,16 +1,9 @@
-/*
- * $Id: inet_aton.c,v 1.4 2003-02-17 01:51:08 srittau Exp $
- */
-
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif /* HAVE_CONFIG_H */
-
 #include <sys/types.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
-#if defined(ultrix) || (defined(sun) && defined(__svr4__))
+#if defined(sun) && defined(__svr4__)
 #ifndef INADDR_NONE
 #define INADDR_NONE ((unsigned) 0xffffffff)
 #endif /* ! INADDR_NONE */
@@ -22,4 +15,4 @@ int inet_aton(const char *name, struct in_addr *addr)
 
   return 1;
 }
-#endif /* ultrix || ( sun && __svr4__ ) */
+#endif /* sun && __svr4__ */
