@@ -1,6 +1,4 @@
 /*
- * $Id: dsi_attn.c,v 1.5.6.1 2003/10/17 00:01:14 didg Exp $
- *
  * Copyright (c) 1997 Adrian Sun (asun@zoology.washington.edu)
  * All rights reserved. See COPYRIGHT.
  */
@@ -24,20 +22,21 @@
 
 /* send an attention. this may get called at any time, so we can't use
  * DSI buffers to send one. 
-   return 0 on error
- 
+ * return 0 on error
  */
 int dsi_attention(DSI *dsi, AFPUserBytes flags)
 {
   /* header + AFPUserBytes */
   char block[DSI_BLOCKSIZ + sizeof(AFPUserBytes)];
-  sigset_t oldset;
   u_int32_t len, nlen;
   u_int16_t id;
 
-  if (dsi->asleep)
+  if (dsi->flags & DSI_SLEEPING)
       return 1;
-      
+
+  if (dsi->in_write) {
+      return -1;
+  }      
   id = htons(dsi_serverID(dsi));
   flags = htons(flags);
   len = MIN(sizeof(flags), dsi->attn_quantum);
@@ -53,9 +52,5 @@ int dsi_attention(DSI *dsi, AFPUserBytes flags)
   /* reserved = 0 */
 
   /* send an attention */
-  sigprocmask(SIG_BLOCK, &dsi->sigblockset, &oldset);
-  len = dsi_stream_write(dsi, block, DSI_BLOCKSIZ + len, 0);
-  sigprocmask(SIG_SETMASK, &oldset, NULL);
-
-  return len;
+  return dsi_stream_write(dsi, block, DSI_BLOCKSIZ + len, DSI_NOWAIT);
 }
