@@ -1,6 +1,4 @@
 /*
- * $Id: dsi_init.c,v 1.3.14.1.2.1 2005/09/27 10:40:41 didg Exp $
- *
  * Copyright (c) 1997 Adrian Sun (asun@zoology.washington.edu)
  * All rights reserved. See COPYRIGHT.
  */
@@ -17,7 +15,7 @@
 
 DSI *dsi_init(const dsi_proto protocol, const char *program, 
 	      const char *hostname, const char *address,
-	      const int port, const int proxy, const u_int32_t quantum)
+	      const char *port, const int proxy, const u_int32_t quantum)
 {
     DSI		*dsi;
 
@@ -28,15 +26,6 @@ DSI *dsi_init(const dsi_proto protocol, const char *program,
     dsi->server_quantum = quantum; /* default server quantum */
     dsi->program = program;
 
-    /* signals to block. we actually disable timers for "known" 
-     * large transfers (i.e., dsi_read/write). */
-    sigemptyset(&dsi->sigblockset);
-    sigaddset(&dsi->sigblockset, SIGTERM);
-    sigaddset(&dsi->sigblockset, SIGHUP);
-    sigaddset(&dsi->sigblockset, SIGALRM);
-    sigaddset(&dsi->sigblockset, SIGUSR1);
-    /* always block SIGUSR2 even if SERVERTEXT is not defined */
-    sigaddset(&dsi->sigblockset, SIGUSR2);
     switch (protocol) {
       /* currently the only transport protocol that exists for dsi */
     case DSI_TCPIP: 
