@@ -1,6 +1,4 @@
 /*
- * $Id: dsi_private.h,v 1.4 2009-11-05 14:38:08 franklahm Exp $
- *
  * Copyright (c) 1997 Adrian Sun (asun@zoology.washington.edu)
  * All rights reserved. See COPYRIGHT.
  */
@@ -13,7 +11,7 @@
  * include it.
  */
 
-#include <sys/cdefs.h>
+#include <sys/types.h>
 #include <sys/types.h>
 #include <netatalk/endian.h>
 
