@@ -1,6 +1,4 @@
 /*
- * $Id: nbp_conf.h,v 1.2 2001/06/29 14:14:46 rufustfirefly Exp $
- *
  * Copyright (c) 1990,1991 Regents of The University of Michigan.
  * All Rights Reserved.
  *
@@ -26,7 +24,7 @@
 #ifndef NBP_CONF_H
 #define NBP_CONF_H 1
 
-#include <sys/cdefs.h>
+#include <sys/types.h>
 #include <atalk/nbp.h>
 
 extern char		nbp_send[ 1024 ];
@@ -35,7 +33,7 @@ extern u_char		nbp_port;
 extern unsigned char    nbp_id;
 
 
-int nbp_parse __P((char *, struct nbpnve *, int));
-int nbp_match __P((struct nbpnve *, struct nbpnve *, int));
+int nbp_parse (char *, struct nbpnve *, int);
+int nbp_match (struct nbpnve *, struct nbpnve *, int);
 
 #endif /* NBP_CONF_H */
