@@ -122,26 +122,30 @@ static int unpack_header(struct ea * restrict ea)
 {
     int ret = 0;
     unsigned int count = 0;
+    uint16_t uint16;
     uint32_t uint32;
     char *buf;
 
     /* Check magic and version */
     buf = ea->ea_data;
-    if (*(uint32_t *)buf != htonl(EA_MAGIC)) {
-        LOG(log_error, logtype_afpd, "unpack_header: wrong magic 0x%08x", *(uint32_t *)buf);
+    memcpy(&uint32, buf, sizeof(uint32_t));
+    if (uint32 != htonl(EA_MAGIC)) {
+        LOG(log_error, logtype_afpd, "unpack_header: wrong magic 0x%08x", uint32);
         ret = -1;
         goto exit;
     }
     buf += 4;
-    if (*(uint16_t *)buf != htons(EA_VERSION)) {
-        LOG(log_error, logtype_afpd, "unpack_header: wrong version 0x%04x", *(uint16_t *)buf);
+    memcpy(&uint16, buf, sizeof(uint16_t));
+    if (uint16 != htons(EA_VERSION)) {
+        LOG(log_error, logtype_afpd, "unpack_header: wrong version 0x%04x", uint16);
         ret = -1;
         goto exit;
     }
     buf += 2;
 
     /* Get EA count */
-    ea->ea_count = ntohs(*(uint16_t *)buf);
+    memcpy(&uint16, buf, sizeof(uint16_t));
+    ea->ea_count = ntohs(uint16);
     LOG(log_debug, logtype_afpd, "unpack_header: number of EAs: %u", ea->ea_count);
     buf += 2;
 
@@ -389,6 +393,8 @@ static int create_ea_header(const char * restrict uname,
 {
     int fd = -1, err = 0;
     char *ptr;
+    uint16_t uint16;
+    uint32_t uint32;
 
     if ((fd = open(uname, O_RDWR | O_CREAT | O_EXCL, 0666 & ~ea->vol->v_umask)) == -1) {
         LOG(log_error, logtype_afpd, "ea_create: open race condition with ea header for file: %s", uname);
@@ -404,11 +410,15 @@ static int create_ea_header(const char * restrict uname,
 
     /* Now init it */
     ptr = ea->ea_data;
-    *(uint32_t *)ptr = htonl(EA_MAGIC);
+    uint32 = htonl(EA_MAGIC);
+    memcpy(ptr, &uint32, sizeof(uint32_t));
     ptr += EA_MAGIC_LEN;
-    *(uint16_t *)ptr = htons(EA_VERSION);
+
+    uint16 = htons(EA_VERSION);
+    memcpy(ptr, &uint16, sizeof(uint16_t));
     ptr += EA_VERSION_LEN;
-    *(uint16_t *)ptr = 0;       /* count */
+
+    memset(ptr, 0, 2);          /* count */
 
     ea->ea_size = EA_HEADER_SIZE;
     ea->ea_inited = EA_INITED;
@@ -841,7 +851,7 @@ int ea_openat(const struct vol * restrict vol,
 
     if (dirfd != -1) {
         if (fchdir(cwdfd) != 0) {
-            LOG(log_error, logtype_afpd, "ea_openat: cant chdir back, exiting");
+            LOG(log_error, logtype_afpd, "ea_openat: can't chdir back, exiting");
             exit(EXITERR_SYS);
         }
     }
@@ -894,7 +904,7 @@ int ea_close(struct ea * restrict ea)
             if (ea->ea_count == 0) {
                 /* Check if EA header exists and remove it */
                 eaname = ea_path(ea, NULL, 0);
-                if ((lstatat(ea->dirfd, eaname, &st)) == 0) {
+                if ((statat(ea->dirfd, eaname, &st)) == 0) {
                     if ((netatalk_unlinkat(ea->dirfd, eaname)) != 0) {
                         LOG(log_error, logtype_afpd, "ea_close('%s'): unlink: %s",
                             eaname, strerror(errno));
@@ -961,7 +971,7 @@ exit:
         ea->ea_fd = -1;
     }
 
-    return 0;
+    return ret;
 }
 
 
@@ -984,7 +994,7 @@ exit:
  *    oflag        (r) link and create flag
  *    attruname    (r) name of attribute
  *
- * Returns: AFP code: AFP_OK on success or appropiate AFP error code
+ * Returns: AFP code: AFP_OK on success or appropriate AFP error code
  *
  * Effects:
  *
@@ -1045,7 +1055,7 @@ int get_easize(VFS_FUNC_ARGS_EA_GETSIZE)
  *    attruname    (r) name of attribute
  *    maxreply     (r) maximum EA size as of current specs/real-life
  *
- * Returns: AFP code: AFP_OK on success or appropiate AFP error code
+ * Returns: AFP code: AFP_OK on success or appropriate AFP error code
  *
  * Effects:
  *
@@ -1133,7 +1143,7 @@ int get_eacontent(VFS_FUNC_ARGS_EA_GETCONTENT)
  *    uname        (r) filename
  *    oflag        (r) link and create flag
  *
- * Returns: AFP code: AFP_OK on success or appropiate AFP error code
+ * Returns: AFP code: AFP_OK on success or appropriate AFP error code
  *
  * Effects:
  *
@@ -1214,7 +1224,7 @@ exit:
  *    attrsize     (r) length EA in ibuf
  *    oflag        (r) link and create flag
  *
- * Returns: AFP code: AFP_OK on success or appropiate AFP error code
+ * Returns: AFP code: AFP_OK on success or appropriate AFP error code
  *
  * Effects:
  *
@@ -1267,7 +1277,7 @@ exit:
  *    attruname    (r) EA name
  *    oflag        (r) link and create flag
  *
- * Returns: AFP code: AFP_OK on success or appropiate AFP error code
+ * Returns: AFP code: AFP_OK on success or appropriate AFP error code
  *
  * Effects:
  *
@@ -1355,7 +1365,7 @@ int ea_deletefile(VFS_FUNC_ARGS_DELETEFILE)
     }
 
     if (dirfd != -1 && fchdir(cwd) != 0) {
-        LOG(log_error, logtype_afpd, "ea_deletefile: cant chdir back. exit!");
+        LOG(log_error, logtype_afpd, "ea_deletefile: can't chdir back. exit!");
         exit(EXITERR_SYS);
     }
 
@@ -1571,7 +1581,7 @@ int ea_chown(VFS_FUNC_ARGS_CHOWN)
         }
     }
 
-    if ((lchown(ea_path(&ea, NULL, 0), uid, gid)) != 0) {
+    if ((ochown(ea_path(&ea, NULL, 0), uid, gid, vol_syml_opt(vol))) != 0) {
         switch (errno) {
         case EPERM:
         case EACCES:
@@ -1588,7 +1598,7 @@ int ea_chown(VFS_FUNC_ARGS_CHOWN)
             ret = AFPERR_MISC;
             goto exit;
         }
-        if ((lchown(eaname, uid, gid)) != 0) {
+        if ((ochown(eaname, uid, gid, vol_syml_opt(vol))) != 0) {
             switch (errno) {
             case EPERM:
             case EACCES:
@@ -1632,7 +1642,7 @@ int ea_chmod_file(VFS_FUNC_ARGS_SETFILEMODE)
     }
 
     /* Set mode on EA header file */
-    if ((setfilmode(ea_path(&ea, NULL, 0), ea_header_mode(mode), NULL, vol->v_umask)) != 0) {
+    if ((setfilmode(vol, ea_path(&ea, NULL, 0), ea_header_mode(mode), NULL)) != 0) {
         LOG(log_error, logtype_afpd, "ea_chmod_file('%s'): %s", ea_path(&ea, NULL, 0), strerror(errno));
         switch (errno) {
         case EPERM:
@@ -1651,7 +1661,7 @@ int ea_chmod_file(VFS_FUNC_ARGS_SETFILEMODE)
             ret = AFPERR_MISC;
             goto exit;
         }
-        if ((setfilmode(eaname, ea_mode(mode), NULL, vol->v_umask)) != 0) {
+        if ((setfilmode(vol, eaname, ea_mode(mode), NULL)) != 0) {
             LOG(log_error, logtype_afpd, "ea_chmod_file('%s'): %s", eaname, strerror(errno));
             switch (errno) {
             case EPERM:
@@ -1708,7 +1718,7 @@ int ea_chmod_dir(VFS_FUNC_ARGS_SETDIRUNIXMODE)
     }
 
     /* Set mode on EA header */
-    if ((setfilmode(ea_path(&ea, NULL, 0), ea_header_mode(mode), NULL, vol->v_umask)) != 0) {
+    if ((setfilmode(vol, ea_path(&ea, NULL, 0), ea_header_mode(mode), NULL)) != 0) {
         LOG(log_error, logtype_afpd, "ea_chmod_dir('%s'): %s", ea_path(&ea, NULL, 0), strerror(errno));
         switch (errno) {
         case EPERM:
@@ -1738,7 +1748,7 @@ int ea_chmod_dir(VFS_FUNC_ARGS_SETDIRUNIXMODE)
             ret = AFPERR_MISC;
             goto exit;
         }
-        if ((setfilmode(eaname, ea_mode(mode), NULL, vol->v_umask)) != 0) {
+        if ((setfilmode(vol, eaname, ea_mode(mode), NULL)) != 0) {
             LOG(log_error, logtype_afpd, "ea_chmod_dir('%s'): %s", eaname, strerror(errno));
             switch (errno) {
             case EPERM:
